// 
//  Copyright (C) 2007,2010-2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PEGAPP_HH
#define PEGAPP_HH

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <iostream>
#include <string>

class PegStatusArea;
class PegCmdButtons; 
class PegMenu;
class PegPrefs;
class PegPrefsDialog;
class CiaoGuiIPCMessage; 
class CiaoGuiIPCServer; 

using namespace std;

extern bool _check_is_directory(string path);

class PegApp
{
//    friend class PegMenuWindow;

  public:

    PegApp();
    ~PegApp();

    static int exit_cb();

    void exit_application(); 
    int  GetChanges()  { return _changeCnt; }
    void SetChanges()  { _changeCnt++; }

    int SetFilename(string& filename, int argc, const char **argv, int status=1);
    int SetFilename(string& filename, int status=1);
    int SetFilename(char* fname, int argc, const char **argv, int status=1);
    int SetFilename(char* fname, int status=1);
    string GetFilename()  {return _fname; }
    string get_file_path(); 
    string GetXpaName()   { return _xpaname; }

    int update_filelist(string fname); 

    int save_changes(); 
    int save_file(string fname=""); 

    static void set_status_message(string val);

    int status_message(string msg, int type=0);
    void initialize(string toolmenu="", string xpaname=""); 

    GtkTooltips* get_tips_group()  { return _tips; }
    GtkTooltips* get_prompts_group()  { return _prompts; }

    void set_window_title(string val); 

    GtkWidget*  get_scrolled()     { return _scrolled; }
    PegCmdButtons* getCmdButtons() { return _buttons;  }

    int get_preference(string pref, bool &val);
    int get_preference(string pref, string &val);
    void launch_preferences_dialog();

    void enable_help_on_tool(bool val); 
    void enable_help_on_params(bool val); 

    void cleanup(); 
  protected: 
    void reset_tips(); 

    PegStatusArea* _statusArea;
    PegCmdButtons* _buttons; 

    static CiaoGuiIPCMessage* FileMessageCB(CiaoGuiIPCMessage&);
    static CiaoGuiIPCMessage* SetMessageCB(CiaoGuiIPCMessage&);
    static CiaoGuiIPCMessage* GetMessageCB(CiaoGuiIPCMessage&);
    static CiaoGuiIPCMessage* OpenMessageCB(CiaoGuiIPCMessage&);
    static CiaoGuiIPCMessage* PunlearnMessageCB(CiaoGuiIPCMessage&);
    static CiaoGuiIPCMessage* RunMessageCB(CiaoGuiIPCMessage&);
    static CiaoGuiIPCMessage* SaveMessageCB(CiaoGuiIPCMessage&);

    void   add_ipc();
    CiaoGuiIPCServer* ipc;


  private:
    GtkWidget* _window; 
    GtkWidget* _vbox;        // holds file menu and vpane
    GtkWidget* _vpane;       // holds vpane2 and status msg area
    GtkWidget* _vpane2;      // holds scrollwindow and cmd buttons 
    GtkWidget* _scrolled;    // holds parameter widgets 
    GtkTooltips *_tips;      // tool tips group
    GtkTooltips *_prompts;   // prompt tips group

    PegMenu*   _menu;
    PegPrefs*  _prefs;       // user preferences
    PegPrefsDialog *_prefsDialog;

    string     _xpaname;     // xpa access name 
    string     _fname;
    int        _changeCnt;
    int        _height;      // initial height 
    int        _width;       // initial width  
    int        _init;        // has peg been initialized 
    bool       _autopath;    // did cxcparam lib autoset path?
    bool       _showtips;    // display tooltips 
    bool       _showprompts; // display param prompts as tips 
};

extern PegApp *theApp;

#endif 
