// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <fstream>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include "PegApp.hh"
#include "PegFileMenu.hh"
#include "PegPrintFileDialog.hh"
#include "PegSaveAsDialog.hh"

#define PEG_FLIST_BUF_SZ   256

using namespace std;


PegFileMenu::PegFileMenu()
{
   //   menuFile = gtk_menu_item_new_with_mnemonic(("_File"));
   //   gtk_menu_shell_insert(GTK_MENU_SHELL(menubar), menuFile, 0);
   int pos = 0; 
   base_w = gtk_menu_item_new_with_mnemonic("_File");

   menu = gtk_menu_new();

   menuItemOpen = gtk_menu_item_new_with_mnemonic(("_Open"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemOpen, pos++);

   menuItemRecent = gtk_menu_item_new_with_mnemonic(("Open _List"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemRecent, pos++);
   // setup file list and pull down 
   _filelist = getenv("HOME");
   if (_filelist.length() == 0) _filelist.append(".");
   _filelist.append("/");
   _filelist.append(".peg.filelist");
   _submenuRecent = NULL; 
   load_recent_submenu(); 
   menuItemSave = gtk_menu_item_new_with_mnemonic(("_Save"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemSave, pos++);

   menuItemSaveAs = gtk_menu_item_new_with_mnemonic(("Save _As"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemSaveAs, pos++);

   menuItemPrint = gtk_menu_item_new_with_mnemonic(("_Print"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemPrint, pos++);

   sep = gtk_separator_menu_item_new();
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), sep, pos++);

   menuItemQuit = gtk_menu_item_new_with_mnemonic(("_Quit"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemQuit, pos++);

   /* We can attach the Quit menu item to our exit function */
   g_signal_connect(G_OBJECT(menuItemOpen), "activate",
                    G_CALLBACK(open_cb), (gpointer) this);
   g_signal_connect(G_OBJECT(menuItemSave), "activate",
                    G_CALLBACK(save_cb), (gpointer) this);
   g_signal_connect(G_OBJECT(menuItemSaveAs), "activate",
                    G_CALLBACK(saveas_cb), (gpointer) this);
   g_signal_connect_swapped(G_OBJECT(menuItemQuit), "activate",
                    G_CALLBACK (quit_cb), (gpointer) NULL);
   g_signal_connect_swapped(G_OBJECT(menuItemPrint), "activate",
                    G_CALLBACK (print_cb), (gpointer) NULL);
   g_signal_connect_swapped(G_OBJECT(menuItemRecent), "activate",
                    G_CALLBACK (recent_cb), (gpointer) NULL);

   GtkTooltips *tips = theApp->get_tips_group();
   if (tips)
   {
      gtk_tooltips_set_tip(tips, menuItemOpen,
         "Open parameter file via file selection dialog",NULL);
      gtk_tooltips_set_tip(tips, menuItemRecent,
         "Open file from list of recently accessed files",NULL);
      gtk_tooltips_set_tip(tips, menuItemSave,
         "Save changes to the current parameter file",NULL);
      gtk_tooltips_set_tip(tips, menuItemSaveAs,
         "Save changes to a different file",NULL);
      gtk_tooltips_set_tip(tips, menuItemPrint,
         "Print the current parameter file",NULL);
      gtk_tooltips_set_tip(tips, menuItemQuit,
         "Exit peg",NULL);
   }

   gtk_menu_item_set_submenu (GTK_MENU_ITEM(base_w), menu);
   _prev_dir = "";
}

PegFileMenu::~PegFileMenu()
{
   
}

int PegFileMenu::show()
{
   gtk_widget_show(menuItemOpen);
   gtk_widget_show(menuItemSave);
   gtk_widget_show(menuItemRecent);
   gtk_widget_show(menuItemQuit);
   gtk_widget_show(base_w); 
   return 0; 
}

int PegFileMenu::open_cb(GObject *object, gpointer user_data)
{
   PegFileMenu *fm = (PegFileMenu*)user_data;
   GtkWidget* dialog = gtk_file_chooser_dialog_new ("Open File",
                                         GTK_WINDOW(NULL),
                                         GTK_FILE_CHOOSER_ACTION_OPEN,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                         GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                         NULL);
   // if file modified, query to save changes before proceeding
   if (theApp)
      if (theApp->save_changes() != 0) return 0; 

   if (fm)
   {
     const char* predir = fm->get_prev_dir();
      if (predir && strlen(predir))
         gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), predir);
   }
  // add basic filter support
   GtkFileFilter *filt = gtk_file_filter_new ();
   gtk_file_filter_add_pattern (filt, "*");
   gtk_file_filter_set_name(filt,"All Files");
   gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);
   filt = gtk_file_filter_new ();
   gtk_file_filter_add_pattern (filt, "*.par");
   gtk_file_filter_set_name(filt,"Parameter Files (*.par)");
   gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);

   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
   {
      gchar * filename = (char*)gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
      if (filename)
      {
	 string fn = filename; 
	 // check if the file exists- otherwise strip off the path
         if (access(filename, R_OK) != 0)
         {
	    long pos = fn.find_last_of("/");
	    fn = fn.substr(++pos); 
	 }

         theApp->SetFilename(fn);
         if (fm)
         {
            string newpath = fn;
            unsigned int delim = newpath.find_last_of('/');

            if (_check_is_directory(newpath))
            {
               if (delim + 1 != newpath.length()) newpath.append("/");
            }
            else if (delim != string::npos)
               newpath = newpath.substr(0, delim+1);
            fm->set_prev_dir(newpath);
         }
         g_free (filename);
      }
   }
   gtk_widget_destroy (dialog);
   return 0;
}


int PegFileMenu::save_cb(GObject *object, gpointer user_data)
{
   if (theApp)  return theApp->save_file(); 
   return 1; 
}


int PegFileMenu::saveas_cb(GObject *object, gpointer user_data)
{
   PegFileMenu *fm = (PegFileMenu*)user_data;
   int rval = 0; 
   string dpath = ""; 
   if (fm) dpath = fm->get_prev_dir();
   PegSaveAsDialog* dlg = new PegSaveAsDialog(dpath);
   if (dlg) 
   {
      rval = dlg->doit(); 
      fm->set_prev_dir(dlg->get_default_path());
      delete dlg; 
   }
   return rval;
}

int PegFileMenu::print_cb()
{
   PegPrintFileDialog* dlg = new PegPrintFileDialog();
   if (dlg)  dlg->doit(); 
   return 0;
}

int PegFileMenu::quit_cb()
{
   if (theApp)   theApp->exit_application();
   return 0;
}

int PegFileMenu::recent_cb()
{
   return 0; 
}

int PegFileMenu::filelist_cb(gchar* buf)
{
   // if file modified, query to save changes before proceeding
   if (theApp)
      if (theApp->save_changes() != 0) return 0; 

   // for now open in current tab
   if (buf) theApp->SetFilename(buf);  
   return 0; 
}

void PegFileMenu::load_recent_submenu()
{
   if (_submenuRecent)
   {
      gtk_menu_item_remove_submenu(GTK_MENU_ITEM(menuItemRecent)); 
/***
Is this necessary?
      gtk_widget_destroy(_submenuRecent);
      for (int rr = 0; rr < PEG_FLIST_SZ; rr++)
         if (_filelistItem[rr])   gtk_widget_destroy(_filelistItem[rr]); 
****/
   }
   _submenuRecent = gtk_menu_new();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuItemRecent), _submenuRecent); 

   ifstream in(_filelist.c_str());
   if (in)
   {
      int cnt = 0;
      char buf[PEG_FLIST_BUF_SZ];
      string fn; 
      while(in.getline(buf, PEG_FLIST_BUF_SZ)  && (cnt < PEG_FLIST_SZ))
      {
         _filelistItem[cnt] = gtk_menu_item_new_with_label(buf);
         g_signal_connect_swapped(G_OBJECT(_filelistItem[cnt]), "activate",
                                  G_CALLBACK(filelist_cb),
                                  (gpointer)g_strdup(buf));
         gtk_menu_shell_insert(GTK_MENU_SHELL(_submenuRecent), 
	    _filelistItem[cnt], cnt);
         cnt++; 
      }
      in.close();
   }
   if (_submenuRecent)  gtk_widget_show_all(_submenuRecent);

   GtkTooltips *tips = theApp->get_tips_group();
   if (tips)
      gtk_tooltips_set_tip(tips, _submenuRecent,
         "Open file from list of recently accessed files",NULL);
}

// add new file to filelist if not already there
void PegFileMenu::update_filelist(string file)
{
   if (file.length())
   {
      ifstream in(_filelist.c_str());
      int rr, cnt = 0, found = -1;
      char buf[PEG_FLIST_SZ+1][PEG_FLIST_BUF_SZ];

      if (in)   // if file DNE then don't bother
      {
         while(in.getline(buf[cnt], PEG_FLIST_BUF_SZ) && (cnt < PEG_FLIST_SZ))
         {
            if (strcmp(buf[cnt], file.c_str()) == 0) found = cnt;
            cnt++;
         }
         in.close();
      }
      if (found == -1)  // entry not in file- update file 
      {
         ofstream out(_filelist.c_str());
         if (out)
         {
            out << file << endl;
            for (rr = 0; rr < cnt; rr++)
               if ((rr != found) && (rr < PEG_FLIST_SZ))
                  out << buf[rr] << endl;
            out.close();
         }
         else
         {
	    string warn_msg = "Unable to update open list " + _filelist + "with file " + file;
            theApp->status_message(warn_msg);
         }
      }
      load_recent_submenu();
   }
}



