// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PegApp.hh"
#include "PegMenu.hh"
#include "PegFileMenu.hh"
#include "PegEditMenu.hh"
#include "PegHelpMenu.hh"
#include "CiaoAnalysisMenu.hh"

using namespace std;

PegMenu::PegMenu(string filename, GtkTooltips* tips)
{
   base_w = gtk_menu_bar_new();
   menuFile = new PegFileMenu();
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuFile->base_widget());
   menuEdit = new PegEditMenu();
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuEdit->base_widget());

   menuAnalysis = new CiaoAnalysisMenu(filename, tips, NULL,
      theApp->set_status_message);
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuAnalysis->base_widget());

   menuHelp = new PegHelpMenu();
   gtk_menu_item_set_right_justified(GTK_MENU_ITEM(menuHelp->base_widget()),
      TRUE);
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuHelp->base_widget());
}

PegMenu::~PegMenu()
{
   if (menuFile)
   {
      delete menuFile;
      menuFile = NULL;
   }
   if (menuEdit)
   {
      delete menuEdit;
      menuEdit = NULL;
   }
   if (menuAnalysis)
   {
      delete menuAnalysis;
      menuAnalysis = NULL;
   }
   if (menuHelp)
   {
      delete menuHelp;
      menuHelp = NULL;
   }
}

int PegMenu::show()
{
   menuFile->show();
   menuEdit->show();
   menuHelp->show();
   menuAnalysis->show();
   gtk_widget_show(base_w);
   return 0; 
}

int PegMenu::update_filelist(string fname)
{
   if (!menuFile)  return 1; 
   menuFile->update_filelist(fname); 
   return 0; 
}

void PegMenu::enable_help_current_tool(bool val)
{
  if (menuHelp)
     if (val)
        menuHelp->enable_tool_help();
     else 
        menuHelp->disable_tool_help();
}
