// 
//  Copyright (C) 2002,2003,2007,2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PEGPARENTRY_HH
#define PEGPARENTRY_HH


#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <string>

#include "cxcconfig.h"

class ParamItem;
class PegFloatingMenu;

class PegParEntry {

  public:

   typedef enum ENTRY_TYPE {STRING,     // string/text type 
                            FILENAME,   // filename type
                            PSET,       
                            BOOLEAN,
                            INTEGER,
                            REAL,
                            NO_PARAM};  // undefined 
 
   typedef enum DISP_TYPE {DEFAULT,     // default widgets for data type
                           INDIR,       // for indirection, command-eval
                           ENUM};       // for enumerations 

    PegParEntry(ParamItem* item);

    virtual void initEntry();   // initialize values for entry 

    virtual void manage();   // Manage and unmanage widget tree
    virtual void unmanage();

    virtual void set_error_color()  {}   // set widget red to indicate err 
    virtual void clear_error_color() {}  // clear error code  

    void  launchPopup(GdkEventButton *event);

    void        reset_label();    // set the label to use pango markup 

    void     set(const char* val);
    char*    get();

    PegFloatingMenu* Popup()          {return _popup;}
    ParamItem* GetItem()               {return _item;}
    ENTRY_TYPE getEntryType()          {return _entry_type;}
    DISP_TYPE getOrigType()            {return _orig_type;}
    DISP_TYPE getDispType()            {return _disp_type;}
    DISP_TYPE getDefaultType()         {return _default;}
    void      toggleDispType(); 
    int       getPosition()            {return _pos;} 
    int       getSpacing()             {return _spacing;} 
    string   getName()                {return _name;} 
    GtkWidget*   GetTextField()        {return _txt_w;}
    GtkWidget*   GetIndirField()       {return _ind_txt;}
    GtkWidget*   GetIndirValField()    {return _ind_val;}
    GtkWidget*   getLabel()            {return _lbl_w; }
    GtkWidget*   getOptionMenu()       {return _enumer; }
    GtkWidget*  getEntry()             {return _entry;}
    GtkWidget*  getIndEntry()          {return _ind_entry;}

    GtkWidget*  getLabelBox()          {return _lbl_box; }
    GtkWidget*  getEntryBox()          {return _entry_box; }
    GtkWidget*  getIndBox()            {return _ind_box; }

    static int eventbox_cb(GObject *object,GdkEvent *event,gpointer user_data);
    static int  _setColors;      // have the label colors been set?  

  protected:
   virtual  ~PegParEntry();
   string set_markup(string clr, string fmt); // set pango markup for label

   ENTRY_TYPE _entry_type; 
   DISP_TYPE  _disp_type; 
   DISP_TYPE  _orig_type; 
   DISP_TYPE  _default; 
   string     _name;
   GtkWidget* _entry; 
   GtkWidget* _ind_entry; 
   GtkWidget* _lbl_w; 
   GtkWidget* _ind_lbl; 
   GtkWidget*  _txt_w;
   GtkWidget*  _ind_txt;
   GtkWidget*  _ind_val;
   GtkWidget*  _enumer; 

   GtkWidget* _lbl_box;
   GtkWidget* _entry_box;
   GtkWidget* _ind_box; 

   GtkTooltips* _tip_grp; 

   ParamItem* _item;
   int        _pos;      // position in parameter list 
   int        _spacing;  // offset spacing for widgets 

   static int ButtonPressCB(GObject *object, GdkEvent *event,
                            gpointer user_data);
   PegFloatingMenu* _popup; // popup menu object
   static GdkColor _errcolor;    // color structure for invalid backgrounds
   static string   _amarkup;     // pango markup for auto params
   static string   _hmarkup;     // pango markup for hidden params
};

#endif

