// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PegSpinAdjustDialog.hh"
#include "PegApp.hh"
#include <iostream>

using namespace std;

PegSpinAdjustDialog::PegSpinAdjustDialog(string name, GtkAdjustment* adj, 
   int precis)
{
   _name = name;
   _adj = adj; 
   _precis = precis; 
}

PegSpinAdjustDialog::~PegSpinAdjustDialog()
{
}

void PegSpinAdjustDialog::create_dialog()
{
   _dialog = gtk_dialog_new_with_buttons(_name.c_str(), NULL,
      GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_OK,
      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
   GtkWidget* tab = gtk_table_new(2,8, true);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(_dialog)->vbox), tab);
   g_object_get (_adj, "step-increment", &_adjustcurr,
                 "upper", &_adjustmax, NULL);
   _adjustmin = 0; 

   _adjustmax /= 10;
   if (_precis == 0)  _adjustmax += 0.5; 
   double step = 1.0; 
   if (_adjustmax < step)   step = _adjustmax/10; 
   double step2 = step*5; 
   if (_precis == 0 && step < 1)
   {
      step2 = step = 1;  
      _adjustmax = 1; 
   }
   if (step2 > _adjustmax) step2 = step;
   GtkAdjustment *adjust_adj = (GtkAdjustment *) 
      gtk_adjustment_new(_adjustcurr, _adjustmin, _adjustmax, step, step2, 0);
   _adjustspin_w   = gtk_spin_button_new(adjust_adj, step, _precis);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_adjustspin_w), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_adjustspin_w), GTK_UPDATE_IF_VALID);
   g_signal_connect(_adjustspin_w, "value-changed", G_CALLBACK(adjustspin_value_callback),
                    (void*)this);
   _adjustscale_w = gtk_hscale_new(adjust_adj);
   gtk_scale_set_digits(GTK_SCALE(_adjustscale_w), _precis); 
   gtk_scale_set_draw_value(GTK_SCALE(_adjustscale_w), 0);
   g_signal_connect(_adjustscale_w,"value-changed",G_CALLBACK(adjustscale_value_callback),
                    (void*)this);

   char buf[128];
   if (_precis)  sprintf(buf, "Max (%g)", _adjustmax); 
      else       sprintf(buf, "Max (%1ld)", (long)_adjustmax);
//      else       sprintf(buf, "Max (%1ld)", (long)(_adjustmax+0.5));
   _minlab = gtk_label_new("Min (0)"); 
   _maxlab = gtk_label_new(buf);

   gtk_table_attach_defaults(GTK_TABLE(tab), _adjustspin_w, 3,5,0,1);
   gtk_table_attach_defaults(GTK_TABLE(tab), _minlab, 0,2,1,2);
   gtk_table_attach_defaults(GTK_TABLE(tab), _adjustscale_w, 2,6,1,2);
   gtk_table_attach_defaults(GTK_TABLE(tab), _maxlab, 6,8,1,2);
   show(); 
}

int PegSpinAdjustDialog::show()
{
   gtk_widget_show_all(_dialog);
   return 0;
}

int PegSpinAdjustDialog::doit()
{
   if (_adj)
   {
      create_dialog();
      if (gtk_dialog_run(GTK_DIALOG(_dialog)) == GTK_RESPONSE_OK)
      {
	 g_object_set(_adj, "step-increment",
            gtk_spin_button_get_value(GTK_SPIN_BUTTON(_adjustspin_w)), NULL);
      }
      gtk_widget_destroy(_dialog);
      return 0; 
   } 
   return 1;
}

int PegSpinAdjustDialog::set_adjustscale_value(double val)
{
   if ((val < _adjustmin) || (val > _adjustmax)) return 1; 
   if (val != gtk_range_get_value(GTK_RANGE(_adjustscale_w)))
      gtk_range_set_value(GTK_RANGE(_adjustscale_w), val); 
   return 0; 
}

int PegSpinAdjustDialog::set_adjustspin_value(double val)
{
   if ((val < _adjustmin) || (val > _adjustmax)) return 1; 
   if (val != gtk_spin_button_get_value(GTK_SPIN_BUTTON(_adjustspin_w)))
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(_adjustspin_w), val);
   return 0; 
}


double PegSpinAdjustDialog::adjustspin_value_callback(GObject *object, gpointer user_data)
{
   GtkSpinButton *ent = GTK_SPIN_BUTTON(object);
   PegSpinAdjustDialog* cmd = (PegSpinAdjustDialog*)user_data;

   if (!(ent&&cmd))  return 1;
   return cmd->set_adjustscale_value(gtk_spin_button_get_value(ent));
}

double PegSpinAdjustDialog::adjustscale_value_callback(GObject *object, gpointer user_data)
{
   GtkScale *ent = GTK_SCALE(object);
   PegSpinAdjustDialog* cmd = (PegSpinAdjustDialog*)user_data;

   if (!(ent&&cmd))  return 1;
   return cmd->set_adjustspin_value(gtk_range_get_value(GTK_RANGE(ent))); 
}

