// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PegUtils.hh"
#include "Parameter.hh"
#include "float.h"

#include <sstream>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

using namespace std; 

string _double_to_string(double val)
{
   static string outval;
   if (val == INDEF_REAL)
      outval = "INDEF";
   else 
   {
      stringstream ss;
      ss.precision(DBL_DIG);
      ss << val;
      ss >> outval;
   }
   return outval;
}

string _int_to_string(int val)
{
   static string outval; 
   if (val == INDEF_INT)
      outval = "INDEF";
   else
   {
      stringstream ss;
      ss << val;
      ss >> outval;
   }
   return outval;
}

string _bool_to_string(bool val)
{
   static string outval;
   if (val)
      outval = "yes";
   else 
      outval = "no";
   return outval; 
}

int _string_to_bool(string val, bool &outv)
{
   char cbuf[129];
   char *cp = cbuf;

   string tmp2, tmp = val;
   tmp.erase(tmp.find_last_not_of(" \t")+1);
   istringstream ss(tmp);
   if (!((ss >> tmp2) && ss.eof())) return 1;

   strncpy(cbuf, tmp2.c_str(), 128);
   while (*cp)
   {
      if (isupper(*cp)) *cp = tolower(*cp);
         cp++;
   }
   if ((strcmp(cbuf, "yes") == 0) || (strcmp(cbuf, "y") == 0) ||
       (strcmp(cbuf, "true") == 0) || (strcmp(cbuf, "t") == 0) ||
       (strcmp(cbuf, "on") == 0) || (strcmp(cbuf, "1") == 0))
      outv = 1;
   else if ((strcmp(cbuf, "no") == 0) || (strcmp(cbuf, "n") == 0) ||
       (strcmp(cbuf, "false") == 0) || (strcmp(cbuf, "f") == 0) ||
       (strcmp(cbuf, "off") == 0) || (strcmp(cbuf, "0") == 0))
      outv = 0;
   else
      return 1;   // unrecognized text as value
   return 0;
}


string _file_get_root_name(string infile)
{
   static string outval;
   string::size_type len, p1, p2;

   if ((len = infile.length()) != 0)
   {
      p1 = infile.find_last_of('/');
      if ((p2 = infile.find('.', ++p1)) == string::npos)  p2 = len;
      outval = infile.substr(p1, p2-p1);
   }
   else  outval = "";
   return outval;
}

bool _check_is_directory(string path)
{
   struct stat   sbuf;
   mode_t        mode;

   if (!stat(path.c_str(), &sbuf))
      mode = sbuf.st_mode;
   else
      return 0;
   return (S_ISDIR(mode) && !(S_ISREG(mode) || S_ISCHR(mode) ||
           S_ISBLK(mode) || S_ISFIFO(mode)));
}

