// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismAppendColumnCmd.hh"
#include "PrismDataTable.hh"
#include "PrismApp.hh"
#include "PrismUtils.hh"
#include <iostream>
#include <sstream>

using namespace std;


PrismAppendColumnCmd::PrismAppendColumnCmd():PrismBaseObj()
{
   _ctype = PRISM_SCALAR_COL;
   _dtype = PRISM_SHORT_TYPE;
   _ssize = 8;
   _asize = 2;
   _vdim = 2;
   _infra.components = NULL; 
   _infra.values = NULL; 
   _colnames = NULL; 
   gdk_color_parse ("red", &_errcolor);
}

PrismAppendColumnCmd::~PrismAppendColumnCmd()
{
}

void PrismAppendColumnCmd::create_dialog()
{
   base_w = gtk_dialog_new_with_buttons("Append Column to Table", NULL,
      GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_OK,
      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
   _errtxt_lw = gtk_label_new("Name field is required     "); 

   GtkWidget* tab = gtk_table_new(6, 8, true); //sine no initialize fields yet
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), tab);
   _scalar_w = gtk_radio_button_new_with_label(NULL, "Scalar ");
   _array_w = gtk_radio_button_new_with_label_from_widget(
      GTK_RADIO_BUTTON(_scalar_w), "Array ");
   _vector_w = gtk_radio_button_new_with_label_from_widget(
      GTK_RADIO_BUTTON(_scalar_w), "Vector");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_scalar_w), true);

   _frame_w = gtk_frame_new(" Column Type ");
   _coltype_w = gtk_hbutton_box_new();
   gtk_container_add(GTK_CONTAINER(_frame_w), _coltype_w);
   gtk_container_add(GTK_CONTAINER(_coltype_w), _scalar_w);
   gtk_container_add(GTK_CONTAINER(_coltype_w), _array_w);
   gtk_container_add(GTK_CONTAINER(_coltype_w), _vector_w);

   _attframe_w = gtk_frame_new(" Attributes ");
   _atttable_w = gtk_table_new(6,9, true);
   _name_lw = gtk_label_new("Name");
   _name_w = gtk_entry_new();
   _units_lw = gtk_label_new("Units");
   _units_w = gtk_entry_new();
   _desc_lw = gtk_label_new("Description");
   _desc_w = gtk_entry_new();
   _type_lw = gtk_label_new("Data Type");
   _type_w = gtk_combo_box_new_text();
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_type_w), PRISM_SHORT_TYPE, 
                             "short");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_type_w), PRISM_LONG_TYPE,
                             "long");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_type_w), PRISM_FLOAT_TYPE,
                             "float");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_type_w), PRISM_DOUBLE_TYPE,
                             "double");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_type_w), PRISM_STRING_TYPE,
                             "string");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_type_w), PRISM_UBYTE_TYPE,
                             "unsigned byte");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_type_w), PRISM_USHORT_TYPE, 
                             "unsigned short");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_type_w), PRISM_ULONG_TYPE,
                             "unsigned long");
   gtk_combo_box_set_active(GTK_COMBO_BOX(_type_w), _dtype);
   _ssize_lw = gtk_label_new("String Length");
   GtkAdjustment *adj = (GtkAdjustment *)
      gtk_adjustment_new(_ssize, 1, 1024, 1, 1, 0);
   _ssize_w = gtk_spin_button_new(adj, 1, 0);
   _asize_lw = gtk_label_new("Array size");
   adj = (GtkAdjustment *)gtk_adjustment_new(_asize, 2, 1024, 1, 1, 0);
   _asize_w = gtk_spin_button_new(adj, 1, 0);
   _vdim_lw = gtk_label_new("Vector Dim");
   adj = (GtkAdjustment *)gtk_adjustment_new(_asize, 2, 32, 1, 1, 0);
   _vdim_w = gtk_spin_button_new(adj, 1, 0);
   _vcomp_lw = gtk_label_new("Components ");
   _vcomp_w = gtk_entry_new();
   _defval_lw = gtk_label_new("Default Value");
   _defval_w = gtk_entry_new();

   gtk_container_add(GTK_CONTAINER(_attframe_w), _atttable_w);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _name_lw, 0,2,0,1);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _name_w, 2,8,0,1);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _units_lw, 0,2,1,2);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _units_w, 2,8,1,2);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _desc_lw, 0,2,2,3);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _desc_w, 2,8,2,3);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _type_lw, 0,2,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _type_w, 2,5,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _ssize_lw, 5,7,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _ssize_w, 7,8,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _asize_lw, 0,2,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _asize_w, 2,3,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _vdim_lw, 0,2,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _vdim_w, 2,3,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _vcomp_lw, 3,5,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _vcomp_w, 5,8,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _defval_lw, 0,2,5,6);
   gtk_table_attach_defaults(GTK_TABLE(_atttable_w), _defval_w, 2,8,5,6);

   gtk_table_attach_defaults(GTK_TABLE(tab), _errtxt_lw, 1,7,5,6);

   g_signal_connect(_scalar_w,"toggled",G_CALLBACK(toggled_value_callback),
                    (void*)this);
   g_signal_connect(_array_w,"toggled",G_CALLBACK(toggled_value_callback),
                    (void*)this);
   g_signal_connect(_vector_w,"toggled",G_CALLBACK(toggled_value_callback),
                    (void*)this);
   g_signal_connect(_type_w,"changed",G_CALLBACK(combobox_callback),
                    (void*)this);
   g_signal_connect(_vdim_w,"value-changed",G_CALLBACK(check_callback),
                    (void*)this);
   g_signal_connect(_vcomp_w,"changed",G_CALLBACK(check_callback),
                    (void*)this);
   g_signal_connect(_name_w,"changed",G_CALLBACK(check_callback),
                    (void*)this);
   g_signal_connect(_defval_w,"changed",G_CALLBACK(check_callback),
                    (void*)this);

   gtk_table_attach_defaults(GTK_TABLE(tab), _frame_w, 1,7,0,1);
   gtk_table_attach_defaults(GTK_TABLE(tab), _attframe_w, 1,7,1,5);
   show(); 
}

int PrismAppendColumnCmd::show()
{
   gtk_widget_show_all(base_w);
   return 0;
}

int PrismAppendColumnCmd::doit(PrismDataTable* table)
{
   int rv  = 0; 

   if (!theApp)             return 1;   // app doesn't exist
   if (!(_table = table))   return 2;    // table doesn't exist

   _colcnt = _table->get_column_names(_colnames);    
   create_dialog();
   set_visible(); 
   if (gtk_dialog_run(GTK_DIALOG(base_w)) == GTK_RESPONSE_OK)
   {
      _infra.name = gtk_entry_get_text(GTK_ENTRY(_name_w));
      _infra.units = gtk_entry_get_text(GTK_ENTRY(_units_w));
      _infra.comment = gtk_entry_get_text(GTK_ENTRY(_desc_w));
      _infra.ctype = _ctype;
      _infra.dtype = _dtype;
      _infra.vsize = _vdim = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(_vdim_w));
      _infra.asize = _asize = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(_asize_w));
      _infra.ssize = _ssize = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(_ssize_w));
      if (_vdim > 1)
      {
	 if (_infra.components)  delete[] _infra.components; 
         _infra.components = new string[_vdim]; 
         stringstream ss(gtk_entry_get_text(GTK_ENTRY(_vcomp_w)));
         int ii = 0;
	 while ((ii < _vdim) && (ss >> _infra.components[ii])) ii++;
      }

      // initial value
      _infra.valcnt = 0; 
      if (_infra.values) 
      {
         delete[] _infra.values;
         _infra.values = NULL;
      }
      if (_defsize && (_dtype == PRISM_SHORT_TYPE || _dtype == PRISM_LONG_TYPE ||
          _dtype == PRISM_FLOAT_TYPE || _dtype == PRISM_DOUBLE_TYPE ||
          _dtype == PRISM_USHORT_TYPE || _dtype == PRISM_ULONG_TYPE))
      {
	 int ii = 0;
	 stringstream ss(gtk_entry_get_text(GTK_ENTRY(_defval_w)));
         
         if (_infra.ctype == PRISM_SCALAR_COL)
         {
            _infra.values = new string[_infra.valcnt = 1];
            ss >> _infra.values[ii++];
         }
         else if (_infra.ctype == PRISM_VECTOR_COL) 
         {
            _infra.values = new string[_infra.valcnt = _vdim];
            ss >> _infra.values[ii++];
            if (_defsize > 1)
	       while ((ii < _vdim) && (ss >> _infra.values[ii])) ii++;
	    else
	       while (ii < _vdim) _infra.values[ii++] = _infra.values[0]; 
	 }
         else 
         {
            _infra.values = new string[_infra.valcnt = _asize];
            ss >> _infra.values[ii++];
            if (_defsize > 1)
	       while ((ii < _asize) && (ss >> _infra.values[ii])) ii++;
	    else
	       while (ii < _asize) _infra.values[ii++] = _infra.values[0]; 
	 }
      }

      rv= _table->append_column(&_infra);

      if (_infra.components)  delete[] _infra.components;
      _infra.components = NULL; 
      if (_infra.values)  delete[] _infra.values;
      _infra.values = NULL; 
   }
   if (_colnames)
   {
      delete[] _colnames;
      _colnames = NULL; 
   }
   gtk_widget_destroy(base_w);
   return rv;
}


int PrismAppendColumnCmd::toggled_value_callback(GObject *object, gpointer user_data)
{
   PrismAppendColumnCmd* cmd = (PrismAppendColumnCmd*)user_data;
   if (!(cmd))  return 1;
   cmd->set_visible();
   return 0; 
}

int PrismAppendColumnCmd::combobox_callback(GObject *object, gpointer user_data)
{
   PrismAppendColumnCmd* cmd = (PrismAppendColumnCmd*)user_data;
   if (!(cmd))  return 1;
   cmd->set_visible();
   return 0; 
}

int PrismAppendColumnCmd::check_callback(GObject *object, gpointer user_data)
{
   PrismAppendColumnCmd* cmd = (PrismAppendColumnCmd*)user_data;
   if (!(cmd))  return 1;
   cmd->validity_check();
   return 0; 
}


void PrismAppendColumnCmd::set_visible()
{
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_scalar_w)))
   {
      _ctype = PRISM_SCALAR_COL;
      gtk_widget_hide(_asize_lw); 
      gtk_widget_hide(_asize_w); 
      gtk_widget_hide(_vdim_lw); 
      gtk_widget_hide(_vdim_w); 
      gtk_widget_hide(_vcomp_lw); 
      gtk_widget_hide(_vcomp_w); 
   }
   else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_array_w)))
   {
      _ctype = PRISM_ARRAY_COL;
      gtk_widget_show(_asize_lw); 
      gtk_widget_show(_asize_w); 
      gtk_widget_hide(_vdim_lw); 
      gtk_widget_hide(_vdim_w); 
      gtk_widget_hide(_vcomp_lw); 
      gtk_widget_hide(_vcomp_w); 
   }
   else 
   {
      _ctype = PRISM_VECTOR_COL;
      gtk_widget_show(_vdim_lw); 
      gtk_widget_show(_vdim_w); 
      gtk_widget_show(_vcomp_lw); 
      gtk_widget_show(_vcomp_w); 
      gtk_widget_hide(_asize_lw); 
      gtk_widget_hide(_asize_w); 
   }
   // if datatype is 'string' - show widgets needed to set string length 
   if ((_dtype = gtk_combo_box_get_active(GTK_COMBO_BOX(_type_w))) != PRISM_STRING_TYPE)
   {
      gtk_widget_set_sensitive(GTK_WIDGET(_ssize_lw), 0); 
      gtk_widget_set_sensitive(GTK_WIDGET(_ssize_w), 0); 
      if (_dtype != PRISM_UBYTE_TYPE)
         gtk_widget_set_sensitive(GTK_WIDGET(_defval_w), 1);
      else
         gtk_widget_set_sensitive(GTK_WIDGET(_defval_w), 0);
   }
   else 
   {
      gtk_widget_set_sensitive(GTK_WIDGET(_ssize_lw), 1); 
      gtk_widget_set_sensitive(GTK_WIDGET(_ssize_w), 1); 
      gtk_widget_set_sensitive(GTK_WIDGET(_defval_w), 0);
   }
   validity_check(); 
}


void PrismAppendColumnCmd::validity_check()
{
   bool valid = 1; 
   string tmpstr =  gtk_entry_get_text(GTK_ENTRY(_name_w));

   // column name must be provided 
   if (!tmpstr.length())
   {
      valid = 0; 
      gtk_widget_modify_bg(_name_w, GTK_STATE_NORMAL, &_errcolor);
      gtk_label_set_label(GTK_LABEL(_errtxt_lw), "Name field is required"); 
   }
   else if (check_column_name(tmpstr)) 
   {
      gtk_widget_modify_bg(_name_w, GTK_STATE_NORMAL, &_errcolor);
      string emsg = "Column name " + tmpstr + " is already in use";
      gtk_label_set_label(GTK_LABEL(_errtxt_lw), emsg.c_str()); 
      valid = 0;
   }
   else 
   {
      gtk_widget_modify_bg(_name_w, GTK_STATE_NORMAL, NULL);
   }

   if (_ctype == PRISM_VECTOR_COL)
   {
      // must contain 1 cpt name for each dim 
      _vdim = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(_vdim_w));
      stringstream ss(gtk_entry_get_text(GTK_ENTRY(_vcomp_w)));
      int ii = 0;
      string tmpstr2;
      while (ss >> tmpstr2)
      {
	 // check that block doesn't already contain the name
	 if (check_column_name(tmpstr2))
	 {
            gtk_widget_modify_bg(_name_w, GTK_STATE_NORMAL, &_errcolor);
            string emsg = "Component name " + tmpstr2 + " is already in use";
            gtk_label_set_label(GTK_LABEL(_errtxt_lw), emsg.c_str()); 
            valid = 0;
	    return; 
	 }
         ii++;
      }
      if (_vdim != ii)
      {
         valid = 0;  
   gtk_label_set_label(GTK_LABEL(_errtxt_lw), "Components field must contain Vector Dim entries"); 
         gtk_widget_modify_bg(_vcomp_w, GTK_STATE_NORMAL, &_errcolor);
      }
      else 
      {
	 int ii = 0; 
         string* tmplist = new string[_vdim + 1];
	 tmplist[ii] =  tmpstr; 
         stringstream ss(gtk_entry_get_text(GTK_ENTRY(_vcomp_w)));

	 // check for duplicate entries
         for (ii = 1; ii <= _vdim; ii++)  
	 {
	    ss >> tmplist[ii];
            for (int jj = 0; jj < ii; jj++)
               if (!_strcmp_cis(tmplist[ii].c_str(), tmplist[jj].c_str()))
	       {
	          valid = 0;
		  string emsg = tmplist[ii] + " is already used in the Components or Name field";
                  gtk_label_set_label(GTK_LABEL(_errtxt_lw), emsg.c_str());
                  gtk_widget_modify_bg(_vcomp_w, GTK_STATE_NORMAL, &_errcolor);
                  if (tmplist) delete[] tmplist; 
	          return; 
	       }
	 }
         if (tmplist) delete[] tmplist; 

         gtk_widget_modify_bg(_vcomp_w, GTK_STATE_NORMAL, NULL);
      }
   }

   // check default values entry 
   _asize = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_asize_w));
   string defstr =  gtk_entry_get_text(GTK_ENTRY(_defval_w));
   if (defstr.length())
   {
      _defsize = 0;
      string tmpstr2;
      stringstream dvss(defstr);
      while (dvss >> tmpstr2) _defsize++;
     
      if (_ctype == PRISM_SCALAR_COL && _defsize > 1)
      {
         valid = 0;
         string emsg = "A scalar column may have a single default value";
         gtk_label_set_label(GTK_LABEL(_errtxt_lw), emsg.c_str());
         gtk_widget_modify_bg(_defval_w, GTK_STATE_NORMAL, &_errcolor);
         return; 
      }
      else if (_ctype == PRISM_ARRAY_COL && 
               (_defsize > 1 && _defsize != _asize))
      {
         valid = 0;
	 char buff[80];
	 sprintf(buff,"A %1ld element array column may have 0, 1, or %1ld default values",
	        _asize, _asize);
         gtk_label_set_label(GTK_LABEL(_errtxt_lw), buff);
         gtk_widget_modify_bg(_defval_w, GTK_STATE_NORMAL, &_errcolor);
         return; 
      }
      else if (_ctype == PRISM_VECTOR_COL && 
               (_defsize > 1 && _defsize != _vdim))
      {
         valid = 0;
	 char buff[80];
	 sprintf(buff,"The vector column may have 0, 1, or %1ld default values",
	        _vdim);
         gtk_label_set_label(GTK_LABEL(_errtxt_lw), buff);
         gtk_widget_modify_bg(_defval_w, GTK_STATE_NORMAL, &_errcolor);
         return; 
      }
   }
   gtk_widget_modify_bg(_defval_w, GTK_STATE_NORMAL, NULL);
   if (valid)  gtk_label_set_label(GTK_LABEL(_errtxt_lw), "     "); 
   gtk_dialog_set_response_sensitive(GTK_DIALOG(base_w), GTK_RESPONSE_OK,
                                     valid);
}

int PrismAppendColumnCmd::check_column_name(string proposed)
{
   if (_colcnt && _colnames)
   {
      for (int ii = _colcnt; ii--; )
         if (!_strcmp_cis(_colnames[ii].c_str(), proposed.c_str()))   return 1; 
   }
   return 0; 
}

