// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//



#include "PrismDataColumn.hh"
#include "PrismUtils.hh"
#include <iostream>
#include <sstream>
#include <float.h>
#include <ctype.h>
#include <string.h>
#include "dsnan.h"
#include <string.h>
#include <stdlib.h>

using namespace std; 


PrismDataColumn::PrismDataColumn()
{
   _vpos = _vmin = _vmax = _apos = _amin = _amax = -1;
   _adim = _asize = 0;
   _vis = 1; 
   _virtual = _image = 0;
   _compname = ""; 
   _expstate = 0;
   _vcomp = _acomp = false; 
   _format = ""; 
}


PrismDataColumn::PrismDataColumn(dmDescriptor* desc, int pos)
{
   _vpos = _vmin = _vmax = _apos = _amin = _amax = -1;
   _vis = 1; 
   _virtual = _image = 0;
   _expstate = 0;
   _compname = ""; 
   _vcomp = _acomp = false; 
   _format = ""; 
   set_desc(desc, pos); 
}


void PrismDataColumn::set_image_desc(dmDescriptor* desc)
{
   _image = 1;
   set_desc(desc, 0); 
   long* axes;
   long dim;
   _imgnaxes = dmGetArrayDimensions(desc, &axes);
   if (_imgnaxes > 0 && _imgnaxes < 5)
   {
      if ((dim = dmGetArrayDim(desc)) == _imgnaxes)
      {
         _imgcols = axes[0];
         _imgrows = axes[1];
         // Treat 1D images as Nx1
         if (_imgrows == 0 && _imgcols > 0) _imgrows = 1;
      }
      free(axes);
   }
   else
   {
      _imgcols = _imgrows = 0; 
   }
}


void PrismDataColumn::set_desc(dmDescriptor* desc, int pos,
   int vpos, int vmin, int vmax, int apos, int amin, int amax)
{
   set_desc(desc, pos);
   _vpos = vpos;
   _vmin = vmin;
   _vmax = vmax;
   _apos = apos;
   _amin = amin;
   _amax = amax;
   _vis = 1;
}


void PrismDataColumn::set_desc(dmDescriptor* desc, int pos)
{
   if (desc)
   {
      char colName[129];
      _desc = desc; 
      _pos = pos; 
      _dtype = dmGetDataType(desc);
      if ((_dim = dmGetElementDim(_desc)) > 1)
      {
         _vpos = _pos;
	 _vmin = _vpos + 1;
	 _vmax = _vpos + _dim;
      }
      if ((_adim = dmGetArrayDim(_desc)) > 0 || _dtype == dmBIT) 
      {
         _asize = dmGetArraySize(_desc);
         if (_dtype == dmBIT)  _asize *= 8; 
	 _apos = _pos; 
	 _amin = _apos + 1;
	 _amax = _apos + _asize; 
      }
      colName[0] = '\0'; 
      dmGetName(_desc,colName,128);
      _name = colName; 

      // get meta data
      stringstream ss, ss2, ss3;
      char buffer[1025];
      long lval; 
      dmGetDesc(desc, buffer, 1024);
      _comment = buffer; 
      dmGetUnit(desc, buffer, 1024);
      _units = buffer;
      _coordsys = _crpix = _crval = _cdelt = "";
      _numcoords = dmDescriptorGetNoCoords(desc);

      // handle associated coord systems 
      dmDescriptor* coorddesc;
      stringstream s1, s2, s3; 
      string tmp;
      for (int rr = 0; rr < _numcoords; rr++)
         if ((coorddesc = dmDescriptorGetCoordNo(desc, rr+1)) != NULL)
         {
            double *crval = new double[_dim];  
            double *cdelt = new double[_dim];  
	    char crdName[129], crdVal[129];
	    dmGetName(coorddesc,crdName,128);
	    _coordsys = crdName; 
	    _coordsys.append("("); 
	    for (int zz =1; zz <= _dim; zz++)
	    {
	       dmGetCptName(coorddesc, zz, crdVal, 128);
	       _coordsys.append(crdVal);
	       if (zz < _dim) _coordsys.append(","); 
	    }
	    _coordsys.append(")"); 

            switch(_dtype)
	    {
               case dmDOUBLE:
	       {
	          double *crpix = new double[_dim];
	          dmCoordGetTransform_d(coorddesc, crpix, crval, cdelt, _dim); 
                  s1.precision(16);
                  s2.precision(16);
                  s3.precision(16);
		  _crval = "(";
		  _crpix = "("; 
		  _cdelt = "("; 
		  for (rr = 0; rr < _dim; rr++)
		  {
		     s1 << crpix[rr];
		     s1 >> tmp;
		     s1.clear();
		     _crpix.append(tmp);
		     s2 << crval[rr];
		     s2 >> tmp;
		     s2.clear();
		     _crval.append(tmp);
		     s3 << cdelt[rr];
		     s3 >> tmp;
		     s3.clear();
		     _cdelt.append(tmp);
		     if (rr + 1 < _dim)
		     {
		        _crpix.append(","); 
		        _crval.append(","); 
		        _cdelt.append(","); 
		     }
		  }
		  _crpix.append(")"); 
		  _crval.append(")"); 
		  _cdelt.append(")"); 
                  delete []crpix; 
	       }
	       break; 
               case dmFLOAT:
	       {
                  float *crpix = new float[_dim]; 
	          dmCoordGetTransform_f(coorddesc, crpix, crval, cdelt, _dim); 
		  _crval = "(";
		  _crpix = "("; 
		  _cdelt = "("; 
		  for (rr = 0; rr < _dim; rr++)
		  {
		     s1 << crpix[rr];
		     s1 >> tmp;
		     s1.clear();
		     _crpix.append(tmp);
		     s2 << crval[rr];
		     s2 >> tmp;
		     s2.clear();
		     _crval.append(tmp);
		     s3 << cdelt[rr];
		     s3 >> tmp;
		     s3.clear();
		     _cdelt.append(tmp);
		     if (rr + 1 < _dim)
		     {
		        _crpix.append(","); 
		        _crval.append(","); 
		        _cdelt.append(","); 
		     }
		  }
		  _crpix.append(")"); 
		  _crval.append(")"); 
		  _cdelt.append(")"); 
                  delete []crpix; 
	       }
	       break; 
               case dmSHORT:
	       {
	          short *crpix = new short[_dim];
	          dmCoordGetTransform_s(coorddesc, crpix, crval, cdelt, _dim); 
		  _crval = "(";
		  _crpix = "("; 
		  _cdelt = "("; 
		  for (rr = 0; rr < _dim; rr++)
		  {
		     s1 << crpix[rr];
		     s1 >> tmp;
		     s1.clear();
		     _crpix.append(tmp);
		     s2 << crval[rr];
		     s2 >> tmp;
		     s2.clear();
		     _crval.append(tmp);
		     s3 << cdelt[rr];
		     s3 >> tmp;
		     s3.clear();
		     _cdelt.append(tmp);
		     if (rr + 1 < _dim)
		     {
		        _crpix.append(","); 
		        _crval.append(","); 
		        _cdelt.append(","); 
		     }
		  }
		  _crpix.append(")"); 
		  _crval.append(")"); 
		  _cdelt.append(")"); 
                  delete []crpix; 
	       }
	       break; 
               case dmLONG:
	       {
	          long *crpix = new long[_dim];
	          dmCoordGetTransform_l(coorddesc, crpix, crval, cdelt, _dim); 
		  _crval = "(";
		  _crpix = "("; 
		  _cdelt = "("; 
		  for (rr = 0; rr < _dim; rr++)
		  {
		     s1 << crpix[rr];
		     s1 >> tmp;
		     s1.clear();
		     _crpix.append(tmp);
		     s2 << crval[rr];
		     s2 >> tmp;
		     s2.clear();
		     _crval.append(tmp);
		     s3 << cdelt[rr];
		     s3 >> tmp;
		     s3.clear();
		     _cdelt.append(tmp);
		     if (rr + 1 < _dim)
		     {
		        _crpix.append(","); 
		        _crval.append(","); 
		        _cdelt.append(","); 
		     }
		  }
		  _crpix.append(")"); 
		  _crval.append(")"); 
		  _cdelt.append(")"); 
                  delete []crpix; 
	       }
	       break; 
               case dmULONG:
	       {
	          unsigned long *crpix = new unsigned long[_dim];
	          dmCoordGetTransform_ul(coorddesc, crpix, crval, cdelt, _dim);
		  _crval = "(";
		  _crpix = "("; 
		  _cdelt = "("; 
		  for (rr = 0; rr < _dim; rr++)
		  {
		     s1 << crpix[rr];
		     s1 >> tmp;
		     s1.clear();
		     _crpix.append(tmp);
		     s2 << crval[rr];
		     s2 >> tmp;
		     s2.clear();
		     _crval.append(tmp);
		     s3 << cdelt[rr];
		     s3 >> tmp;
		     s3.clear();
		     _cdelt.append(tmp);
		     if (rr + 1 < _dim)
		     {
		        _crpix.append(","); 
		        _crval.append(","); 
		        _cdelt.append(","); 
		     }
		  }
		  _crpix.append(")"); 
		  _crval.append(")"); 
		  _cdelt.append(")"); 
                  delete []crpix; 
	       }
	       break; 

	       default:
	       break; 
	    }
            delete []crval; 
            delete []cdelt; 
	 }

      switch (_dtype)
      {
         case dmDOUBLE:
	 {
	    double tnulld = 0;
	    dmDescriptorGetRange_d(desc, &_tlmin_d, &_tlmax_d);
	    ss.precision(16);
	    ss2.precision(16);
	    ss3.precision(16);
	    ss << _tlmin_d;
	    ss >> _tlmin;
	    ss2 << _tlmax_d;
	    ss2 >> _tlmax;
	    lval = dmDescriptorGetNull_d(desc, &tnulld);
	    if (lval == dmFAILURE && tnulld == 0)  _tnull = "NaN";
	    else
	    {
	       ss3 << tnulld;
	       ss3 >> _tnull;
	    }
	    _type = "double";
	 }
	 break;
         case dmFLOAT:
	 {
	    float tnullf = 0;
	    dmDescriptorGetRange_f(desc, &_tlmin_f, &_tlmax_f);
	    ss.precision(FLT_DIG);
	    ss2.precision(FLT_DIG);
	    ss3.precision(FLT_DIG);
	    ss << _tlmin_f;
	    ss >> _tlmin;
	    ss2 << _tlmax_f;
	    ss2 >> _tlmax;
	    lval = dmDescriptorGetNull_f(desc, &tnullf);
	    if (lval == dmFAILURE && tnullf == 0)  _tnull = "NaN";
	    else
	    {
	       ss3 << tnullf;
	       ss3 >> _tnull;
	    }
	    _type = "float";
	 }
	 break;
         case dmLONG:
	 {
	    dmDescriptorGetRange_l(desc, &_tlmin_l, &_tlmax_l);
	    ss << _tlmin_l;
	    ss >> _tlmin;
	    ss2 << _tlmax_l;
	    ss2 >> _tlmax;
	    lval = dmDescriptorGetNull_l(desc, &_tnull_l);
	    if (lval == dmFAILURE && _tnull_l == 0)  _tnull = "";
	    else
	    {
	       ss3 << _tnull_l;
	       ss3 >> _tnull;
	    }
	    _type = "long";
	 }
	 break;
         case dmULONG:
	 {
	    dmDescriptorGetRange_ul(desc, &_tlmin_ul, &_tlmax_ul);
	    ss << _tlmin_ul;
	    ss >> _tlmin;
	    ss2 << _tlmax_ul;
	    ss2 >> _tlmax;
	    lval = dmDescriptorGetNull_ul(desc, &_tnull_ul);
	    if (lval == dmFAILURE && _tnull_ul == 0)  _tnull = "";
	    else
	    {
	       ss3 << _tnull_ul;
	       ss3 >> _tnull;
	    }
	    _type = "u. long";
	 }
	 break;
         case dmSHORT:
	 {
	    dmDescriptorGetRange_s(desc, &_tlmin_s, &_tlmax_s);
	    ss << _tlmin_s;
	    ss >> _tlmin;
	    ss2 << _tlmax_s;
	    ss2 >> _tlmax;
	    lval = dmDescriptorGetNull_s(desc, &_tnull_s);
	    if (lval == dmFAILURE && _tnull_s == 0)  _tnull = "";
	    else
	    {
	       ss3 << _tnull_s;
	       ss3 >> _tnull;
	    }
	    _type = "short";
	 }
	 break;
         case dmUSHORT:
	 {
	    dmDescriptorGetRange_us(desc, &_tlmin_us, &_tlmax_us);
	    ss << _tlmin_us;
	    ss >> _tlmin;
	    ss2 << _tlmax_us;
	    ss2 >> _tlmax;
	    lval = dmDescriptorGetNull_us(desc, &_tnull_us);
	    if (lval == dmFAILURE && _tnull_us == 0)  _tnull = "";
	    else
	    {
	       ss3 << _tnull_us;
	       ss3 >> _tnull;
	    }
	    _type = "u. short";
	 }
	 break;
         case dmBYTE:
	 {
	    dmDescriptorGetRange_ub(desc, &_tlmin_ub, &_tlmax_ub);
	    ss << (int)_tlmin_ub;
	    ss >> _tlmin;
	    ss2 << (int)_tlmax_ub;
	    ss2 >> _tlmax;
	    lval = dmDescriptorGetNull_ub(desc, &_tnull_ub);
	    if (lval == dmFAILURE && _tnull_ub == 0)  _tnull = "";
	    else
	    {
	       ss3 << (int)_tnull_ub;
	       ss3 >> _tnull;
	    }
	    _type = "byte";
	 }
	 break;
         case dmBOOL:
         {
            _tnull = "";
            _tlmin = "FALSE";
            _tlmax = "TRUE"; 
            _type = "boolean";
         }
         break; 
	 case dmBIT:
	 {
	    dmDescriptorGetRange_bit(desc, &_tlmin_bit, &_tlmax_bit);
            unsigned long ul = _tlmin_bit;
	    ss << ul;
	    ss >> _tlmin;
            ul = _tlmax_ub;
	    ss2 << ul;
            ss2 >> _tlmax;
            _tnull = "";
	    _type = "bit"; 
	 }
	 break; 
	 case dmTEXT:
	 {
	    char *tnv = NULL;
	    lval = dmDescriptorGetNull_c(desc, &tnv);
            if (lval == dmFAILURE || tnv == NULL)  _tnull = "";
	       else    _tnull = tnv; 
	    _tlmin = ""; 
	    _tlmax = ""; 
	    _type = "string"; 
	 }
	 break; 
	 default:
	 break; 
      }
   }
}

PrismDataColumn::~PrismDataColumn()
{
}

int PrismDataColumn::validate_value(string value)
{
   int rval = 1; 
   switch (_dtype)
   {
      case dmSHORT:
      {
         short val;
         if (!_string_to_short(value, val))
	    if (val < _tlmin_s || val > _tlmax_s)   rval = 2; // out of range
	    else   rval = 0; // valid and within range  
         else if ((_strcmp_cis(value.c_str(), "nan") == 0) && _tnull.length()) 
	    rval = 0;   // NaN provided and column supports  
      }
      break;
      case dmUSHORT:
      {
         unsigned short val;
         if (!_string_to_ushort(value, val))
	    if (val < _tlmin_us || val > _tlmax_us)   rval = 2; // out of range
	    else   rval = 0; // valid and within range  
         else if ((_strcmp_cis(value.c_str(), "nan") == 0) && _tnull.length()) 
	    rval = 0;   // NaN provided and column supports  
      }
      break;
      case dmLONG:
      {
         long val;
         if (!_string_to_long(value, val))
	    if (val < _tlmin_l || val > _tlmax_l)   rval = 2; // out of range
	    else   rval = 0; // valid and within range  
         else if ((_strcmp_cis(value.c_str(), "nan") == 0) && _tnull.length()) 
	    rval = 0;   // NaN provided and column supports  
      }
      break;
      case dmULONG:
      {
         unsigned long val;
         if (!_string_to_ulong(value, val))
	    if (val < _tlmin_ul || val > _tlmax_ul)   rval = 2; // out of range
	    else   rval = 0; // valid and within range  
         else if ((_strcmp_cis(value.c_str(), "nan") == 0) && _tnull.length()) 
	    rval = 0;   // NaN provided and column supports  
      }
      break;
      case dmDOUBLE:
      {
         double val;
         if (!_string_to_double(value, val))
	    if (val < _tlmin_d || val > _tlmax_d)   rval = 2; // out of range
	    else   rval = 0; // valid and within range  
         else if (_strcmp_cis(value.c_str(), "nan") == 0) 
	    rval = 0;   // NaN provided and column supports  
      }
      break;
      case dmFLOAT:
      {
         float val;
         if (!_string_to_float(value, val))
	    if (val < _tlmin_f || val > _tlmax_f)   rval = 2; // out of range
	    else   rval = 0; // valid and within range  
         else if (_strcmp_cis(value.c_str(), "nan") == 0) 
	    rval = 0;   // NaN provided and column supports  
      }
      break;
      case dmBYTE:
      {
	 short val;
	 if (!_string_to_short(value, val))
         {
	    if (val < 0 || val > 255) rval = 2;
	    else   rval = 0; 
         }
      }
      break; 
      case dmBIT:
      {
	 short val;
	 if (!_string_to_short(value, val))
         {
	    if (val < 0 || val > 1) rval = 2;
	    else   rval = 0; 
         }
      }
      break; 
      case dmBOOL:
      {
         if ((_strcmp_cis(value.c_str(),"true") == 0) || 
             (_strcmp_cis(value.c_str(),"false") == 0) || 
             (_strcmp_cis(value.c_str(),"T") == 0) || 
             (_strcmp_cis(value.c_str(),"F") == 0))
            rval = 0;
         else
            rval = 2; 
      }
      break; 
      case dmTEXT:
      {
         rval = 0; 
      }
      break; 
      default:
         // unknown or unsupported type (do nothing- rval already set to 1) 
      break;
   }
   if (rval == 0)
      _errmsg = "";
   else if (rval == 2)
      _errmsg = "Data value " + value + " is out of range ["+_tlmin+":"+_tlmax+"]";
   else 
      _errmsg = "Unable to convert " + value + " to datatype " + _type;
   return rval; 
}

int PrismDataColumn::set_value(string value)
{
   if (_adim == 0)
   {
      switch (_dtype)
      {
         case dmDOUBLE:
         {
	    double val; 
	    if (_strcmp_cis(value.c_str(), "nan") == 0)
	    {
               ds_MAKE_DNAN(val); 
	    }
	    else if (_string_to_double(value, val)) return 8; //conversion failed
	    dmSetScalar_d(_desc, val); 
         }
         break; 
         case dmFLOAT:
         {
	    float val; 
	    if (_strcmp_cis(value.c_str(), "nan") == 0)
	    {
               ds_MAKE_FNAN(val); 
	    }
	    else if (_string_to_float(value, val)) return 8; // conversion failed
	    dmSetScalar_f(_desc, val); 
         }
         break; 
         case dmLONG:
         {
	    long val; 
	    if (_strcmp_cis(value.c_str(), "nan") == 0)
               val = _tnull_l;
	    else if (_string_to_long(value, val)) return 8; // conversion failed
	    dmSetScalar_l(_desc, val); 
         }
         break; 
         case dmULONG:
         {
	    unsigned long val; 
	    if (_strcmp_cis(value.c_str(), "nan") == 0)
               val = _tnull_l;
	    else if (_string_to_ulong(value,val)) return 8; // conversion failed
	    dmSetScalar_ul(_desc, val); 
         }
         break; 
         case dmSHORT:
         {
	    short val; 
	    if (_strcmp_cis(value.c_str(), "nan") == 0)
               val = _tnull_s;
	    else if (_string_to_short(value, val)) return 8; // conversion failed
	    dmSetScalar_s(_desc, val); 
         }
         break; 
         case dmUSHORT:
         {
	    unsigned short val; 
	    if (_strcmp_cis(value.c_str(), "nan") == 0)
               val = _tnull_s;
	    else if (_string_to_ushort(value,val)) return 8; //conversion failed
	    dmSetScalar_us(_desc, val); 
         }
         break; 
         case dmBYTE:
         {
	    short val;
	    if (_strcmp_cis(value.c_str(), "nan") == 0)
               val = _tnull_ub;
	    else if (_string_to_short(value, val)) return 8;
	    { 
	       if (val < 0 || val > 255) return 8;
               unsigned char uch = (unsigned char)val;
	       dmSetScalar_ub(_desc, uch); 
	    }
         }
         break; 
	 case dmBIT:
	 {
	    short val; 
	    unsigned char uc;
	    if (_string_to_short(value, val)) return 8; // conversion failed
	    uc = val;
	    dmSetScalar_bit(_desc, (void*)uc);     
	 }
	 break; 
         case dmBOOL:
         {
            dmBool bv;
            if ((_strcmp_cis(value.c_str(), "true") == 0) ||
                (_strcmp_cis(value.c_str(), "t") == 0))
               bv = dmTRUE; 
            else
               bv = dmFALSE; 
            dmSetScalar_q(_desc, bv); 
         }
         break; 
	 case dmTEXT:
	 {
	    dmSetScalar_c(_desc, (char*)value.c_str());
	 }
	 break; 
         default:
	 break; 
      }
   }
   else
   {
      int loc = _pos - _amin; // entry offset = current pos - first array elem 
      return set_array_value(loc, value); 
   }
   return 0; 
}


int PrismDataColumn::set_array_value(long pos, string value)
{
   if (!(is_array() || is_array_comp()))   return 10;  // not an array  

   switch(_dtype)
   {
      case dmDOUBLE:
      {
         double *arr = new double[_asize];
         double val;
	 if (pos < 0 || pos >= _asize)  return 7;   // position out of range
	 if (_strcmp_cis(value.c_str(), "nan") == 0)
	 {
            ds_MAKE_DNAN(val); 
	 }
	 else if (_string_to_double(value, val)) return 8; //conversion failed
         dmGetArray_d(_desc, arr, _asize);
         arr[pos] = val;
         dmSetArray_d(_desc, arr, _asize);
	 delete []arr; 
      }
      break; 
      case dmFLOAT:
      {
         float *arr = new float[_asize];
         float val;
         if (pos < 0 || pos >= _asize)  return 7;   // position out of range
         if (_strcmp_cis(value.c_str(), "nan") == 0)
         {
            ds_MAKE_FNAN(val); 
         }
         else if (_string_to_float(value, val)) return 8; // conversion failed
         dmGetArray_f(_desc, arr, _asize);
         arr[pos] = val;
         dmSetArray_f(_desc, arr, _asize);
         delete []arr; 
      }
      break; 
      case dmLONG:
      {
         long *arr = new long[_asize];
         long val;
         if (pos < 0 || pos >= _asize)  return 7;   // position out of range
         if (_strcmp_cis(value.c_str(), "nan") == 0)
            val = _tnull_l;
	 else if (_string_to_long(value, val)) return 8; // conversion failed
         dmGetArray_l(_desc, arr, _asize);
         arr[pos] = val;
         dmSetArray_l(_desc, arr, _asize);
         delete []arr; 
      }
      break; 
      case dmULONG:
      {
         unsigned long *arr = new unsigned long[_asize];
         unsigned long val;
         if (pos < 0 || pos >= _asize)  return 7;   // position out of range
         if (_strcmp_cis(value.c_str(), "nan") == 0)
            val = _tnull_ul;
	 else if (_string_to_ulong(value, val)) return 8; // conversion failed
         dmGetArray_ul(_desc, arr, _asize);
         arr[pos] = val;
         dmSetArray_ul(_desc, arr, _asize);
         delete []arr; 
      }
      break; 
      case dmSHORT:
      {
         short *arr = new short[_asize];
         short val;
         if (pos < 0 || pos >= _asize)  return 7;   // position out of range
         if (_strcmp_cis(value.c_str(), "nan") == 0)
            val = _tnull_s;
         else if (_string_to_short(value, val)) return 8; // conversion failed
         dmGetArray_s(_desc, arr, _asize);
         arr[pos] = val;
         dmSetArray_s(_desc, arr, _asize);
         delete []arr; 
      }
      break; 
      case dmBOOL:
      {
         int *arr = new int[_asize];
         int val;
         if (pos < 0 || pos >= _asize)  return 7;   // position out of range
         if ((_strcmp_cis(value.c_str(), "true") == 0) ||
             (_strcmp_cis(value.c_str(), "t") == 0))
            val = dmTRUE;
         else if ((_strcmp_cis(value.c_str(), "false") == 0) ||
             (_strcmp_cis(value.c_str(), "f") == 0))
            val = dmFALSE;
         else 
            return 8; // conversion failed
         dmGetArray_q(_desc, arr, _asize);
         arr[pos] = val;
         dmSetArray_q(_desc, arr, _asize);
         delete []arr;
      }
      break;
      case dmUSHORT:
      {
         unsigned short *arr = new unsigned short[_asize];
         unsigned short val;
         if (pos < 0 || pos >= _asize)  return 7;   // position out of range
         if (_strcmp_cis(value.c_str(), "nan") == 0)
            val = _tnull_s;
         else if (_string_to_ushort(value, val)) return 8; // conversion failed
         dmGetArray_us(_desc, arr, _asize);
         arr[pos] = val;
         dmSetArray_us(_desc, arr, _asize);
         delete []arr; 
      }
      break; 
      case dmBIT:
      {
         int mysize = _asize / 8; 
         if (mysize > 0)
	 {
            unsigned char *arr = new unsigned char[mysize];
            short val;
	    if (pos < 0 || pos >= _asize)  return 7;   // position out of range
	    if (_string_to_short(value, val)) return 8; // conversion failed
            dmGetArray_bit(_desc, arr, mysize);
	    int mybyte =  pos/8; 
	    unsigned char bitpat = 0x80 >> pos%8; 
	    if (val)
	       arr[mybyte] |= bitpat;
	    else 
	    {
	       // do we need to change the bit from a 1 to a zero?
	       if ((arr[mybyte] & bitpat) == bitpat)
	          arr[mybyte] -= bitpat; 
	    }
            dmSetArray_bit(_desc, arr, mysize);
	    delete []arr; 
         }
      }
      break; 
      case dmTEXT:
      {
         // allocate array for strings 
         char **arr = new char*[_asize];
         long xx, blen = dmDescriptorGetLength(_desc) + 1; 
         for (xx = _asize; xx--; )   arr[xx] = new char[blen];

         if (pos < 0 || pos >= _asize)  return 7;   // position out of range
         dmGetArray_c(_desc, arr, _asize);
         strcpy(arr[pos],(char*)value.c_str());
         dmSetArray_c(_desc, arr, _asize);
         for (xx = _asize; xx--; )   delete [](arr[xx]);
         delete []arr;
      }
      break; 
      default:
      break; 
   }
   return 0; 
}


int PrismDataColumn::set_image_value(long col, long row, string value)
{
   long pix_pos[2];
   
   pix_pos[0] = col + 1;
   pix_pos[1] = row;

   switch(_dtype)
   {
      case dmDOUBLE:
      {
         double val;
         if (_strcmp_cis(value.c_str(), "nan") == 0)
	 {
            ds_MAKE_DNAN(val);
	 }
         else if (_string_to_double(value, val))  return 2; // conversion failed
         else if (val < _tlmin_d || val > _tlmax_d)   return 3;
         dmImageDataSetPixel_d(_desc, pix_pos, val);
      }
      break;
      case dmFLOAT:
      {
         float val;
         if (_strcmp_cis(value.c_str(), "nan") == 0)
	 {
            ds_MAKE_FNAN(val);
	 }
         else if (_string_to_float(value, val))  return 2; // conversion failed
         else if (val < _tlmin_f || val > _tlmax_f)   return 3;
         dmImageDataSetPixel_f(_desc, pix_pos, val);
      }
      break;
      case dmLONG:
      {
         long val;
         if ((_strcmp_cis(value.c_str(), "nan") == 0) && _tnull.length())
            val = _tnull_l;
         else if (_string_to_long(value, val))  return 2; // conversion failed
         else if (val < _tlmin_l || val > _tlmax_l)   return 3;
         dmImageDataSetPixel_l(_desc, pix_pos, val);
      }
      break;
      case dmULONG:
      {
         unsigned long val;
         if ((_strcmp_cis(value.c_str(), "nan") == 0) && _tnull.length())
            val = _tnull_ul;
         else if (_string_to_ulong(value, val))  return 2; // conversion failed
         else if (val < _tlmin_ul || val > _tlmax_ul)   return 3;
         dmImageDataSetPixel_ul(_desc, pix_pos, val);
      }
      break;
      case dmSHORT:
      {
         short val;
         if ((_strcmp_cis(value.c_str(), "nan") == 0) && _tnull.length())
            val = _tnull_s;
         else if (_string_to_short(value, val))  return 2; // conversion failed
         else if (val < _tlmin_s || val > _tlmax_s)   return 3;
         dmImageDataSetPixel_s(_desc, pix_pos, val);
      }
      break;
      case dmUSHORT:
      {
         unsigned short val;
         if ((_strcmp_cis(value.c_str(), "nan") == 0) && _tnull.length())
            val = _tnull_us;
         else if (_string_to_ushort(value, val))  return 2; // conversion failed
         else if (val < _tlmin_us || val > _tlmax_us)   return 3;
         dmImageDataSetPixel_us(_desc, pix_pos, val);
      }
      break;
      case dmBYTE:
      {
	 short val;
	 unsigned char cval; 
	 if (_string_to_short(value, val)) return 2; // conversion failed
	 if (val < 0 || val > 255)  return 3;        // invalid ubyte 
	 cval = (unsigned char) val; 
         dmImageDataSetPixel_ub(_desc, pix_pos, cval);
      }
      break; 
      default:
         return 1;   // unknown or unsupported type 
      break; 
   }
   return 0; 
}

void PrismDataColumn::set_expansion_status(int val)
{
   if (val == 2 && (_expstatus == 0 || _expstatus == 3))
      _expstatus = 2;
   else if (val == 3 && _expstatus == 2)
      _expstatus = 3;
}

void PrismDataColumn::set_table_minmax_column(int minv, int maxv)
{
   if (minv > -1)
   {
      _tabmincol = minv;
      if (maxv == -1)   _tabmaxcol = minv;
         else           _tabmaxcol = maxv;
   }
}

void PrismDataColumn::set_component_type(unsigned int typ)
{
   _acomp = ((typ & PRISM_ARRAY_COMP) == PRISM_ARRAY_COMP);
   _vcomp = ((typ & PRISM_VECTOR_COMP) == PRISM_VECTOR_COMP); 
}

int PrismDataColumn::set_format(string val)
{
   if (val == "")  // reset to use default formatting 
      _format.clear();
//   else if (val.find("%") == string::npos)
//      return 1;   // invalid format entry 
   else if (!validate_format(val))
      return 1; 
   else
      _format = val; 
   return 0;
}


// return true if valid format of    %{flag}{width}{.precision}{size}conv
// where optional flag    = [-|+|0| |#]*  
//       optional width   =  integer  
//       optional precision = .  integer 
//       optional size      = [h|l|L]
//       required conv      = [d|u|o|x|X|f|e|E|g|G|c|s]
bool PrismDataColumn::validate_format(string val)
{
   char cv, buffer[256];
   int state = 0, len = val.length(); 
   char* start = &buffer[1]; 
   char *end, *period;

   if (len == 0)  return true;   // for our use- empty string is fine
   if (len == 1 || len > 255)  return false;  // too short/long 
   strcpy(buffer, val.c_str());
   if (buffer[0] != '%')  return false;  // must start w/ % 
   cv = buffer[len-1];
   buffer[len-1] = '\0'; 
   end = &buffer[len - 2]; 
   period = strchr(start, '.');
//   if (strchr("duoxXfeEgGcs", cv) == NULL) return false; // invalid conv 
   if ((_dtype == dmSHORT || _dtype == dmLONG || _dtype == dmBYTE || 
        _dtype == dmUSHORT || _dtype == dmULONG || _dtype == dmBIT) && 
      strchr("duoxXc", cv) == NULL) return false; 
   else if ((_dtype == dmDOUBLE || _dtype == dmFLOAT) && 
      strchr("feEgG", cv) == NULL) return false; 
   else if ((_dtype == dmDOUBLE || _dtype == dmFLOAT) && 
      strchr("feEgG", cv) == NULL) return false; 
   else if (_dtype == dmTEXT  && strchr("s", cv) == NULL) return false;

   if (len == 2) return true; // simple format- ie. %d, %g 
   if (*end == 'h' || *end == 'l' || *end == 'L') *end = '\0';  
   if (period) 
   {
      *period = '\0';
      period++; 
      while(period != end && isdigit(*period))  { *period = '\0'; period++;} 
   }
   while (*start != '\0')
   {
      if (strchr("-+#0 ", *start))
      {
         if ((!(state == 0 || state == 1)) && (*start != '0'))  return false; 
	 *start = '\0'; 
	 state = 1; 
	 start++;
      } 
      else if (isdigit(*start))
      {
	 state = 2;  
	 *start = '\0'; 
	 start++; 
      }
      else return false;  // invalid flag or field width 
   }
   return (strlen(&buffer[1]) == 0);
}



