// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PRISMDATACOLUMN_HH
#define PRISMDATACOLUMN_HH

#include <stdio.h>
#include <string>
#include "ascdm.h"

using namespace std; 

#define    PRISM_ARRAY_COMP       0x001
#define    PRISM_VECTOR_COMP      0x002 
#define    PRISM_VECTOR_ARRAY_COMP (PRISM_ARRAY_COMP|PRISM_VECTOR_COMP)

class PrismDataColumn 
{
  public:
    PrismDataColumn(); 
    PrismDataColumn(dmDescriptor *desc, int pos);
    ~PrismDataColumn();
    
    string get_name()         { return _name; }
    string get_component_name()   { return _compname; }
    string get_vector_name()  { return _vecname; }
    void   set_vector_name(string vv) { _vecname = vv; }
    string get_units()        { return _units; }
    string get_comment()      { return _comment; }
    string get_type()         { return _type; }
    string get_tlmin()        { return _tlmin; }
    string get_tlmax()        { return _tlmax; }
    string get_tnull()        { return _tnull; }
    string get_coordsys()     { return _coordsys; }
    string get_crpix()        { return _crpix; }
    string get_crval()        { return _crval; }
    string get_cdelt()        { return _cdelt; }

    string get_errmsg()       { return _errmsg; }

    long   get_imgrows()      { return _imgrows; }
    long   get_imgcols()      { return _imgcols; }
    long   get_imgnaxes()     { return _imgnaxes; }

    int    get_vector_pos()   { return _vpos; }
    int    get_pos()          { return _pos;  }
    int    get_dim()          { return _dim; }

    int    get_adim()         { return _adim; }
    int    array_min()        { return _amin; }
    int    array_pos()        { return _apos; }
    int    array_max()        { return _amax; }
    long   get_array_size()   { return _asize; }

    int    vector_min()       { return _vmin; }
    int    vector_pos()       { return _vpos; }
    int    vector_max()       { return _vmax; }

    bool   is_visible()       { return _vis; }
    bool   is_vector()        { return (_dim > 1); }
    bool   is_vector_comp()   { return _vcomp; }
    bool   is_array()         { return (_asize != 0); }
    bool   is_array_comp()    { return _acomp; }
    bool   is_vector_array()  { return (is_vector() && is_array()); }
    bool   is_vector_array_comp() 
                              { return (is_vector_comp() && is_array_comp()); }

    bool   is_image()         { return _image; }
    bool   is_virtual()       { return _virtual; }

    dmDescriptor* get_descriptor()   { return _desc; }
    dmDataType    get_datatype() { return _dtype; }

    void   set_component_name(string val)  { _compname = val; }
    void   set_name(string val)  { _name = val;}
    void   set_desc(dmDescriptor *desc, int pos);
    void   set_desc(dmDescriptor *desc, int pos, int vpos, int vmin=-1,
                    int vmax=-1, int apos=-1, int amin=-1, int amax=-1);
    void   set_image_desc(dmDescriptor *desc);
    void   set_visible(bool vis)  { _vis = vis; }
    void   set_virtual(bool val)  { _virtual = val; }

    bool   validate_format(string value); 
    int    validate_value(string value);
    int    set_value(string value);
    int    set_array_value(long pos, string value);
    int    set_image_value(long col, long row, string value);

    bool   get_expander_state()  { return _expstate; }
    void   set_expander_state(bool bb) { _expstate = bb; }

    int    get_expansion_status()     { return _expstatus; }
    void   set_expansion_status(int val); 
    long   get_table_min_column()     { return _tabmincol; }
    long   get_table_max_column()     { return _tabmaxcol; }
    void   set_table_minmax_column(int minv, int maxv = -1); 

    void   set_component_type(unsigned int typ); 

    int    set_format(string val);
    string get_format()   { return _format; }
    
  protected:

    dmDescriptor   *_desc;       // The DM key descriptors
    string         _name;        // column name
    string         _compname;    // component name for vector arrays 
    dmDataType     _dtype;        // column data type

    string         _coordsys;    // associated coordinate system
    string         _crpix;       // crpix of coord sys
    string         _crval;       // crpix of coord sys
    string         _cdelt;       // crpix of coord sys

    double         _tlmin_d;     // tlmin for doubles 
    double         _tlmax_d;     // tlmax for doubles 
    float          _tlmin_f;     // tlmin for floats
    float          _tlmax_f;     // tlmax for floats
    long           _tlmin_l;     // tlmin for longs 
    long           _tnull_l;     // tnull for longs 
    long           _tlmax_l;     // tlmax for longs 
    unsigned long  _tlmin_ul;    // tlmin for unsigned longs 
    unsigned long  _tnull_ul;    // tnull for unsigned longs 
    unsigned long  _tlmax_ul;    // tlmax for unsigned longs 
    short          _tlmin_s;     // tlmin for shorts 
    short          _tnull_s;     // tnull for shorts 
    short          _tlmax_s;     // tlmax for shorts 
    unsigned short _tlmin_us;    // tlmin for unsigned shorts 
    unsigned short _tnull_us;    // tnull for unsigned shorts 
    unsigned short _tlmax_us;    // tlmax for unsigned shorts 
    unsigned char  _tlmin_ub;    // tlmin for unsigned byte 
    unsigned char  _tnull_ub;    // tnull for unsigned byte 
    unsigned char  _tlmax_ub;    // tlmax for unsigned byte 
    unsigned char  _tlmin_bit;   // tlmin for bits 
    unsigned char  _tlmax_bit;   // tlmax for bits 

    // table column mappings 
    long           _tabmincol;   // table minimum column position
    long           _tabmaxcol;   // table maximum column position
    int           _expstatus;   //  0 = not set, 1 = inline, 2 = added, 3 = removed 


    int            _pos;         // column position
    bool           _vis;         // is visible 
    int            _dim;         // column dimension 
    long           _adim;        // array dimensions
    long           _asize;       // array size
    int            *_comps;      // indices of vector components  
    bool           _acomp;       // array component 
    bool           _vcomp;       // vector component 

    int            _apos;        // column position of array parent
    int            _amin;        // array min index 
    int            _amax;        // array max index 

    int            _vpos;        // column position of vector parent
    int            _vmin;        // vector component minimum column pos
    int            _vmax;        // vector component maximum column pos

    bool           _expstate;    // column expander needs to be reset

    bool           _image;       // is column an image descriptor 
    long           _imgrows;     // number of image rows 
    long           _imgcols;     // number of image columns 
    long           _imgnaxes;    // image array dimension

    // meta data 
    string         _vecname;     // vector name 
    string         _comment;     // column description 
    string         _type;        // column data type as a string
    string         _units;       // column units 
    string         _tlmin;       // column range minimum 
    string         _tlmax;       // column range minimum 
    string         _tnull;       // column NULL value  
    long           _numcoords;   // number of associated coord systems 
    bool           _virtual;     // is column really a virtual column?
    string         _errmsg;      // stored error message 
    string         _format;      // format string 
};


#endif 
