// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PRISMDATATABLE_HH
#define PRISMDATATABLE_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <gtkextra/gtksheet.h>
#include <iostream>
#include <string>
#include <list>

#include "PrismBaseObj.hh"
#include "PrismColumnInfraP.hh"

class PrismDataBlock; 
class PrismDataColumn; 
class PrismDataArea; 
class PrismTableColumn; 

using namespace std;

class PrismDataTable: public PrismBaseObj
{
  public:
    PrismDataTable(PrismDataArea* parent, string exname="");
    ~PrismDataTable();

    int load_entries(PrismDataBlock* blk, long row=1);
    int load_page(long page=1);
    int load_row(long row=1);
    int first_page(); 
    int prev_page();
    int next_page();
    int last_page(); 
    int reload_page()   { return load_page(curr_page); } 

    int get_valid_edit()  { return _valid_edit; }
    void set_valid_edit(int val)  { _valid_edit = val; }

    int  get_extnum(); 
    int  set_pagesize(long val);
    long get_pagesize()  { return pagesize; } 

    long get_nrows()     { return rows; }
    long get_ncols()     { return cols; }
    long get_npages()    { return numpages; }
    long get_current_page()   { return curr_page; }
    virtual int show(); 

    void  set_page_label();
    bool  is_loading()   { return _loading; }

    void select_column(int column);
    void select_all_columns();
    void deselect_all_columns(); 
    void select_row(int row);
    void select_all_rows();
    void deselect_all_rows(); 
    void set_column_bgcolor(int column); 
    void set_row_bgcolor(int row); 
    void set_cell_bgcolor(int row, int col); 
    string get_selected_rowspec(); 
    string get_selected_imgcol_spec(); 

    int  append_row(long copyrow=0);
    int  append_column(PrismColumnInfra_p col_info); 
    void expand_oversized_column(PrismTableColumn* col); 

    int  validate_cell(int col, int row, string value);
    int  edit_cell(int col, int row, string value);
    int  traverse_cell(int col, int row); 

    void launch_floating_menu(int col); 
    void toggle_column(PrismTableColumn* tp=NULL); 

    PrismDataColumn* get_selected_columns(long &cnt, bool override=false);
    PrismDataColumn* get_data_columns(long &cnt); 
    int  get_column_names(string* &colnames);

    void enable_editing(); 
    void disable_editing(); 
    void selected_columns_adjusted(); 

    int  table_to_data_column(int col); 
    PrismTableColumn* data_column_to_table_entry(int col);
    PrismTableColumn* find_root_column(PrismTableColumn* tcp);

  protected:
    int  setup_table_columns();
    gboolean  load_data_row(); 
    int  set_column_data_values(string* data, long col1, long col2, long rows); 

    static int change_cell_callback(GtkWidget *widget, gint row, gint column,
                                    gpointer data);
    static int edit_cell_callback(GtkWidget *widget, gint row, gint column,
                                   gpointer data);
    static int enter_cell_callback(GtkWidget *widget, gint row, gint column,
                                    gpointer data);
    static int column_select_callback(GtkWidget *widget, gint column,
                                       gpointer data);
    static int row_select_callback(GtkWidget *widget, gint row,
                                    gpointer data);
    static int traverse_callback(GtkWidget *widget, gint row,gint col, 
                                 gint *new_row, gint *new_col, gpointer data);
    static void toggle_column_callback(GObject *object, gpointer user_data);
    static int floating_menu_callback(GtkWidget* widget, gint column, 
                                       gpointer data);
    static int idle_callback(gpointer data); 

    GdkColor _select_bg; 
    GdkColor _select_fg; 
    // since the gtksheet doesn't support odd/even row theme hinting, set up
    // a workaround manner using the altrow flag to indicate whether or not to
    // do odd/even row coloring. 
    GdkColor _oddrow_bg;
    GdkColor _evenrow_bg;
    bool     _altrow; 

    list <PrismTableColumn *> _collist; 
    GtkWidget* _floatmenu;     // expand/collapse popup menu
    GtkWidget* _fm_option;     // expand/collapse popup menu option
    GtkWidget* sheet; 
    PrismDataArea *_parent; 
    PrismDataBlock *_block; 
    PrismTableColumn *_expcol; // table column for expansion/collapse 
    long     rows;
    long     cols; 
    long     pagesize;
    long     numpages; 
    long     curr_page; 
    long     curr_row;
    long     _disp_rows;       // number of rows displayed 

    long     _selpps;          // number of row selection pages 
    long     _selrpp;          // number of rows per select page 
    int      _selectMode;      // default selection mode 
    int**    _selected_rows;   // selected rows per page 
    int*     _selected_cols;   // selected cols 
    int      _selcval;         // value of next selected column (used to
                               // determine order of selected columns)
    gulong   _sid1;            // signal handler id  
    gulong   _sid2;            // signal handler id  
    guint    _idletag;         // tag to remove

    long     _load_curr_row;   // track idle row loading 
    long     _load_last_row;   // last row for loading  
    string*  _data;            // data for table 
    int      _valid_edit;      

    string   _edstring;        // 'column_name:row = value' label 
    string   _oldcellval;      // old cell value before edit  
    bool     _loading;         // are we in a loading state
    bool     _editable;        // are we editable 
};

#endif 
