// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismDataToolBar.hh"
#include "PrismDataArea.hh"
#include "PrismApp.hh"

using namespace std;

PrismDataToolBar::PrismDataToolBar(PrismDataArea* parent):PrismBaseToolBar()
{
   _parent = parent; 
   int pos = 0; 
   GtkTooltips* tips = NULL;
    
   if (theApp)  tips = theApp->get_tips_group(); 
      else      tips = gtk_tooltips_new();

   if (editBtn = gtk_tool_item_new())
   {
      // Add an Edit Pulldown menu w/ options such as 'Append Row' 
      gtk_tool_item_set_homogeneous(editBtn, true); 
      menu  = gtk_menu_new(); 
      editMenu = gtk_menu_item_new_with_label("Edit");
      menuItem   = gtk_menu_item_new_with_label("Append Row");
      menuItem2   = gtk_menu_item_new_with_label("Append Column");
      menuItem3   = gtk_menu_item_new_with_label("Format Display Columns");
      mbar = gtk_menu_bar_new();
      gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem); 
      gtk_widget_show(menuItem); 
      gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem2); 
      gtk_widget_show(menuItem2); 
      gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItem3); 
      gtk_widget_show(menuItem3); 
      gtk_menu_item_set_submenu(GTK_MENU_ITEM(editMenu), menu);
      gtk_widget_show(editMenu); 
      gtk_menu_bar_append(GTK_MENU_BAR(mbar), editMenu);
      gtk_container_add(GTK_CONTAINER(editBtn), mbar);
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), editBtn, pos++);
      g_signal_connect(G_OBJECT (menuItem), "activate",
                       G_CALLBACK(append_row_callback), (gpointer)_parent);
      g_signal_connect(G_OBJECT (menuItem2), "activate",
                       G_CALLBACK(append_column_callback), (gpointer)_parent);
      g_signal_connect(G_OBJECT (menuItem3), "activate",
                       G_CALLBACK(format_column_callback), (gpointer)_parent);
   }
   if (selectBtn = gtk_tool_item_new())
   {
      gtk_tool_item_set_homogeneous(selectBtn, true); 
      // Add a Select Pulldown menu w/ options such as 'select all rows'
      smenu  = gtk_menu_new(); 
      selMenu = gtk_menu_item_new_with_label("Select");
      selAllCol = gtk_menu_item_new_with_label("Select All Columns");
      selNoCol = gtk_menu_item_new_with_label("Deselect All Columns");
      selAllRow = gtk_menu_item_new_with_label("Select All Rows");
      selNoRow = gtk_menu_item_new_with_label("Deselect All Rows");
      smbar = gtk_menu_bar_new();
      gtk_menu_shell_append(GTK_MENU_SHELL(smenu), selAllCol); 
      gtk_widget_show(selAllCol); 
      gtk_menu_shell_append(GTK_MENU_SHELL(smenu), selAllRow); 
      gtk_widget_show(selAllRow); 
      gtk_menu_shell_append(GTK_MENU_SHELL(smenu), selNoCol); 
      gtk_widget_show(selNoCol); 
      gtk_menu_shell_append(GTK_MENU_SHELL(smenu), selNoRow); 
      gtk_widget_show(selNoRow); 
      gtk_menu_item_set_submenu(GTK_MENU_ITEM(selMenu), smenu);
      gtk_widget_show(selMenu); 
      gtk_menu_bar_append(GTK_MENU_BAR(smbar), selMenu);
      gtk_container_add(GTK_CONTAINER(selectBtn), smbar);
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), selectBtn, pos++);
      g_signal_connect(G_OBJECT (selAllCol), "activate",
                       G_CALLBACK(select_column_callback), (gpointer)_parent);
      g_signal_connect(G_OBJECT (selNoCol), "activate",
                       G_CALLBACK(deselect_column_callback), (gpointer)_parent);
      g_signal_connect(G_OBJECT (selAllRow), "activate",
                       G_CALLBACK(select_row_callback), (gpointer)_parent);
      g_signal_connect(G_OBJECT (selNoRow), "activate",
                       G_CALLBACK(deselect_row_callback), (gpointer)_parent);
   }
   if (plotBtn = gtk_menu_tool_button_new(NULL, "Plot"))
   {
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), plotBtn, pos++); 
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(plotBtn), GTK_TOOLTIPS(tips),
         "Generate Plot of selected/specified table columns", "plot the data");
      g_signal_connect(plotBtn, "clicked", G_CALLBACK(plot_button_callback), 
         (void*)_parent);
      gtk_tool_item_set_homogeneous(plotBtn, true); 

      pmenu  = gtk_menu_new(); 
      pmenu1 = gtk_menu_item_new_with_label("Quick Plot");
      pmenu2 = gtk_menu_item_new_with_label("Interactive Plot");
      pmenu3 = gtk_menu_item_new_with_label("Print Current Plot");
      gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), pmenu1); 
      gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), pmenu2); 
      gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), pmenu3); 
      gtk_widget_show(pmenu1); 
      gtk_widget_show(pmenu2); 
      gtk_widget_show(pmenu3); 
      g_signal_connect(G_OBJECT(pmenu1), "activate",
                       G_CALLBACK(qplot_button_callback), (gpointer)_parent);
      g_signal_connect(G_OBJECT(pmenu2), "activate",
                       G_CALLBACK(iplot_button_callback), (gpointer)_parent);
      g_signal_connect(G_OBJECT(pmenu3), "activate",
                       G_CALLBACK(print_button_callback), (gpointer)_parent);
      gtk_menu_tool_button_set_menu(GTK_MENU_TOOL_BUTTON(plotBtn), pmenu); 
   }
   if (histogramBtn = gtk_menu_tool_button_new(NULL, "Histogram"))
   {
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), histogramBtn, pos++); 
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(histogramBtn), GTK_TOOLTIPS(tips),
         "Generate Histogram of selected/specified table column", "histogram the data");
      g_signal_connect(histogramBtn, "clicked", G_CALLBACK(histo_button_callback), 
         (void*)_parent);
      gtk_tool_item_set_homogeneous(histogramBtn, true); 

      qmenu  = gtk_menu_new(); 
      qmenu1 = gtk_menu_item_new_with_label("Quick Histogram");
      qmenu2 = gtk_menu_item_new_with_label("Interactive Histogram");
      qmenu3 = gtk_menu_item_new_with_label("Print Current Histogram");
      gtk_menu_shell_append(GTK_MENU_SHELL(qmenu), qmenu1); 
      gtk_menu_shell_append(GTK_MENU_SHELL(qmenu), qmenu2); 
      gtk_menu_shell_append(GTK_MENU_SHELL(qmenu), qmenu3); 
      gtk_widget_show(qmenu1); 
      gtk_widget_show(qmenu2); 
      gtk_widget_show(qmenu3); 
      g_signal_connect(G_OBJECT(qmenu1), "activate",
                       G_CALLBACK(qhist_button_callback), (gpointer)_parent);
      g_signal_connect(G_OBJECT(qmenu2), "activate",
                       G_CALLBACK(ihist_button_callback), (gpointer)_parent);
      g_signal_connect(G_OBJECT(qmenu3), "activate",
                       G_CALLBACK(print_button_callback), (gpointer)_parent);
      gtk_menu_tool_button_set_menu(GTK_MENU_TOOL_BUTTON(histogramBtn), qmenu); 
   }
   if (imageBtn = gtk_menu_tool_button_new(NULL, "Image"))
   {
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), imageBtn, pos++); 
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(imageBtn), GTK_TOOLTIPS(tips),
         "Display image of bin selected table columns or image data", "image the data");
      g_signal_connect(imageBtn, "clicked", G_CALLBACK(image_button_callback), 
         (void*)_parent);
      gtk_tool_item_set_homogeneous(imageBtn, true); 

      imenu  = gtk_menu_new(); 
      imenu1 = gtk_menu_item_new_with_label("Quick Image");
      imenu2 = gtk_menu_item_new_with_label("Interactive Image");
      gtk_menu_shell_append(GTK_MENU_SHELL(imenu), imenu1); 
      gtk_menu_shell_append(GTK_MENU_SHELL(imenu), imenu2); 
      gtk_widget_show_all(imenu);
      g_signal_connect(G_OBJECT(imenu1), "activate",
                       G_CALLBACK(qimage_button_callback), (gpointer)_parent);
      g_signal_connect(G_OBJECT(imenu2), "activate",
                       G_CALLBACK(iimage_button_callback), (gpointer)_parent);
      gtk_menu_tool_button_set_menu(GTK_MENU_TOOL_BUTTON(imageBtn), imenu); 
   }

   if (metaBtn = gtk_tool_button_new(NULL, "Info"))
   {
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), metaBtn, pos++); 
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(metaBtn), GTK_TOOLTIPS(tips),
         "Display meta-data for selected columns", "column meta data");
      g_signal_connect(metaBtn, "clicked", G_CALLBACK(meta_button_callback), 
         (void*)_parent);
      gtk_tool_item_set_homogeneous(metaBtn, true); 
   }
#ifdef WLM_IMPLEMENTED
   if (sortBtn = gtk_tool_button_new(NULL, "Sort"))
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), sortBtn, pos++); 
   if (statBtn = gtk_tool_button_new(NULL, "Stat"))
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), statBtn, pos++); 
#endif
   sep = gtk_separator_tool_item_new();
   gtk_separator_tool_item_set_draw(GTK_SEPARATOR_TOOL_ITEM(sep), false);
   gtk_tool_item_set_expand(sep, true);
   gtk_toolbar_insert(GTK_TOOLBAR(base_w), sep, pos++); 
   
   GtkWidget *img;
   if (backBtn = gtk_tool_button_new(NULL, "Back")) 
   {
      img = gtk_image_new_from_stock(GTK_STOCK_GOTO_FIRST, GTK_ICON_SIZE_MENU);
      gtk_tool_button_set_label_widget(GTK_TOOL_BUTTON(backBtn),img);
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), backBtn, pos++); 
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(backBtn), GTK_TOOLTIPS(tips),
         "Move to beginning of table", "Moves to the first row of a data file");
      g_signal_connect(backBtn, "clicked", G_CALLBACK(first_button_callback), 
         (void*)_parent);
      gtk_tool_item_set_homogeneous(backBtn, false); 
   }
   if (prevBtn = gtk_tool_button_new(NULL, "Prev"))
   {
      img = gtk_image_new_from_stock(GTK_STOCK_GO_BACK, GTK_ICON_SIZE_MENU);
      gtk_tool_button_set_label_widget(GTK_TOOL_BUTTON(prevBtn),img);
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), prevBtn, pos++); 
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(prevBtn), GTK_TOOLTIPS(tips),
         "Move backward one page of data", "Move to previous page of data");
      g_signal_connect(prevBtn, "clicked", G_CALLBACK(prev_button_callback), 
         (void*)_parent);
      gtk_tool_item_set_homogeneous(prevBtn, false); 
   }
   if (jumpBtn = gtk_tool_button_new(NULL, "Jump"))
   {
      img = gtk_image_new_from_stock(GTK_STOCK_JUMP_TO, GTK_ICON_SIZE_MENU);
      gtk_tool_button_set_label_widget(GTK_TOOL_BUTTON(jumpBtn),img);
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), jumpBtn, pos++); 
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(jumpBtn), GTK_TOOLTIPS(tips),
         "Jump to page containing specified row", "Jump to page containing row");
      g_signal_connect(jumpBtn, "clicked", G_CALLBACK(jump_button_callback), 
         (void*)_parent);
      gtk_tool_item_set_homogeneous(jumpBtn, false); 
   }
   if (nextBtn = gtk_tool_button_new(NULL, "forward"))
   {
      img = gtk_image_new_from_stock(GTK_STOCK_GO_FORWARD, GTK_ICON_SIZE_MENU);
      gtk_tool_button_set_label_widget(GTK_TOOL_BUTTON(nextBtn),img);
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), nextBtn, pos++); 
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(nextBtn), GTK_TOOLTIPS(tips),
         "Move forward one page of data", "Move to next page of data");
      g_signal_connect(nextBtn, "clicked", G_CALLBACK(next_button_callback), 
         (void*)_parent);
      gtk_tool_item_set_homogeneous(nextBtn, false); 
   }
   if (frontBtn = gtk_tool_button_new(NULL, "Last"))
   {
      img = gtk_image_new_from_stock(GTK_STOCK_GOTO_LAST, GTK_ICON_SIZE_MENU);
      gtk_tool_button_set_label_widget(GTK_TOOL_BUTTON(frontBtn),img);
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), frontBtn, pos++); 
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(frontBtn), GTK_TOOLTIPS(tips),
         "Move to end of table", "Move to last row of a data file");
      g_signal_connect(frontBtn, "clicked", G_CALLBACK(last_button_callback), 
         (void*)_parent);
      gtk_tool_item_set_homogeneous(frontBtn, false); 
   }
}

PrismDataToolBar::~PrismDataToolBar()
{
   if (editBtn)  
   {   
      delete editBtn;  
      editBtn = NULL; 
   }
   if (imageBtn)  
   {   
      delete imageBtn;  
      imageBtn = NULL; 
   }
   if (plotBtn)  
   {   
      delete plotBtn;  
      plotBtn = NULL; 
   }
   if (histogramBtn)  
   {   
      delete histogramBtn;  
      histogramBtn = NULL; 
   }
   if (metaBtn)  
   {   
      delete metaBtn;  
      metaBtn = NULL; 
   }
   if (selectBtn)  
   {   
      delete selectBtn;  
      selectBtn = NULL; 
   }
#ifdef WLM_IMPLEMENTED
   if (sortBtn)  
   {   
      delete sortBtn;  
      sortBtn = NULL; 
   }
   if (statBtn)  
   {   
      delete statBtn;  
      statBtn = NULL; 
   }
#endif 
   if (backBtn)
   {
      delete backBtn;
      backBtn = NULL;
   }
   if (prevBtn)
   {
      delete prevBtn;
      prevBtn = NULL;
   }
   if (jumpBtn)
   {
      delete jumpBtn;
      jumpBtn = NULL;
   }
   if (nextBtn)
   {
      delete nextBtn;
      nextBtn = NULL;
   }
   if (frontBtn)
   {
      delete frontBtn;
      frontBtn = NULL;
   }
   if (menuItem)
   {
      delete menuItem;
      menuItem = NULL;
   }
   if (menuItem2)
   {
      delete menuItem2;
      menuItem2 = NULL;
   }
   if (menuItem3)
   {
      delete menuItem3;
      menuItem3 = NULL;
   }
   if (mbar)
   {
      delete mbar;
      mbar = NULL;
   }
   if (editMenu)
   {
      delete editMenu;
      editMenu = NULL;
   }
   if (menu)
   {
      delete menu;
      menu = NULL;
   }
   if (smbar)
   {
      delete smbar;
      smbar = NULL;
   }
   if (selMenu)
   {
      delete selMenu;
      selMenu = NULL;
   }
   if (selAllCol)
   {
      delete selAllCol;
      selAllCol = NULL;
   }
   if (selNoCol)
   {
      delete selNoCol;
      selNoCol = NULL;
   }
   if (selNoRow)
   {
      delete selNoCol;
      selAllCol = NULL;
   }
   if (selAllCol)
   {
      delete selAllCol;
      selAllCol = NULL;
   }
}


int PrismDataToolBar::enable_all(int flag)
{
   bool image = TRUE, plot = TRUE; 
   if (theApp)
   {
      image = theApp->can_image();
      plot = theApp->can_plot();
   }
   if (editBtn)       gtk_widget_set_sensitive(GTK_WIDGET(editBtn), TRUE); 
   if (imageBtn)      gtk_widget_set_sensitive(GTK_WIDGET(imageBtn), image); 
   if (selectBtn)     gtk_widget_set_sensitive(GTK_WIDGET(selectBtn), TRUE); 
   if (metaBtn)       gtk_widget_set_sensitive(GTK_WIDGET(metaBtn), 
      (flag & PRISM_DATA_INFO_BTN == PRISM_DATA_INFO_BTN)); 
#ifdef WLM_IMPLEMENTED
   if (sortBtn)       gtk_widget_set_sensitive(GTK_WIDGET(sortBtn), TRUE); 
   if (statBtn)       gtk_widget_set_sensitive(GTK_WIDGET(statBtn), TRUE); 
#endif
   img_flag = (flag & PRISM_DATA_IMAGE_BTN == PRISM_DATA_IMAGE_BTN);
   if (menuItem)      gtk_widget_set_sensitive(GTK_WIDGET(menuItem),!img_flag);
   if (menuItem2)     gtk_widget_set_sensitive(GTK_WIDGET(menuItem2),!img_flag);
   if (menuItem3)     gtk_widget_set_sensitive(GTK_WIDGET(menuItem3),TRUE);
   plot = (plot && !img_flag); 
   if (plotBtn)       gtk_widget_set_sensitive(GTK_WIDGET(plotBtn), plot); 
   if (histogramBtn)  gtk_widget_set_sensitive(GTK_WIDGET(histogramBtn), plot); 
   return 0;
}


int PrismDataToolBar::disable_all()
{
   if (editBtn)       gtk_widget_set_sensitive(GTK_WIDGET(editBtn), FALSE); 
   if (imageBtn)      gtk_widget_set_sensitive(GTK_WIDGET(imageBtn), FALSE); 
   if (plotBtn)       gtk_widget_set_sensitive(GTK_WIDGET(plotBtn), FALSE); 
   if (histogramBtn)  gtk_widget_set_sensitive(GTK_WIDGET(histogramBtn), FALSE); 
   if (selectBtn)     gtk_widget_set_sensitive(GTK_WIDGET(selectBtn), FALSE); 
   if (metaBtn)       gtk_widget_set_sensitive(GTK_WIDGET(metaBtn), FALSE); 
#ifdef WLM_IMPLEMENTED
   if (sortBtn)       gtk_widget_set_sensitive(GTK_WIDGET(sortBtn), FALSE); 
   if (statBtn)       gtk_widget_set_sensitive(GTK_WIDGET(statBtn), FALSE); 
#endif
   if (menuItem)      gtk_widget_set_sensitive(GTK_WIDGET(menuItem), FALSE);
   if (menuItem2)     gtk_widget_set_sensitive(GTK_WIDGET(menuItem2), FALSE);
   if (menuItem3)     gtk_widget_set_sensitive(GTK_WIDGET(menuItem3), FALSE);
   return 0;
}

int PrismDataToolBar::set_nav_button_sensitivity(long cp, long np)
{
   if (np < 2)
   {
      gtk_widget_set_sensitive(GTK_WIDGET(backBtn), FALSE);
      gtk_widget_set_sensitive(GTK_WIDGET(prevBtn), FALSE);
      gtk_widget_set_sensitive(GTK_WIDGET(jumpBtn), FALSE);
      gtk_widget_set_sensitive(GTK_WIDGET(nextBtn), FALSE);
      gtk_widget_set_sensitive(GTK_WIDGET(frontBtn), FALSE);
   }
   else 
   {
      if (cp == 1)
      { 
         gtk_widget_set_sensitive(GTK_WIDGET(backBtn), FALSE);
         gtk_widget_set_sensitive(GTK_WIDGET(prevBtn), FALSE);
         gtk_widget_set_sensitive(GTK_WIDGET(jumpBtn), TRUE);
         gtk_widget_set_sensitive(GTK_WIDGET(nextBtn), TRUE);
         gtk_widget_set_sensitive(GTK_WIDGET(frontBtn), TRUE);
      }
      else if (cp == np)
      {
         gtk_widget_set_sensitive(GTK_WIDGET(backBtn), TRUE);
         gtk_widget_set_sensitive(GTK_WIDGET(prevBtn), TRUE);
         gtk_widget_set_sensitive(GTK_WIDGET(jumpBtn), TRUE);
         gtk_widget_set_sensitive(GTK_WIDGET(nextBtn), FALSE);
         gtk_widget_set_sensitive(GTK_WIDGET(frontBtn), FALSE);
      }
      else
      {
         gtk_widget_set_sensitive(GTK_WIDGET(backBtn), TRUE);
         gtk_widget_set_sensitive(GTK_WIDGET(prevBtn), TRUE);
         gtk_widget_set_sensitive(GTK_WIDGET(jumpBtn), TRUE);
         gtk_widget_set_sensitive(GTK_WIDGET(nextBtn), TRUE);
         gtk_widget_set_sensitive(GTK_WIDGET(frontBtn), TRUE);
      }
   }
   return 0; 
}

void PrismDataToolBar::set_plot_button_sensitivity(bool val)
{
   gtk_widget_set_sensitive(GTK_WIDGET(plotBtn), (val && theApp->can_plot()));
}

void PrismDataToolBar::set_hist_button_sensitivity(bool val)
{
   gtk_widget_set_sensitive(GTK_WIDGET(histogramBtn), 
      (val && theApp->can_plot()));
}

int PrismDataToolBar::show()
{
   gtk_widget_show_all(base_w);
   return 0;
}



void PrismDataToolBar::next_button_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area) area->next_page(); 
}

void PrismDataToolBar::prev_button_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area) area->prev_page(); 
}

void PrismDataToolBar::first_button_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area) area->first_page(); 
}

void PrismDataToolBar::last_button_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area) area->last_page(); 
}

void PrismDataToolBar::meta_button_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->get_meta_data();
}

void PrismDataToolBar::jump_button_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->goto_page();
}

void PrismDataToolBar::image_button_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->generate_image(); 
}

void PrismDataToolBar::plot_button_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->generate_plot(); 
}

void PrismDataToolBar::histo_button_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->generate_histogram(); 
}

int PrismDataToolBar::append_row_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->append_row(); 
   return 0; 
}

int PrismDataToolBar::append_column_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->append_column(); 
   return 0; 
}

int PrismDataToolBar::format_column_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->format_column(); 
   return 0; 
}


int PrismDataToolBar::select_column_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->set_selection(-1, 1); 
   return 0; 
}

int PrismDataToolBar::deselect_column_callback(GObject *object,gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->set_selection(-2, 1); 
   return 0; 
}

int PrismDataToolBar::select_row_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->set_selection(-1, 0); 
   return 0; 
}

int PrismDataToolBar::deselect_row_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->set_selection(-2, 0); 
   return 0; 
}

void PrismDataToolBar::enable_editing()
{
//   if (editBtn)   gtk_widget_set_sensitive(GTK_WIDGET(editBtn), !img_flag); 
   if (menuItem)  gtk_widget_set_sensitive(GTK_WIDGET(menuItem), !img_flag);
   if (menuItem2) gtk_widget_set_sensitive(GTK_WIDGET(menuItem2), !img_flag);
}

void PrismDataToolBar::disable_editing()
{
//   if (editBtn)   gtk_widget_set_sensitive(GTK_WIDGET(editBtn), FALSE);
   if (menuItem)  gtk_widget_set_sensitive(GTK_WIDGET(menuItem), FALSE);
   if (menuItem2) gtk_widget_set_sensitive(GTK_WIDGET(menuItem2), FALSE);
}

void PrismDataToolBar::set_info_button_sensitivity(bool val)
{
   gtk_widget_set_sensitive(GTK_WIDGET(metaBtn), val);
}

void PrismDataToolBar::set_image_button_sensitivity(bool val)
{
   gtk_widget_set_sensitive(GTK_WIDGET(imageBtn), val);
}

void PrismDataToolBar::iplot_button_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->generate_plot(1); 
}

void PrismDataToolBar::qplot_button_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->generate_plot(0); 
}

void PrismDataToolBar::ihist_button_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->generate_histogram(1); 
}

void PrismDataToolBar::qhist_button_callback(GObject *object, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->generate_histogram(0); 
}

void PrismDataToolBar::iimage_button_callback(GObject*, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->generate_image(1); 
}

void PrismDataToolBar::qimage_button_callback(GObject*, gpointer user_data)
{
   PrismDataArea *area = (PrismDataArea*)user_data;
   if (area)  area->generate_image(0); 
}


void PrismDataToolBar::print_button_callback(GObject *object, gpointer user_data)
{
   theApp->print_current_plot();
}


