// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PRISMDATATOOLBAR_HH
#define PRISMDATATOOLBAR_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>

#include "PrismBaseToolBar.hh"

#define PRISM_DATA_IMAGE_BTN       0x02
#define PRISM_DATA_INFO_BTN        0x04
#define PRISM_DATA_PLOT_BTN        0x08
#define PRISM_DATA_HIST_BTN        0x10


class PrismDataArea; 

using namespace std;

class PrismDataToolBar: public PrismBaseToolBar 
{
  public:
   PrismDataToolBar(PrismDataArea* parent);
   ~PrismDataToolBar();

   virtual int enable_all(int flag=0);
   virtual int disable_all();
   virtual int show(); 

   int set_nav_button_sensitivity(long cp, long np); 
   void set_plot_button_sensitivity(bool val); 
   void set_hist_button_sensitivity(bool val); 
   void set_info_button_sensitivity(bool val); 
   void set_image_button_sensitivity(bool val); 
   void disable_editing(); 
   void enable_editing(); 

  protected:
   static void first_button_callback(GObject *object, gpointer user_data);
   static void prev_button_callback(GObject *object, gpointer user_data);
   static void jump_button_callback(GObject *object, gpointer user_data);
   static void next_button_callback(GObject *object, gpointer user_data);
   static void last_button_callback(GObject *object, gpointer user_data);
   static void meta_button_callback(GObject *object, gpointer user_data);
   static void image_button_callback(GObject *object, gpointer user_data);
   static void plot_button_callback(GObject *object, gpointer user_data);
   static void histo_button_callback(GObject *object, gpointer user_data);
   static int  append_row_callback(GObject *object, gpointer user_data);
   static int  append_column_callback(GObject *object, gpointer user_data);
   static int  format_column_callback(GObject *object, gpointer user_data);
   static int  select_row_callback(GObject *object, gpointer user_data);
   static int  select_column_callback(GObject *object, gpointer user_data);
   static int  deselect_row_callback(GObject *object, gpointer user_data);
   static int  deselect_column_callback(GObject *object, gpointer user_data);
   
   static void iplot_button_callback(GObject *object, gpointer user_data);
   static void qplot_button_callback(GObject *object, gpointer user_data);
   static void print_button_callback(GObject *object, gpointer user_data);
   static void ihist_button_callback(GObject *object, gpointer user_data);
   static void qhist_button_callback(GObject *object, gpointer user_data);
   static void iimage_button_callback(GObject *object, gpointer user_data);
   static void qimage_button_callback(GObject *object, gpointer user_data);

   // Data Interaction Tool Buttons
   GtkToolItem *editBtn;
   GtkToolItem *imageBtn;
   GtkToolItem *plotBtn;
   GtkToolItem *histogramBtn;
   GtkToolItem *selectBtn;
   GtkToolItem *metaBtn;
   GtkToolItem *sortBtn;
   GtkToolItem *statBtn;

   // Navigation Tool Buttons
   GtkToolItem* sep;              // separator 
   GtkToolItem *backBtn;          // first row 
   GtkToolItem *prevBtn;          // previous page
   GtkToolItem *jumpBtn;          // goto row 
   GtkToolItem *nextBtn;          // next page 
   GtkToolItem *frontBtn;         // last row 

   // Edit Menu items 
   GtkWidget* menu;               // Menu
   GtkWidget* editMenu;           // Edit Menu
   GtkWidget* mbar;               // menu bar 
   GtkWidget* menuItem;           // menu item (Append Row) 
   GtkWidget* menuItem2;          // menu item (Append Column) 
   GtkWidget* menuItem3;          // menu item (Format Display Columns)

   // Select Menu items           
   GtkWidget* smenu;              // Menu
   GtkWidget* selMenu;            // select menu
   GtkWidget* smbar;              // select menu bar
   GtkWidget* selAllCol;          // menu item (Select All columns)  
   GtkWidget* selAllRow;          // menu item (Select All rows)  
   GtkWidget* selNoCol;           // menu item (Deselect All columns)  
   GtkWidget* selNoRow;           // menu item (Deselect All rows)  

   GtkToolItem *printBtn;
   GtkToolItem *qplotBtn;
   GtkToolItem *qhistBtn;
   GtkToolItem *qimageBtn;

   GtkWidget* pmenu; 
   GtkWidget* pmenu1; 
   GtkWidget* pmenu2; 
   GtkWidget* pmenu3; 
   GtkWidget* qmenu; 
   GtkWidget* qmenu1; 
   GtkWidget* qmenu2; 
   GtkWidget* qmenu3; 
   GtkWidget* imenu;
   GtkWidget* imenu1;
   GtkWidget* imenu2;

   bool       img_flag;           // disabled for images 

   PrismDataArea* _parent;        // parent data area

};

#endif 
