// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismExtnTable.hh"
#include "PrismFileFrame.hh"

using namespace std;

PrismExtnTable::PrismExtnTable(PrismFileFrame* parent):PrismBaseObj()
{
   GtkCellRenderer *renderer;
   GtkTreeSelection *select;
   model = gtk_list_store_new(NUM_COLUMNS, G_TYPE_STRING, G_TYPE_STRING,
                              G_TYPE_STRING);
   
   treeview = GTK_TREE_VIEW (gtk_tree_view_new_with_model (GTK_TREE_MODEL(model)));
   table = GTK_WIDGET(treeview);
   g_object_unref (model);

   gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(treeview) ,1);
   GtkTreeViewColumn* tvcol = gtk_tree_view_column_new();

   renderer = gtk_cell_renderer_text_new ();
   tvcol = gtk_tree_view_column_new_with_attributes(
                               "Extension", renderer, "text", KEY_NAME, NULL);
   gtk_tree_view_append_column (treeview, tvcol);
   gtk_tree_view_column_set_resizable(tvcol, true);
   gtk_tree_view_column_set_reorderable(tvcol, true);

   renderer = gtk_cell_renderer_text_new ();
   tvcol = gtk_tree_view_column_new_with_attributes(
                               "Type", renderer, "text", KEY_TYPE, NULL);
   gtk_tree_view_append_column (treeview, tvcol);
   gtk_tree_view_column_set_resizable(tvcol, true);
   gtk_tree_view_column_set_reorderable(tvcol, true);
   renderer = gtk_cell_renderer_text_new ();
   tvcol = gtk_tree_view_column_new_with_attributes(
                               "Dimensions", renderer, "text", KEY_DIM, NULL);
   gtk_tree_view_append_column (treeview, tvcol);
   gtk_tree_view_column_set_resizable(tvcol, true);
   gtk_tree_view_column_set_reorderable(tvcol, true);

   base_w = gtk_scrolled_window_new (NULL,NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(base_w),
                                  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

   _parent = parent; 
   select = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
   gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
   g_signal_connect(G_OBJECT (select), "changed", G_CALLBACK(PrismExtnTable::select_entry), _parent);

   gtk_container_add(GTK_CONTAINER(base_w), table);
}

PrismExtnTable::~PrismExtnTable()
{
   
}

int PrismExtnTable::load_entries(list<ExtensionInfo_t> exts, int extnum)
{
   list<ExtensionInfo_t>::iterator ii=exts.begin();
   GtkTreeIter defiter; 
   int cc = 1, defext = 1; 
   if (extnum != -1) defext = extnum;
      else if (exts.size() > 1) defext = 2; 

   gtk_list_store_clear(model);  // remove existing entries 
   for(; ii !=exts.end(); ++ii)
   {
      gtk_list_store_append(model, &iter);
      gtk_list_store_set (model, &iter,
                          KEY_NAME, (*ii).name.c_str(),
		          KEY_TYPE, (*ii).type.c_str(),
		          KEY_DIM, (*ii).dim.c_str(),
		          -1);
      if (cc++ == defext)   defiter = iter;
   }

   // for now, initialize the 'selected' extension to the second extension
   // (first extension if only 1 exists) 
   GtkTreeSelection *select = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
   gtk_tree_selection_select_iter(select, &defiter);
   return 0; 
}

int PrismExtnTable::show()
{
   gtk_widget_show_all(GTK_WIDGET(treeview));
   return 0;
}

void PrismExtnTable::select_entry(GtkWidget *widget, gpointer data)
{
   // this method is only called when the selected extension changes in
   // the widget. Clicking on the selected widget will not activate this
   // callback. 
   GtkTreeIter itr;
   GtkTreeModel *modl;
   gchar *ext;
   PrismFileFrame* frame = (PrismFileFrame*)data; 

   if (gtk_tree_selection_get_selected (GTK_TREE_SELECTION(widget), &modl, &itr))
   {
      gtk_tree_model_get (modl, &itr, KEY_NAME, &ext, -1);
      if (ext && frame) 
      {
	 string ename = ext;
         frame->open_extension(ename); 
      }
      g_free (ext);
   }
}



