// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PRISMFILEFRAME_HH
#define PRISMFILEFRAME_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>
#include <string>

#include "PrismBaseObj.hh"

using namespace std;

class PrismDataArea; 
class PrismDataSet; 
class PrismHeaderArea; 
class PrismExtnTable;

class PrismFileFrame: public PrismBaseObj
{
  public:
    PrismFileFrame(string filename="");
    virtual ~PrismFileFrame();

    virtual int show(); 

    int  load_file(string filename);
    bool loaded()   { return _loaded; }

    int  open_extension(string ename);
    int  open_extension(int num, long row); 

    int  visualize(int type, int dialog=1); 
    int  select_entry(int val, int as_col); 

    void show_frame_title() 
       { gtk_frame_set_label(GTK_FRAME(base_w), frame_title.c_str());}
    void hide_frame_title() 
       { gtk_frame_set_label(GTK_FRAME(base_w), NULL);}
    void set_frame_title(string val);

    static void lock_button_callback(GObject *object, gpointer user_data);
    void set_lock_mode(int md=-1);
    int  get_lock_mode()    { return _currmode; }
    void set_modified(); 

    GtkWidget*   get_icon(); 
    void set_tab_icon(string fname); 
    void set_lock_sensitivity(); 

    string get_filename(int filter=0); 

  protected:
    // class members
    GtkWidget        *vpane;
    GtkWidget        *hpane;
    PrismDataSet     *_data; 
    PrismDataArea    *dataArea; 
    PrismHeaderArea  *headerArea; 
    PrismExtnTable   *extnTable; 
    string frame_title;
    string iconfile_path; 
    bool             _editlock; 
    bool             _loaded; 
    bool             _changed;     // has file been modified 
    int              _currmode;    // current edit mode 
    GtkWidget        *tabdisplay;  // tab image and filename 
    GtkWidget        *tablab;      // tab label
    GtkWidget        *tabbtn;      // tag image 
    GtkWidget        *deficon;     // default - no file icon 
    GtkWidget        *locked;      // locked icon 
    GtkWidget        *lockro;      // read only icon  
    GtkWidget        *edit;        // editable icon  
    GtkWidget        *edited;      // changes made icon  
};

#endif 
