// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismFileList.hh"
#include "PrismFileFrame.hh"
#include "PrismApp.hh"


using namespace std;

PrismFileList::PrismFileList():PrismBaseObj()
{
   base_w = gtk_notebook_new ();
   gtk_notebook_set_tab_pos (GTK_NOTEBOOK(base_w), tab_pos = GTK_POS_TOP);
   gtk_notebook_popup_enable(GTK_NOTEBOOK(base_w));
   gtk_notebook_set_scrollable(GTK_NOTEBOOK(base_w), GTK_POLICY_AUTOMATIC);
   frames.clear();
   tabs_on = true; 
}

PrismFileList::~PrismFileList()
{
  map<GtkWidget*, PrismFileFrame*>::iterator mi;
  for (mi= frames.begin(); mi != frames.end(); mi++)
     delete (*mi).second; 
  frames.clear(); 
}

int PrismFileList::add_file(string filename, int new_tab)
{
   PrismFileFrame* ff = NULL;
   GtkWidget* bw = NULL;
   int sts = 1; 
   string menulab = "(empty tab)"; 
   GtkWidget *hbox;
   if (new_tab)
   {
      // request to load file in new tab
      theApp->status_message("Adding new tab to display");
      ff = new PrismFileFrame(filename); 
      ff->hide_frame_title(); 

      hbox = ff->get_icon(); 
      int loc = gtk_notebook_append_page(GTK_NOTEBOOK(base_w), 
                                         bw=ff->base_widget(),
                                         hbox);
      frames.insert(pair<GtkWidget*, PrismFileFrame*>(bw, ff));
      gtk_widget_show_all(base_w);
      gtk_notebook_set_current_page(GTK_NOTEBOOK(base_w), loc); 
      if ((sts = !ff->loaded()) == 0)   menulab = filename;
      gtk_notebook_set_menu_label_text(GTK_NOTEBOOK(base_w), ff->base_widget(), 
         menulab.c_str());
   }
   else
   {
      int psn = gtk_notebook_get_current_page (GTK_NOTEBOOK(base_w));
      if (psn == -1)
      {
	 // no current tab
         theApp->status_message("Adding new tab to display");
         ff = new PrismFileFrame(filename); 
         ff->hide_frame_title(); 

         hbox = ff->get_icon(); 
//      gtk_image_set_pixel_size(GTK_IMAGE(img), 1);
         gtk_notebook_append_page(GTK_NOTEBOOK(base_w), bw=ff->base_widget(), 
	                          hbox);
	 frames.insert(pair<GtkWidget*, PrismFileFrame*>(bw, ff));
         gtk_widget_show_all(base_w);
         if ((sts = !ff->loaded()) == 0)  menulab = filename;
	 gtk_notebook_set_menu_label_text(GTK_NOTEBOOK(base_w), ff->base_widget(),
	    menulab.c_str());
      }
      else
      {
	 // use current tab
         bw = gtk_notebook_get_nth_page(GTK_NOTEBOOK(base_w), psn);
	 
         if (frames.count(bw)) sts = frames[bw]->load_file(filename);
         if (sts == 0)  menulab = filename;
         gtk_notebook_set_menu_label_text(GTK_NOTEBOOK(base_w), 
            frames[bw]->base_widget(), menulab.c_str());
      }
   }
   return sts; 
}


int PrismFileList::remove_file(string filename)
{
   theApp->status_message("Remove File feature currently not implemented"); 
   return 0; 
}

string PrismFileList::get_filename(int ext, int filter)
{
   static string fname; 
   fname = "";
   if (ext == -1)
   {
      // get filename of current tab
      int psn = gtk_notebook_get_current_page (GTK_NOTEBOOK(base_w));
      if (psn != -1)
      {
         GtkWidget* tmp = gtk_notebook_get_nth_page(GTK_NOTEBOOK(base_w), psn);
         if (frames.count(tmp))
         {
            map<GtkWidget*, PrismFileFrame*>::iterator ii;
            ii = frames.find(tmp);
	    fname = (*ii).second->get_filename(filter); 
         }
      }
   }
   else if ((ext > -1) && (ext < gtk_notebook_get_n_pages(GTK_NOTEBOOK(base_w)))) 
   {
      GtkWidget *tmp = gtk_notebook_get_nth_page(GTK_NOTEBOOK(base_w), ext);
      if (frames.count(tmp))
      {
         map<GtkWidget*, PrismFileFrame*>::iterator ii;
         ii = frames.find(tmp);
         fname = (*ii).second->get_filename(filter); 
      }
   }
   return fname;
}


int PrismFileList::remove_tab(int val)
{
   if (val == -1)
   {
      // remove current
      int psn = gtk_notebook_get_current_page (GTK_NOTEBOOK(base_w));
      if (psn != -1)
      {
         GtkWidget* tmp = gtk_notebook_get_nth_page(GTK_NOTEBOOK(base_w), psn);
         if (frames.count(tmp))
         {
            map<GtkWidget*, PrismFileFrame*>::iterator ii;
            ii = frames.find(tmp);
	    delete (*ii).second;
	    frames.erase(ii);
         }
         gtk_notebook_remove_page(GTK_NOTEBOOK(base_w), psn);
      }
   }
   else if ((val > -1) && (val < gtk_notebook_get_n_pages(GTK_NOTEBOOK(base_w)))) 
   {
      GtkWidget *tmp = gtk_notebook_get_nth_page(GTK_NOTEBOOK(base_w), val);
      if (frames.count(tmp))
      {
         map<GtkWidget*, PrismFileFrame*>::iterator ii;
         ii = frames.find(tmp);
	 frames.erase(ii);
      }
      gtk_notebook_remove_page(GTK_NOTEBOOK(base_w), val);
   }
   else 
   {
      char buffer[80];
      sprintf(buffer, "The requested tab %1d does not exist", val);
      theApp->status_message(buffer);
      return 1; 
   }
   return 0; 
}

int PrismFileList::set_current_tab(int val)
{
   if ((val > -1) && (val < gtk_notebook_get_n_pages(GTK_NOTEBOOK(base_w)))) 
   {
      GtkWidget *tmp = gtk_notebook_get_nth_page(GTK_NOTEBOOK(base_w), val);
      if (frames.count(tmp))
         gtk_notebook_set_current_page(GTK_NOTEBOOK(base_w), val);
   }
   else 
   {
      char buffer[80];
      sprintf(buffer, "The requested tab %1d does not exist", val);
      theApp->status_message(buffer);
      return 1; 
   }
   return 0; 
}

int PrismFileList::get_current_tab()
{
   if (base_w)  return gtk_notebook_get_current_page(GTK_NOTEBOOK(base_w)); 
      else      return -1; 
}

void PrismFileList::rotate_tabs(int val)
{
   //  -1=cycle to next pos, 0=left, 1=right, 2=top, 3=bottom
   if (val == -1)  
      tab_pos = (GtkPositionType)(((int)tab_pos + 1) % 4);
   else 
      tab_pos = (GtkPositionType)(val % 4);
   gtk_notebook_set_tab_pos(GTK_NOTEBOOK(base_w), tab_pos);
}

void PrismFileList::show_tabs()
{
   gtk_notebook_set_show_tabs(GTK_NOTEBOOK(base_w), tabs_on= true);
}

void PrismFileList::hide_tabs()
{
   gtk_notebook_set_show_tabs(GTK_NOTEBOOK(base_w), tabs_on= false);
}


int PrismFileList::show()
{
   gtk_widget_show_all(base_w);
   return 0;
}

int PrismFileList::visualize_tab(int type, int dialog)
{
   int psn = gtk_notebook_get_current_page (GTK_NOTEBOOK(base_w));
   if (psn == -1)
   {
      theApp->status_message("Can not perform requested visualization- no current tab to operate on");
      return -1; 
   }
   else
   {
      // use current tab
      GtkWidget* bw = gtk_notebook_get_nth_page(GTK_NOTEBOOK(base_w), psn);
      if (frames.count(bw))  return frames[bw]->visualize(type, dialog);
   }
   return 0; 
}

int PrismFileList::select_in_tab(int val, int as_col)
{
   int psn = gtk_notebook_get_current_page (GTK_NOTEBOOK(base_w));
   if (psn == -1)
   {
      theApp->status_message("Can not perform requested visualization- no current tab to operate on");
      return -1; 
   }
   else
   {
      // use current tab
      GtkWidget* bw = gtk_notebook_get_nth_page(GTK_NOTEBOOK(base_w), psn);
      if (frames.count(bw))  return frames[bw]->select_entry(val, as_col);
   }
   return 0; 
}


int PrismFileList::reload_tab(int ext, long row)
{
   int psn = gtk_notebook_get_current_page (GTK_NOTEBOOK(base_w));
   if (psn == -1)
   {
      theApp->status_message("Can not perform requested reload- no current tab to operate on");
      return -1; 
   }
   else
   {
      // use current tab
      GtkWidget* bw = gtk_notebook_get_nth_page(GTK_NOTEBOOK(base_w), psn);
      if (frames.count(bw))  return frames[bw]->open_extension(ext, row);
   }
   return 0; 
}


void PrismFileList::set_modified()
{
   int psn = gtk_notebook_get_current_page (GTK_NOTEBOOK(base_w));
   if (psn != -1)
   {
      GtkWidget* bw = gtk_notebook_get_nth_page(GTK_NOTEBOOK(base_w), psn);
      if (frames.count(bw))  return frames[bw]->set_modified();
   }
}

int PrismFileList::lock_tab(int mode)
{
   int psn = gtk_notebook_get_current_page (GTK_NOTEBOOK(base_w));
   if (psn == -1)
   {
      if (mode == 0)
         theApp->status_message("Can not perform lock- no current tab to operate on");
      else
         theApp->status_message("Can not perform unlock- no current tab to operate on");

      return -1; 
   }
   else
   {
      // use current tab
      GtkWidget* bw = gtk_notebook_get_nth_page(GTK_NOTEBOOK(base_w), psn);
      if (frames.count(bw)) 
      {
         frames[bw]->set_lock_mode(mode);
         return 0;
      }
   }
   return 0; 
}


int PrismFileList::get_current_mode()
{
   int psn = gtk_notebook_get_current_page (GTK_NOTEBOOK(base_w));
   if (psn == -1)
   {
      theApp->status_message("Can not obtain lock information- no current tab to operate on");
      return -1; 
   }
   else
   {
      // use current tab
      GtkWidget* bw = gtk_notebook_get_nth_page(GTK_NOTEBOOK(base_w), psn);
      if (frames.count(bw))  return frames[bw]->get_lock_mode();
         else return -1;                
   }
   return 0; 
}



