// 
//  Copyright (C) 2007-2008,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <fstream>
#include <unistd.h>

#include "PrismApp.hh"
#include "PrismFileMenu.hh"
#include "PrismUtils.hh"

#define PRSM_FLIST_BUF_SZ   256

using namespace std;

PrismFileMenu::PrismFileMenu(): PrismBaseObj()
{
   //   menuFile = gtk_menu_item_new_with_mnemonic(("_File"));
   //   gtk_menu_shell_insert(GTK_MENU_SHELL(menubar), menuFile, 0);
   base_w = gtk_menu_item_new_with_mnemonic("_File");

   menu = gtk_menu_new();

   menuItemOpen = gtk_menu_item_new_with_mnemonic(("_Open"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemOpen, 0);

   menuItemReopen = gtk_menu_item_new_with_mnemonic(("Reopen w/ _Selected"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemReopen, 1);

   menuItemRecent = gtk_menu_item_new_with_mnemonic(("_Recent"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemRecent, 2);

   menuItemOpenTab = gtk_menu_item_new_with_mnemonic(("Open New _Tab"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemOpenTab, 3);

   menuItemRecentTab = gtk_menu_item_new_with_mnemonic(("Recent _New Tab"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemRecentTab, 4);

   menuItemCloseTab = gtk_menu_item_new_with_mnemonic(("_Close Tab"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemCloseTab, 5);

   menuItemQuit = gtk_menu_item_new_with_mnemonic(("_Quit"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemQuit, 6);

   // setup file list and pull down 
   _filelist = getenv("HOME");
   if (_filelist.length() == 0) _filelist.append(".");
   _filelist.append("/");
   _filelist.append(".prism.filelist");
   _submenuRecent = NULL; 
   _submenuRecentTab = NULL; 
   load_recent_submenu(); 

   _tips = theApp->get_tips_group();
   if (_tips)
   {
      gtk_tooltips_set_tip(_tips, menuItemOpen,
         "open file in current tab via selection dialog",NULL);
      gtk_tooltips_set_tip(_tips, menuItemReopen,
         "reopen the file in current tab filtered on the selected cols",NULL);
      gtk_tooltips_set_tip(_tips, menuItemRecent,
         "open file in current tab from list of recent files",NULL);
      gtk_tooltips_set_tip(_tips, menuItemOpenTab,
         "open file in new tab via selection dialog",NULL);
      gtk_tooltips_set_tip(_tips, menuItemRecentTab,
         "open file in new tab from list of recent files",NULL);
      gtk_tooltips_set_tip(_tips, menuItemCloseTab,
         "remove the current tab",NULL);
      gtk_tooltips_set_tip(_tips, menuItemQuit,
         "Exit prism",NULL);
   }

   g_signal_connect(G_OBJECT (menuItemOpen), "activate",
                    G_CALLBACK(open_cb), (gpointer)this);
   g_signal_connect(G_OBJECT (menuItemReopen), "activate",
                    G_CALLBACK(reopen_cb), (gpointer)this);
   g_signal_connect(G_OBJECT (menuItemOpenTab), "activate",
                    G_CALLBACK(opentab_cb), (gpointer)this);

   g_signal_connect_swapped (G_OBJECT (menuItemCloseTab), "activate",
                             G_CALLBACK (closetab_cb),
                             (gpointer) NULL);
   g_signal_connect_swapped (G_OBJECT (menuItemQuit), "activate",
                             G_CALLBACK (quit_cb),
                             (gpointer) NULL);
   gtk_menu_item_set_submenu (GTK_MENU_ITEM(base_w), menu);

   _prev_dir = ""; 
}

PrismFileMenu::~PrismFileMenu()
{
   
}

int PrismFileMenu::show()
{
   gtk_widget_show(menuItemOpen);
   gtk_widget_show(menuItemReopen);
   gtk_widget_show(menuItemOpenTab);
   gtk_widget_show(menuItemRecent);
   gtk_widget_show(menuItemRecentTab);
   gtk_widget_show(menuItemQuit);
   gtk_widget_show(base_w); 
   return 0; 
}

int PrismFileMenu::open_cb(GObject *object, gpointer user_data)
{
   PrismFileMenu *fm = (PrismFileMenu*)user_data; 
   GtkWidget *dialog;
   dialog = gtk_file_chooser_dialog_new ("Open File",
                                         GTK_WINDOW(NULL),
	 		                 GTK_FILE_CHOOSER_ACTION_OPEN,
			                 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			                 GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
			                 NULL);

   // add basic filter support
   GtkFileFilter *filt = gtk_file_filter_new ();
   gtk_file_filter_add_pattern (filt, "*");
   gtk_file_filter_set_name(filt,"All Files"); 
   gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);
   filt = gtk_file_filter_new ();
   gtk_file_filter_add_pattern (filt, "*.fits");
   gtk_file_filter_set_name(filt,"FITS Files (*.fits)");
   gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);
   filt = gtk_file_filter_new ();
   gtk_file_filter_add_pattern (filt, "*.txt");
   gtk_file_filter_add_pattern (filt, "*.dat");
   gtk_file_filter_add_pattern (filt, "*.ascii");
   gtk_file_filter_set_name(filt,"ASCII/Text Files (*.txt *.dat *.ascii)");
   gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);

   char *cwd = g_get_current_dir();
   gtk_file_chooser_add_shortcut_folder(GTK_FILE_CHOOSER (dialog), cwd, NULL);
   gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER (dialog), cwd);
   g_free( cwd );

   if (fm) 
   {
      const char* predir = fm->get_prev_dir(); 
      if (predir && strlen(predir)) 
         gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), predir);
   }
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
   {
      char * filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
      if (filename)
      {
         theApp->load_file(filename);
         if (fm)
         {
            string newpath = filename;
            string::size_type delim = newpath.find_last_of('/');

            if (_is_directory(newpath))
            {
               if (delim != newpath.length() -1) newpath.append("/");
            }
            else if (delim != string::npos)
               newpath = newpath.substr(0, delim+1);
            fm->set_prev_dir(newpath);
         }
         g_free (filename);
      }
   }
   gtk_widget_destroy (dialog);
   return 0;
}

int PrismFileMenu::reopen_cb(GObject *object, gpointer user_data)
{
   PrismFileMenu *fm = (PrismFileMenu*)user_data; 

   string curfname = theApp->get_filename();
   string filename = theApp->get_filename(-1, 1);  // current tab with filter 
   // only reopen if we have a different filename from the current
   // and a column filter was provided 
   if (filename != curfname && filename.find("[cols ") != string::npos)
      theApp->load_file(filename);

   return 0; 
}


int PrismFileMenu::opentab_cb(GObject *object, gpointer user_data)
{
   PrismFileMenu *fm = (PrismFileMenu*)user_data; 
   GtkWidget *dialog = gtk_file_chooser_dialog_new ("Open File in new Tab",
                                         GTK_WINDOW(NULL),
	 		                 GTK_FILE_CHOOSER_ACTION_OPEN,
			                 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			                 GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
			                 NULL);
   // add basic filter support
   GtkFileFilter *filt = gtk_file_filter_new ();
   gtk_file_filter_add_pattern (filt, "*");
   gtk_file_filter_set_name(filt,"All Files");
   gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);
   filt = gtk_file_filter_new ();
   gtk_file_filter_add_pattern (filt, "*.fits");
   gtk_file_filter_set_name(filt,"Fits Files");
   gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);
   filt = gtk_file_filter_new ();
   gtk_file_filter_add_pattern (filt, "*.txt");
   gtk_file_filter_add_pattern (filt, "*.dat");
   gtk_file_filter_add_pattern (filt, "*.ascii");
   gtk_file_filter_set_name(filt,"Ascii/Text Files");
   gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);

   char *cwd = g_get_current_dir();
   gtk_file_chooser_add_shortcut_folder(GTK_FILE_CHOOSER (dialog), cwd, NULL);
   gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER (dialog), cwd);
   g_free( cwd );

   if (fm)  
   {
      const char* predir = fm->get_prev_dir(); 
      if (predir && strlen(predir)) 
         gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), predir);
   }
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
   {
      char * filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
      if (filename)
      {
	 string tmp = filename;
	 string::size_type pos;
	 if ((pos = tmp.find("[")) != string::npos)
	    tmp = tmp.substr(0, pos);
         if (!access(tmp.c_str(), F_OK))
         {
            theApp->load_file(filename, 1);
            if (fm)
            {
               string newpath = filename;
               string::size_type delim = newpath.find_last_of('/');

               if (_is_directory(newpath))
               {
                  if (delim != newpath.length() -1) newpath.append("/");
               }
               else if (delim != string::npos)
                  newpath = newpath.substr(0, delim+1);
               fm->set_prev_dir(newpath);
            }
         }
         else 
         {
	    string msg = "Unable to access file ";
	    msg.append(filename);
	    msg.append(". New tab not opened.");
            theApp->status_message(msg, 1);
         }
         g_free (filename);
      }
   }
   gtk_widget_destroy(dialog);
   return 0; 
}


int PrismFileMenu::closetab_cb()
{
   if (theApp)   theApp->close_tab();
   return 0;
}

int PrismFileMenu::quit_cb()
{
   if (theApp)   delete theApp;
   return 0;
}

int PrismFileMenu::filelist_cb(gchar* buf)
{
   // for now open in current tab
   if (buf) 
   {
      string tmp = buf;
      string::size_type pos;
      if ((pos = tmp.find("[")) != string::npos)
         tmp = tmp.substr(0, pos);
      if (!access(tmp.c_str(), R_OK))
         theApp->load_file(buf);  
      else
      {
	 string msg = "Unable to access 'recent' file ";
	 msg.append(buf);
         theApp->status_message(msg, 1);
      }
   }
   return 0; 
}

int PrismFileMenu::filelisttab_cb(gchar* buf)
{
   // for now open in current tab
   if (buf) 
   {
      string tmp = buf;
      string::size_type pos;
      if ((pos = tmp.find("[")) != string::npos)
         tmp = tmp.substr(0, pos);
      if (!access(tmp.c_str(), R_OK))
         theApp->load_file(buf,1);  
      else
      {
	 string msg = "Unable to access 'recent' file ";
	 msg.append(buf);
	 msg.append(". New tab not opened.");
         theApp->status_message(msg, 1);
      }
   }
   return 0; 
}

void PrismFileMenu::load_recent_submenu()
{
   if (_submenuRecent)
      gtk_menu_item_remove_submenu(GTK_MENU_ITEM(menuItemRecent)); 
   _submenuRecent = gtk_menu_new();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuItemRecent), _submenuRecent); 

   if (_submenuRecentTab)
      gtk_menu_item_remove_submenu(GTK_MENU_ITEM(menuItemRecentTab)); 
   _submenuRecentTab = gtk_menu_new();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuItemRecentTab), _submenuRecentTab); 
   ifstream in(_filelist.c_str());
   if (in)
   {
      int cnt = 0;
      char buf[PRSM_FLIST_BUF_SZ];
      string fn; 
      while(in.getline(buf, PRSM_FLIST_BUF_SZ)  && (cnt < PRSM_FLIST_SZ))
      {
	 _listEntries[cnt] = buf; 
         _filelistItem[cnt] = gtk_menu_item_new_with_label(buf);
         g_signal_connect_swapped(G_OBJECT(_filelistItem[cnt]), "activate",
                                  G_CALLBACK(filelist_cb),
                                  (gpointer)_listEntries[cnt].c_str());
         gtk_menu_shell_insert(GTK_MENU_SHELL(_submenuRecent), 
	    _filelistItem[cnt], cnt);
         _filelist2Item[cnt] = gtk_menu_item_new_with_label(buf);
         g_signal_connect_swapped(G_OBJECT(_filelist2Item[cnt]), "activate",
                                  G_CALLBACK(filelisttab_cb),
                                  (gpointer)_listEntries[cnt].c_str());
         gtk_menu_shell_insert(GTK_MENU_SHELL(_submenuRecentTab), 
	    _filelist2Item[cnt], cnt);
         cnt++; 
      }
      in.close();
   }
   if (_submenuRecent)  gtk_widget_show_all(_submenuRecent);
   if (_submenuRecentTab)  gtk_widget_show_all(_submenuRecentTab);
}

// add new file to filelist if not already there
void PrismFileMenu::update_filelist(string file)
{
   if (file.length())
   {
      ifstream in(_filelist.c_str());
      int rr, cnt = 0, found = -1;
      char buf[PRSM_FLIST_SZ+1][PRSM_FLIST_BUF_SZ];

      if (in)   // if file DNE then don't bother
      {
         while(in.getline(buf[cnt], PRSM_FLIST_BUF_SZ) && (cnt < PRSM_FLIST_SZ))
         {
            if (strcmp(buf[cnt], file.c_str()) == 0) found = cnt;
            cnt++;
         }
         in.close();
      }
      if (found == -1)  // entry not in file- update file 
      {
         ofstream out(_filelist.c_str());
         if (out)
         {
            out << file << endl;
            for (rr = 0; rr < cnt; rr++)
               if ((rr != found) && (rr < PRSM_FLIST_SZ))
                  out << buf[rr] << endl;
            out.close();
         }
         else
         {
	    string warn_msg = "Unable to update open list " + _filelist + "with file " + file;
            theApp->status_message(warn_msg);
         }
      }
      load_recent_submenu();
   }
}


