// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismFormatDialog.hh"
#include "PrismDataColumn.hh"
#include <gtkextra/gtksheet.h>
#include "PrismApp.hh"

using namespace std;

GdkColor PrismFormatDialog::_errcolor = { 0, 65335, 0, 0};

PrismFormatDialog::PrismFormatDialog(string title, PrismDataColumn* cols, 
   long colcnt):PrismBaseObj()
{
   _title = title; 
   _cols = cols;
   _colcnt = colcnt;
}

PrismFormatDialog::~PrismFormatDialog()
{
   if (_cols)  delete[] _cols;
}

void PrismFormatDialog::generate_form()
{
   string ss; 
   _entrycnt = 1; 

   base_w = gtk_dialog_new_with_buttons(_title.c_str(), NULL,
         GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_OK,
	       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);

   gtk_widget_set_size_request(base_w, 450, 200);

   _scrolled = gtk_scrolled_window_new (NULL,NULL);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), _scrolled);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(_scrolled),
      GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

   _table = gtk_table_new(8,_colcnt+2, false);
   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(_scrolled), _table);

   GtkWidget* tmplabel = gtk_label_new(" ");
   gtk_table_attach_defaults(GTK_TABLE(_table), tmplabel, 6,7,1,2);

   string current_val; 
   long cc; 
   for (cc = 0; cc < _colcnt; cc++)
   {
      if (!(_cols[cc].is_array() || _cols[cc].is_vector()) || 
(_cols[cc].is_array() && _colcnt==1) ||  // handle image for now      
          _cols[cc].is_array_comp())
      {
         FormatElem_p fep = new FormatElem_t;
         fep->column = &_cols[cc];
	 if (_cols[cc].is_array_comp())
            fep->info=_cols[cc].get_name()+" (" +_cols[cc].get_type()+" array)";
	 else 
            fep->info = _cols[cc].get_name() + " (" + _cols[cc].get_type() + ")";
	 fep->label = gtk_label_new(fep->info.c_str()); 
	 fep->entry = gtk_entry_new(); 
	 gtk_table_attach_defaults(GTK_TABLE(_table), fep->label, 1,3,cc+1,cc+2);
	 gtk_table_attach_defaults(GTK_TABLE(_table), fep->entry, 5,6,cc+1,cc+2);
	 current_val = _cols[cc].get_format(); 
	 gtk_entry_set_text(GTK_ENTRY(fep->entry), current_val.c_str());
         g_signal_connect(fep->entry, "changed", G_CALLBACK(key_edited_callback),
	                       (void*)this);
         _elems.push_back(fep); 
      }
   }
   gtk_widget_show_all(base_w);
}

int PrismFormatDialog::show()
{
   generate_form(); 
   gtk_widget_show_all(base_w);
   return 0;
}

int PrismFormatDialog::doit()
{
   int reload = 0;  
   generate_form();
   list<FormatElem_p>::iterator ii;
   if (gtk_dialog_run(GTK_DIALOG(base_w)) == GTK_RESPONSE_OK)
   {
      string newval; 

      // update format strings 
      if (_elems.size())
      {
         for (ii = _elems.begin(); ii != _elems.end(); ii++)
         {
	    newval = gtk_entry_get_text(GTK_ENTRY((*ii)->entry));
	    if ((*ii)->column->get_format() != newval)
	       if ((*ii)->column->set_format(newval) == 0) reload = 1;   
         }
      }
   }

   // delete all elements
   if (_elems.size())
   {
      for (ii = _elems.begin(); ii != _elems.end(); ii++)
         delete(*ii);
      _elems.clear();
   }
   gtk_widget_destroy(base_w);
   return reload; 
}


PrismDataColumn* PrismFormatDialog::get_associated_column(GtkWidget* wid)
{
   PrismDataColumn *col = NULL;

   list<FormatElem_p>::iterator ii;
   for (ii = _elems.begin(); ii != _elems.end() && col == NULL; ii++)
      if ((*ii)->entry == wid)  col = (*ii)->column;
   return col; 
}

void PrismFormatDialog::set_ok_sensitive(bool val)
{
   if (base_w)
      gtk_dialog_set_response_sensitive(GTK_DIALOG(base_w),GTK_RESPONSE_OK,val); 
}

void PrismFormatDialog::key_edited_callback(GObject *object, gpointer user_data)
{
   GtkWidget *ent = GTK_WIDGET(object);
   PrismFormatDialog* cmd = (PrismFormatDialog*)user_data;
   int valid = 1; 
   if (!(ent && cmd))   return;
   string txt = gtk_entry_get_text(GTK_ENTRY(ent));
   PrismDataColumn* col = cmd->get_associated_column(ent); 

   if (col && txt.length()) valid = col->validate_format(txt);
   if (valid)
      gtk_widget_modify_bg(ent, GTK_STATE_NORMAL, NULL);
   else 
      gtk_widget_modify_bg(ent, GTK_STATE_NORMAL, &_errcolor);

   cmd->set_ok_sensitive(valid); 
//   gtk_dialog_set_response_sensitive(GTK_DIALOG(base_w), GTK_RESPONSE_OK, valid);

}


