// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismHeaderAddCmd.hh"
#include "PrismHeaderTable.hh"
#include "PrismApp.hh"

using namespace std;

PrismHeaderAddCmd::PrismHeaderAddCmd():PrismBaseObj()
{
   _type = "string"; 
}

PrismHeaderAddCmd::~PrismHeaderAddCmd()
{
}

void PrismHeaderAddCmd::create_dialog()
{
   base_w = gtk_dialog_new_with_buttons("Add Header Keyword", NULL,
      GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_OK,
      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
   GtkWidget* tab = gtk_table_new(7,8, true);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), tab);

   GtkWidget* labnam = gtk_label_new("Name");
   GtkWidget* labval = gtk_label_new("Value");
   GtkWidget* labcom = gtk_label_new("Comment");
   GtkWidget* labuni = gtk_label_new("Units");
   GtkWidget* labtyp = gtk_label_new("Type");

   _name_w = gtk_entry_new(); 
   g_signal_connect(_name_w, "changed", G_CALLBACK(key_edited_callback),
                    (void*)this);

   _value_w = gtk_entry_new(); 
   if (_value.length())  gtk_entry_set_text(GTK_ENTRY(_value_w), _value.c_str());
   _comment_w = gtk_entry_new(); 
   if (_comment.length()) gtk_entry_set_text(GTK_ENTRY(_comment_w), _comment.c_str());
   _units_w = gtk_entry_new(); 
   if (_units.length()) gtk_entry_set_text(GTK_ENTRY(_units_w), _units.c_str());
   _type_w = gtk_combo_box_new_text(); 
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_type_w), 0, "String");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_type_w), 1, "Double");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_type_w), 2, "Float");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_type_w), 3, "Long");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_type_w), 4, "Short");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_type_w), 5, "Boolean");
   if (_type == "double")
      gtk_combo_box_set_active(GTK_COMBO_BOX(_type_w), 1);  
   else if (_type == "float")
      gtk_combo_box_set_active(GTK_COMBO_BOX(_type_w), 2);  
   else if (_type == "long")
      gtk_combo_box_set_active(GTK_COMBO_BOX(_type_w), 3);  
   else if (_type == "short")
      gtk_combo_box_set_active(GTK_COMBO_BOX(_type_w), 4);  
   else if (_type == "bool")
      gtk_combo_box_set_active(GTK_COMBO_BOX(_type_w), 5);  
   else 
      gtk_combo_box_set_active(GTK_COMBO_BOX(_type_w), 0);  

   _remc_w = gtk_check_button_new_with_label("Remember"); 
   _remu_w = gtk_check_button_new_with_label("Remember"); 
   _remv_w = gtk_check_button_new_with_label("Remember"); 
   _remt_w = gtk_check_button_new_with_label("Remember"); 

   gtk_table_attach_defaults(GTK_TABLE(tab), labnam, 0,2,1,2);
   gtk_table_attach_defaults(GTK_TABLE(tab), _name_w, 2,4,1,2);
   gtk_table_attach_defaults(GTK_TABLE(tab), labtyp, 4,5,1,2);
   gtk_table_attach_defaults(GTK_TABLE(tab), _type_w, 5,6,1,2);
   gtk_table_attach_defaults(GTK_TABLE(tab), _remt_w, 6,8,1,2);
   gtk_table_attach_defaults(GTK_TABLE(tab), labval,  0,2,2,3);
   gtk_table_attach_defaults(GTK_TABLE(tab), _value_w, 2,6,2,3);
   gtk_table_attach_defaults(GTK_TABLE(tab), _remv_w, 6,8,2,3);
   gtk_table_attach_defaults(GTK_TABLE(tab), labcom,  0,2,3,4);
   gtk_table_attach_defaults(GTK_TABLE(tab), _comment_w, 2,6,3,4);
   gtk_table_attach_defaults(GTK_TABLE(tab), _remc_w, 6,8,3,4);
   gtk_table_attach_defaults(GTK_TABLE(tab), labuni,  0,2,4,5);
   gtk_table_attach_defaults(GTK_TABLE(tab), _units_w, 2,6,4,5);
   gtk_table_attach_defaults(GTK_TABLE(tab), _remu_w, 6,8,4,5);

   show(); 
}

int PrismHeaderAddCmd::show()
{
   gtk_widget_show_all(base_w);
   return 0;
}

int PrismHeaderAddCmd::doit(PrismHeaderTable* table)
{
   bool val;

   if (!theApp)   return 1;   // app doesn't exist
   if (!table)   return 2;   // table doesn't exist

   _table = table; 

   if (theApp->get_preference("hdr.add.dialog", val))  val = true;
   
   if (val)
   {
      create_dialog();
      if (gtk_dialog_run(GTK_DIALOG(base_w)) == GTK_RESPONSE_OK)
      {
	 string use_type, use_key, use_value, use_desc, use_units; 

         use_key = gtk_entry_get_text(GTK_ENTRY(_name_w));
         use_value = gtk_entry_get_text(GTK_ENTRY(_value_w));
         use_desc = gtk_entry_get_text(GTK_ENTRY(_comment_w));
         use_units = gtk_entry_get_text(GTK_ENTRY(_units_w));
         int type = gtk_combo_box_get_active(GTK_COMBO_BOX(_type_w));
         switch(type)
	 {
	    case 1:   use_type = "double";   break;
	    case 2:   use_type = "float";    break;
	    case 3:   use_type = "long";     break;
	    case 4:   use_type = "short";    break;
	    case 5:   use_type = "bool";     break;
	    default:  use_type = "string";   break;
	 }
         table->add_item(use_key, use_value, use_type, use_desc, use_units);
	 if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_remt_w))) 
	    _type = use_type;          // store type value 
	 if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_remv_w))) 
	    _value = use_value;        // store value value 
	 if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_remc_w))) 
	    _comment = use_desc;    // store comment value 
	 if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_remu_w))) 
	    _units = use_units;        // store units value 
      }
      gtk_widget_destroy(base_w);
   }
   else  
   {
      table->add_item("", _value, _type, _comment, _units);
   }
   return 0;
}

void PrismHeaderAddCmd::key_edited_callback(GObject *object, gpointer user_data)
{
   GtkEntry *ent = GTK_ENTRY(object);
   PrismHeaderAddCmd* cmd = (PrismHeaderAddCmd*)user_data; 
   if (!(ent && cmd))   return;
   string txt = gtk_entry_get_text(GTK_ENTRY(ent));
   if (txt.length())   cmd->validate_keyword(txt);  
}

int PrismHeaderAddCmd::validate_keyword(string keyname)
{
   if (!_table)  return 1; 
   gtk_dialog_set_response_sensitive(GTK_DIALOG(base_w), GTK_RESPONSE_OK, 
      !_table->key_exists(keyname));   
   return 0; 
}



