// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismHeaderArea.hh"
#include "PrismHeaderTable.hh"
#include "PrismHeaderToolBar.hh"
#include "PrismDataSet.hh"
#include "PrismHeaderAddCmd.hh"
#include "PrismHeaderDeleteCmd.hh"

using namespace std;

PrismHeaderArea::PrismHeaderArea(string extname):PrismBaseObj()
{
   string ename = "";
   if (extname.length())   ename = extname.c_str(); 
   base_w = gtk_frame_new(ename.c_str());

   vbox = gtk_vbox_new(false, 1); 
   gtk_container_add (GTK_CONTAINER (base_w), vbox);

   toolBar = new PrismHeaderToolBar(this);
   gtk_box_pack_start (GTK_BOX(vbox), toolBar->base_widget(), 0, 0, 2);
   table = new PrismHeaderTable();
   gtk_box_pack_end (GTK_BOX(vbox), table->base_widget(), 1, 1, 2);
//   table->load_entries(); 

   _addCmd = new PrismHeaderAddCmd();
   _deleteCmd = new PrismHeaderDeleteCmd();
}

PrismHeaderArea::~PrismHeaderArea()
{
  if (_addCmd)
  {
     delete _addCmd;
     _addCmd = NULL;
  }
  if (_deleteCmd)
  {
     delete _deleteCmd;
     _deleteCmd = NULL;
  }
  if (table)
  {
     delete table; 
     table = NULL; 
  }
}

int PrismHeaderArea::show()
{
   if (table) table->show();
   if (toolBar) toolBar->show();
   gtk_widget_show_all(vbox);
   gtk_widget_show_all(base_w);
   return 0;
}

int PrismHeaderArea::set_data(PrismDataSet* dset)
{
   if (!dset)  return 1;  // no data set
      _dset = dset;
   if (table)  table->load_entries(_dset->get_block()); 
   return 0; 
}

void PrismHeaderArea::add_keyword()
{
   if (_addCmd)   _addCmd->doit(table);
}

void PrismHeaderArea::delete_keyword()
{
   if (_deleteCmd) _deleteCmd->doit(table); 
}

void PrismHeaderArea::disable_editing()
{
   if (toolBar)  toolBar->disable_editing();
   if (table)    table->disable_editing();
}

void PrismHeaderArea::enable_editing()
{
   if (_dset && !_dset->is_ascii()) 
   {
      if (toolBar)  toolBar->enable_editing();
      if (table)    table->enable_editing();
   }
}


