// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismHeaderDeleteCmd.hh"
#include "PrismHeaderTable.hh"
#include "PrismApp.hh"

using namespace std;

PrismHeaderDeleteCmd::PrismHeaderDeleteCmd():PrismBaseObj()
{
}

PrismHeaderDeleteCmd::~PrismHeaderDeleteCmd()
{
}

void PrismHeaderDeleteCmd::create_dialog()
{
   base_w = gtk_dialog_new_with_buttons("Delete Header Keyword", NULL,
      GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_OK,
      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
   GtkWidget* tab = gtk_table_new(6,5, true);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), tab);

   GtkWidget* labnam = gtk_label_new("Name");
   GtkWidget* labval = gtk_label_new("Value:");
   GtkWidget* labcmt = gtk_label_new("Comment:");

   _value_w = gtk_label_new("");
   _comment_w = gtk_label_new("");
   _name_w = gtk_entry_new(); 
   g_signal_connect(_name_w, "changed", G_CALLBACK(key_edited_callback),
                    (void*)this);
   if (!_table->get_selected_key(_keyname))
   {
      if (_keyname.length())  
      {
         gtk_entry_set_text(GTK_ENTRY(_name_w), _keyname.c_str());

         // get values
      }
   }

   gtk_table_attach_defaults(GTK_TABLE(tab), labnam, 0,2,1,2);
   gtk_table_attach_defaults(GTK_TABLE(tab), _name_w, 2,4,1,2);
   gtk_table_attach_defaults(GTK_TABLE(tab), labval, 0,2,2,3);
   gtk_table_attach_defaults(GTK_TABLE(tab), _value_w,  2,4,2,3);
   gtk_table_attach_defaults(GTK_TABLE(tab), labcmt, 0,2,3,4);
   gtk_table_attach_defaults(GTK_TABLE(tab), _comment_w,  2,4,3,4);

   show(); 

   validate_keyword(_keyname); 
}

int PrismHeaderDeleteCmd::show()
{
   gtk_widget_show_all(base_w);
   return 0;
}

int PrismHeaderDeleteCmd::doit(PrismHeaderTable* table)
{
   int rv  = 0; 
   bool val;

   if (!theApp)             return 1;   // app doesn't exist
   if (!(_table = table))   return 2;    // table doesn't exist

   if (theApp->get_preference("hdr.delete.dialog", val))  val = true;
   if (val)
   {
      create_dialog();
      if (gtk_dialog_run(GTK_DIALOG(base_w)) == GTK_RESPONSE_OK)
      {
         string use_key = gtk_entry_get_text(GTK_ENTRY(_name_w));
         rv = _table->delete_keyword(use_key); 
      }
      gtk_widget_destroy(base_w);
   }
   else  
   {
      // delete currently selected key 
      if (!_table->get_selected_key(_keyname)) rv = _table->delete_keyword(_keyname);
   }
   return rv;
}

void PrismHeaderDeleteCmd::key_edited_callback(GObject *object, gpointer user_data)
{
   GtkEntry *ent = GTK_ENTRY(object);
   PrismHeaderDeleteCmd* cmd = (PrismHeaderDeleteCmd*)user_data; 
   if (!(ent && cmd))   return;
   string txt = gtk_entry_get_text(GTK_ENTRY(ent));
   if (txt.length())   cmd->validate_keyword(txt);  
}

int PrismHeaderDeleteCmd::validate_keyword(string keyname)
{
   if (!_table)  return 1; 
   bool exists = _table->key_exists(keyname);
   gtk_dialog_set_response_sensitive(GTK_DIALOG(base_w), GTK_RESPONSE_OK, exists); 
   if (exists)
   {
      // get keys 
      string val, cmt;
      _table->get_key_fields(keyname, val, cmt);
      gtk_label_set_text(GTK_LABEL(_value_w), val.c_str());
      gtk_label_set_text(GTK_LABEL(_comment_w), cmt.c_str());
   }
   else
   {
      gtk_label_set_text(GTK_LABEL(_value_w), "");
      gtk_label_set_text(GTK_LABEL(_comment_w), "");
   }
   return 0; 
}



