// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismHeaderSortCmd.hh"
#include "PrismHeaderTable.hh"
#include "PrismApp.hh"

using namespace std;

PrismHeaderSortCmd::PrismHeaderSortCmd():PrismBaseObj()
{
   _key = 0;
   _order = 0;
   _prev = 3; 
}

PrismHeaderSortCmd::~PrismHeaderSortCmd()
{
}

void PrismHeaderSortCmd::create_dialog()
{
   base_w = gtk_dialog_new_with_buttons("Sort Header Keywords", NULL,
      GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_OK,
      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
   GtkWidget* tab = gtk_table_new(6,5, true);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), tab);

   GtkWidget* labnam = gtk_label_new("Column");

   _col_w = gtk_combo_box_new_text();
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_col_w), 0, "Key Name");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_col_w), 1, "Key Number");
   if (_key)   gtk_combo_box_set_active(GTK_COMBO_BOX(_col_w), 1);
      else     gtk_combo_box_set_active(GTK_COMBO_BOX(_col_w), 0);

   _asc_w = gtk_radio_button_new_with_label(NULL, "Ascending");
   _des_w = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(_asc_w), "Descending");
   if (_order) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_des_w), true);
      else     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_asc_w), true); 

   gtk_table_attach_defaults(GTK_TABLE(tab), labnam, 0,2,1,2);
   gtk_table_attach_defaults(GTK_TABLE(tab), _col_w, 2,4,1,2);

   gtk_table_attach_defaults(GTK_TABLE(tab), _asc_w, 1,3,2,3);
   gtk_table_attach_defaults(GTK_TABLE(tab), _des_w,  3,5,2,3);

   show(); 
}

int PrismHeaderSortCmd::show()
{
   gtk_widget_show_all(base_w);
   return 0;
}

int PrismHeaderSortCmd::doit(PrismHeaderTable* table)
{
   int rv  = 0; 
   bool val;

   if (!theApp)             return 1;   // app doesn't exist
   if (!(_table = table))   return 2;    // table doesn't exist

   if (theApp->get_preference("hdr.sort.dialog", val))  val = true;
   if (val)
   {
      create_dialog();
      if (gtk_dialog_run(GTK_DIALOG(base_w)) == GTK_RESPONSE_OK)
      {
         _key = gtk_combo_box_get_active(GTK_COMBO_BOX(_col_w));
	 _order = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_des_w));  
         rv = _table->sort((_prev = _key * 2 + _order));
      }
      gtk_widget_destroy(base_w);
   }
   else  
   {
      bool val; 
      if (theApp->get_preference("hdr.sort.cycle", val))  val = true;

      if (val)
      {
         ++_prev;
         if (_prev > 3) _prev = 0; 
      }
      rv = _table->sort(_prev);
   }
   return rv;
}




