// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PRISMHEADERTABLE_HH
#define PRISMHEADERTABLE_HH

#include <stdio.h>
#include <gtk/gtk.h>
#include <iostream>
#include <string>

#include "PrismBaseObj.hh"

class PrismDataBlock;

using namespace std;

typedef struct
{
   gchar   *name;
   gchar   *value;
   gchar   *type;
   gchar   *comment;
   gchar   *units;
   gint     pos; 
   gboolean editable;   // keyword is editable (ie. key vs raw/comment)
   gboolean unlocked;   // write access unlocked 
}
HeaderTableItem;

enum
{
   KEY_NAME,
   KEY_VALUE,
   KEY_TYPE,
   KEY_COMMENT,
   KEY_UNITS,
   KEY_POS,
   KEY_EDITABLE,
   KEY_UNLOCKED, 
   NUM_COLUMNS
};


enum
{
   SORTID_INDEX = 0,
   SORTID_KEY
};


class PrismHeaderTable: public PrismBaseObj
{
  public:
    PrismHeaderTable(string filename="");
    ~PrismHeaderTable();

    int load_entries(PrismDataBlock* blk, string setid="");

    virtual int show(); 
    int sort(int method=SORTID_INDEX); 

    int  add_item(string keyname="", string value="", string type="", 
                  string comment="", string units="");

    GtkListStore*  get_model()    { return model; }
    GArray*        get_articles() { return articles; }

    int get_selected_key(string &keyname); 
    bool key_exists(string keyname); 
    int get_key_fields(string keyname, string &value, string &comment);
    int add_keyword(string keyname, string value, string type, string comment,
                    string units);
    int modify_keyword(string keyname, string value, int field); 
    int delete_keyword(string keyname);

    long  get_default_num_keys()  { return _origkeys; }

    int set_column_visibility();

    void disable_editing(); 
    void enable_editing(); 

  protected:
    void  refresh(string setid="");
    static void cell_edited(GtkCellRendererText *cell, const gchar *path_string,
                            const gchar *new_text, gpointer data);

    GtkWidget *table;
    GArray *articles; 
    GtkListStore *model;
    GtkTreeIter iter;
    GtkTreeView *treeview;
    GtkCellRenderer *rndr_name, *rndr_val, *rndr_type, *rndr_cmt, 
              *rndr_units, *rndr_key; 

    PrismDataBlock *_block;
    long _origkeys;
    long _nkeys;
};

#endif 
