// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismHeaderToolBar.hh"
#include "PrismHeaderArea.hh"
#include "PrismApp.hh"

using namespace std;

PrismHeaderToolBar::PrismHeaderToolBar(PrismHeaderArea* parent):PrismBaseToolBar()
{
   _parent = parent; 
   int pos = 0; 

   GtkTooltips* tips = NULL;
   if (theApp)  tips = theApp->get_tips_group();
      else      tips = gtk_tooltips_new();

   if (addBtn = gtk_tool_button_new(NULL, "Add")) 
   {
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), addBtn, pos++); 
      g_signal_connect(addBtn, "clicked", G_CALLBACK(add_button_callback),
         (void*)_parent);
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(addBtn), GTK_TOOLTIPS(tips),
         "Add Keyword to Header", "add button");
   }
   if (deleteBtn = gtk_tool_button_new(NULL, "Delete"))
   {
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), deleteBtn, pos++); 
      g_signal_connect(deleteBtn, "clicked", G_CALLBACK(delete_button_callback),
         (void*)_parent);
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(deleteBtn), GTK_TOOLTIPS(tips),
         "Remove Keyword from Header", "delete button");
   }
#ifdef WLM_IMPLEMENT
   if (findBtn = gtk_tool_button_new(NULL, "Find"))
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), findBtn, pos++); 
   if (filterBtn = gtk_tool_button_new(NULL, "Filter"))
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), filterBtn, pos++); 
#else
   findBtn = NULL;
   filterBtn = NULL; 
#endif 
#ifdef WLM_IMPLEMENT
   if (metaBtn = gtk_tool_button_new(NULL, "Meta"))
      gtk_toolbar_insert(GTK_TOOLBAR(base_w), metaBtn, pos++); 
#else
   metaBtn = NULL; 
#endif 
}


PrismHeaderToolBar::~PrismHeaderToolBar()
{
   if (addBtn)  
   {   
      delete addBtn;  
      addBtn = NULL; 
   }
   if (deleteBtn)  
   {   
      delete deleteBtn;  
      deleteBtn = NULL; 
   }
   if (findBtn)  
   {   
      delete findBtn;  
      findBtn = NULL; 
   }
   if (filterBtn)  
   {   
      delete filterBtn;  
      filterBtn = NULL; 
   }
   if (metaBtn)  
   {   
      delete metaBtn;  
      metaBtn = NULL; 
   }
}


int PrismHeaderToolBar::enable_all(int flag)
{
   if (addBtn)       gtk_widget_set_sensitive(GTK_WIDGET(addBtn), TRUE); 
   if (deleteBtn)      gtk_widget_set_sensitive(GTK_WIDGET(deleteBtn), TRUE); 
   if (findBtn)       gtk_widget_set_sensitive(GTK_WIDGET(findBtn), TRUE); 
   if (filterBtn)     gtk_widget_set_sensitive(GTK_WIDGET(filterBtn), TRUE); 
   if (metaBtn)       gtk_widget_set_sensitive(GTK_WIDGET(metaBtn), TRUE); 
   return 0; 
}


int PrismHeaderToolBar::disable_all()
{
   if (addBtn)       gtk_widget_set_sensitive(GTK_WIDGET(addBtn), FALSE); 
   if (deleteBtn)      gtk_widget_set_sensitive(GTK_WIDGET(deleteBtn), FALSE); 
   if (findBtn)       gtk_widget_set_sensitive(GTK_WIDGET(findBtn), FALSE); 
   if (filterBtn)     gtk_widget_set_sensitive(GTK_WIDGET(filterBtn), FALSE); 
   if (metaBtn)       gtk_widget_set_sensitive(GTK_WIDGET(metaBtn), FALSE); 
   return 0; 
}

int PrismHeaderToolBar::show()
{
   gtk_widget_show_all(base_w);
   return 0; 
}

void PrismHeaderToolBar::add_button_callback(GObject *object, gpointer user_data)
{
   PrismHeaderArea *area = (PrismHeaderArea*)user_data;
   if (area)  area->add_keyword();
}

void PrismHeaderToolBar::delete_button_callback(GObject *object, gpointer user_data)
{
   PrismHeaderArea *area = (PrismHeaderArea*)user_data;
   if (area)  area->delete_keyword();
}


void PrismHeaderToolBar::disable_editing()
{
   if (addBtn)       gtk_widget_set_sensitive(GTK_WIDGET(addBtn), FALSE); 
   if (deleteBtn)    gtk_widget_set_sensitive(GTK_WIDGET(deleteBtn), FALSE); 

}

void PrismHeaderToolBar::enable_editing()
{
   if (addBtn)       gtk_widget_set_sensitive(GTK_WIDGET(addBtn), TRUE); 
   if (deleteBtn)    gtk_widget_set_sensitive(GTK_WIDGET(deleteBtn), TRUE); 
}



