// 
//  Copyright (C) 2008,2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismApp.hh"
#include "PrismHelpMenu.hh"
#include "PrismRefDialog.hh"
#include "cxcds_version.h"

using namespace std;

PrismHelpMenu::PrismHelpMenu(): PrismBaseObj()
{
   base_w = gtk_menu_item_new_with_mnemonic("_Help");

   menu = gtk_menu_new();

   menuRef = gtk_menu_item_new_with_mnemonic("Reference _Manual");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuRef, 0);
   menuHints = gtk_menu_item_new_with_mnemonic("Quick _Hints");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuHints, 1);

   subHints = gtk_menu_new();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuHints), subHints);
   subHintsIface = gtk_menu_item_new_with_label("Interfaces");
   gtk_menu_shell_insert(GTK_MENU_SHELL(subHints), subHintsIface, 0);
   subHintsTabs = gtk_menu_item_new_with_label("Tabs");
   gtk_menu_shell_insert(GTK_MENU_SHELL(subHints), subHintsTabs, 1);
   subHintsViz = gtk_menu_item_new_with_label("Visualizations");
   gtk_menu_shell_insert(GTK_MENU_SHELL(subHints), subHintsViz, 2);
   subHintsEdit = gtk_menu_item_new_with_label("Editing");
   gtk_menu_shell_insert(GTK_MENU_SHELL(subHints), subHintsEdit, 3);
   g_signal_connect_swapped (G_OBJECT(subHintsIface), "activate",
                             G_CALLBACK(hints_cb), (gpointer) 0);
   g_signal_connect_swapped (G_OBJECT(subHintsTabs), "activate",
                             G_CALLBACK(hints_cb), (gpointer) 1);
   g_signal_connect_swapped (G_OBJECT(subHintsViz), "activate",
                             G_CALLBACK(hints_cb), (gpointer) 2);
   g_signal_connect_swapped (G_OBJECT(subHintsEdit), "activate",
                             G_CALLBACK(hints_cb), (gpointer) 3);

   menuFaq = gtk_menu_item_new_with_mnemonic("_FAQ");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuFaq, 3);

   menuAbout = gtk_menu_item_new_with_label("About");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuAbout, 4);

#ifdef WLM_USE_ACK_MENU
   menuAck = gtk_menu_item_new_with_label("Acknowledgements");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuAck, 5);
   g_signal_connect_swapped (G_OBJECT(menuAck), "activate",
                             G_CALLBACK(acknowledgement_cb),
                             (gpointer) NULL);
#else
   menuAck = NULL;
#endif


   g_signal_connect_swapped (G_OBJECT(menuRef), "activate",
                             G_CALLBACK(reference_manual_cb),
                             (gpointer) NULL);
   g_signal_connect_swapped (G_OBJECT(menuFaq), "activate",
                             G_CALLBACK(faq_cb),
                             (gpointer) NULL);
   g_signal_connect_swapped (G_OBJECT(menuAbout), "activate",
                             G_CALLBACK(about_cb),
                             (gpointer) NULL);

   GtkTooltips* tips = theApp->get_tips_group();
   if (tips)
   {
      gtk_tooltips_set_tip(tips, menuRef,
         "display contents of the reference manual",NULL);
      gtk_tooltips_set_tip(tips, subHintsIface,
         "display interface specific help",NULL);
      gtk_tooltips_set_tip(tips, subHintsTabs,
         "display tab specific help",NULL);
      gtk_tooltips_set_tip(tips, subHintsViz,
         "display visualization specific help",NULL);
      gtk_tooltips_set_tip(tips, subHintsEdit,
         "display edit specific help",NULL);
      gtk_tooltips_set_tip(tips, menuFaq,
         "display Frequently Asked Questions",NULL);
      gtk_tooltips_set_tip(tips, menuAbout,
         "display version info",NULL);
#ifdef WLM_USE_ACK_MENU
      gtk_tooltips_set_tip(tips, menuAck,
         "display acknowledgements",NULL);
#endif
   }

   gtk_menu_item_set_submenu (GTK_MENU_ITEM(base_w), menu);
}

PrismHelpMenu::~PrismHelpMenu()
{
   
}

int PrismHelpMenu::show()
{
   gtk_widget_show_all(base_w); 
   return 0; 
}

int PrismHelpMenu::reference_manual_cb()
{
   string reffile;
   char *pth = getenv("ASCDS_INSTALL");
   if (pth)
   {
      reffile = pth;
      reffile.append("/doc/txt/");
   }
   reffile.append("prism_refman.txt"); 
   PrismRefDialog *dlg = new PrismRefDialog("Reference Manual", reffile);
   if (dlg)   dlg->show();
   return 0; 
}

int PrismHelpMenu::faq_cb()
{
   string reffile;
   char *pth = getenv("ASCDS_INSTALL");
   if (pth)
   {
      reffile = pth;
      reffile.append("/doc/txt/");
   }
   reffile.append("prism_faq.txt"); 
   PrismRefDialog *dlg = new PrismRefDialog("Frequently Asked Questions",reffile);
   if (dlg)   dlg->doit();
   delete dlg; 
   return 0; 
}

int PrismHelpMenu::hints_cb(gpointer user_data)
{
   gint pos = GPOINTER_TO_INT(user_data); 
   if (pos < 0 || pos > 3)  return 1; 

   string title, reffile;
   char *pth = getenv("ASCDS_INSTALL");
   if (pth)
   {
      reffile = pth;
      reffile.append("/doc/txt/");
   }
   switch(pos)
   {
      case 0:    // interfaces 
	 title = "Quick Hints - Interfaces"; 
         reffile.append("prism_qh_interfaces.txt"); 
      break; 
      case 1:
	 title = "Quick Hints - Tabs"; 
         reffile.append("prism_qh_tabs.txt"); 
      break; 
      case 2:
	 title = "Quick Hints - Visualizations"; 
         reffile.append("prism_qh_viz.txt"); 
      break; 
      case 3:
	 title = "Quick Hints - Editing"; 
         reffile.append("prism_qh_editing.txt"); 
      break; 
      default:
         return 1;
      break; 
   }
   PrismRefDialog *dlg = new PrismRefDialog(title, reffile);
   // display dialog as non-modal
   if (dlg)   dlg->show();
   return 0; 
}

int PrismHelpMenu::about_cb()
{
   gtk_show_about_dialog(NULL, 
      "name", "prism", 
      "title", "About Prism",
      "copyright", CXCDS_COPYRIGHT_STRING,
      "version", CXCDS_VERSION_STRING,
      NULL);  
   return 0; 
}

int PrismHelpMenu::acknowledgement_cb()
{
   return 0; 
}




