// 
//  Copyright (C) 2007-2008,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismImageCmd.hh"
#include "PrismDataSet.hh"
#include "PrismApp.hh"
#include "PrismDataBlock.hh"
#include "PrismDataColumn.hh"

#include <gtkextra/gtksheet.h>
#include <unistd.h>

using namespace std;


PrismImageCmd::PrismImageCmd():PrismBaseObj()
{
   _launched_ds9 = false; 
   _instantiate = true; 
   _xpa_name = "ds9"; 
   _imagers.push_back(_xpa_name); 
   _blockx = _blocky = 8.0;
   _img = 1; 
}

PrismImageCmd::~PrismImageCmd()
{
   list<string>::iterator ii = _imagers.begin();
   string cmd, cmd2;  
   char buf[129];
   char *s;
   bool ds9_exists = false;
   int zz;
   FILE* fd; 
   for(; ii !=_imagers.end(); ++ii)
   {
      if ((*ii) != "ds9" || _launched_ds9) 
      {
         cmd2 = "xpaaccess " + (*ii);
         if ((fd = popen(cmd2.c_str(), "r")) != NULL)
         {
            *buf = '\0';
            if (s=fgets(buf, 128, fd))  ds9_exists =  (s[0] == 'y');
         }
         // refresh gui- widgets in sheet seem to hang g_main_context_iter, add
         zz = 100; // a countdown to bail so we can get back to the main process
         while (g_main_context_iteration(NULL, FALSE) && zz--);
         if (ds9_exists)
         {
            cmd = "xpaset -p " + (*ii) + " quit > /dev/null; echo $?";
            fd = popen(cmd.c_str(), "r");
         }
      }
   }
}

void PrismImageCmd::generate_form()
{
   int lblcnt = 0; 
   base_w = gtk_dialog_new_with_buttons ("Image Dialog", NULL,
      GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
      GTK_STOCK_APPLY, GTK_RESPONSE_APPLY, GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
   GtkWidget* table = gtk_table_new(7,20, true); 
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), table);

   _label_w[lblcnt++] = gtk_label_new("Xpa Access");
   GtkWidget* cb_w = gtk_combo_box_entry_new_text();
   _launch = gtk_check_button_new_with_label("Launch");

   // load combo box w/ possible access points
   list<string>::iterator ii = _imagers.begin();
   for(; ii !=_imagers.end(); ++ii)
      gtk_combo_box_append_text(GTK_COMBO_BOX(cb_w), (*ii).c_str());
   // set default xpa access name
   _entry = GTK_ENTRY(GTK_BIN(cb_w)->child);
   if (_entry)  gtk_entry_set_text(_entry, _xpa_name.c_str()); 

   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_launch), _instantiate); 

   // set up curve column entry widget
   _label_w[lblcnt++] = gtk_label_new("X Column"); 
   _label_w[lblcnt++] = gtk_label_new("Y Column"); 
   _binx_w = gtk_combo_box_entry_new_text();
   // set up curve column entry widget
   _biny_w = gtk_combo_box_entry_new_text();
   PrismDataBlock *bl = _dset->get_block();
   if (bl && !(_img = bl->is_image()))
   {
      PrismDataColumn *dcols = bl->get_columns();
      long cc, rr, cnt = bl->get_dispcols();
      string cname;

      for (cc = rr = 0; rr < cnt; rr++)
         if (!(dcols[rr].is_array() || dcols[rr].is_array_comp() ||
             dcols[rr].is_vector()))
      {
         cname = dcols[rr].get_name();
	 gtk_combo_box_append_text(GTK_COMBO_BOX(_binx_w), cname.c_str());
	 gtk_combo_box_append_text(GTK_COMBO_BOX(_biny_w), cname.c_str());
	 if (cname == _binx)
	    gtk_combo_box_set_active(GTK_COMBO_BOX(_binx_w), cc);
	 else if (cname == _biny)
	    gtk_combo_box_set_active(GTK_COMBO_BOX(_biny_w), cc);
         cc++;
      }
   }

   _label_w[lblcnt++] = gtk_label_new("X block Factor");
   _adjx = (GtkAdjustment *)
   gtk_adjustment_new(_blockx, 0, 256.0, 1.0, 1.0, 0.0);
   _blockx_w   = gtk_spin_button_new(_adjx, 1.0, 3);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_blockx_w), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_blockx_w),
      GTK_UPDATE_IF_VALID);
   _label_w[lblcnt++] = gtk_label_new("Y block Factor");
   _adjy = (GtkAdjustment *)
   gtk_adjustment_new(_blocky, 0, 256.0, 1.0, 1.0, 0.0);
   _blocky_w   = gtk_spin_button_new(_adjy, 1.0, 3);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_blocky_w), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_blocky_w),
      GTK_UPDATE_IF_VALID);
   
   gtk_table_attach_defaults(GTK_TABLE(table), _label_w[0], 1,8,1,2);
   gtk_table_attach_defaults(GTK_TABLE(table), cb_w, 8,14,1,2);
   gtk_table_attach_defaults(GTK_TABLE(table), _launch, 15,19,1,2);
   gtk_table_attach_defaults(GTK_TABLE(table), _label_w[1], 1,8,2,3); 
   gtk_table_attach_defaults(GTK_TABLE(table), _binx_w, 8,14,2,3);
   gtk_table_attach_defaults(GTK_TABLE(table), _label_w[2],1,8,3,4); 
   gtk_table_attach_defaults(GTK_TABLE(table), _biny_w, 8,14,3,4);
   gtk_table_attach_defaults(GTK_TABLE(table), _label_w[3], 1,8,4,5);
   gtk_table_attach_defaults(GTK_TABLE(table), _blockx_w, 8,14,4,5);
   gtk_table_attach_defaults(GTK_TABLE(table), _label_w[4], 1,8,5,6);
   gtk_table_attach_defaults(GTK_TABLE(table), _blocky_w, 8,14,5,6);

   // gray out file specific options 
   if (_binx_w)   gtk_widget_set_sensitive(GTK_WIDGET(_binx_w), !_img);
   if (_biny_w)   gtk_widget_set_sensitive(GTK_WIDGET(_biny_w), !_img);
   if (_blockx_w) gtk_widget_set_sensitive(GTK_WIDGET(_blockx_w), !_img);
   if (_blocky_w) gtk_widget_set_sensitive(GTK_WIDGET(_blocky_w), !_img);

   gtk_widget_show_all(table);
   gtk_widget_set_size_request(base_w, 400, 300);
}

int PrismImageCmd::show()
{
   generate_form(); 
   gtk_widget_show_all(base_w);
   return 0;
}

int PrismImageCmd::doit(PrismDataSet* dset, int dialog)
{
   bool use_dialog = false;

   if (!theApp) return 1;       // app does not exist 
   
   if (dialog == -1)
   {
       if (theApp->get_preference("data.image.dialog", use_dialog))
	  use_dialog = true;

   }
   else if (dialog == 1)
      use_dialog = true; 
   
   if ((_dset = dset) == NULL) 
   {
      theApp->status_message("No data to image in current tab"); 
      return 1; 
   }

   if (use_dialog)
   {
      // generate a dialog to allow user to specify defaults 
      generate_form(); 

      int resp = GTK_RESPONSE_APPLY;
      while (resp == GTK_RESPONSE_APPLY)
      {
         resp = gtk_dialog_run(GTK_DIALOG(base_w));
         if (resp == GTK_RESPONSE_APPLY || resp == GTK_RESPONSE_OK)
         {
	    if (_entry)  _xpa_name = gtk_entry_get_text(_entry);
            if (_launch) 
	       _instantiate = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_launch));
	    if (!_img)  // load default values for tables 
	    {
               GtkEntry* entry_w = GTK_ENTRY(GTK_BIN(_binx_w)->child);
	       _binx = gtk_entry_get_text(GTK_ENTRY(entry_w));
               entry_w = GTK_ENTRY(GTK_BIN(_biny_w)->child);
	       _biny = gtk_entry_get_text(GTK_ENTRY(entry_w));
	       _blockx = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_blockx_w));
	       _blocky = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_blocky_w));
            }
            generate_image(); 
         }
      }
      gtk_widget_destroy(base_w); 
   }
   else
   {
      // skip dialog and just generate an image using defaults
      generate_image(); 
   }
   return 0; 
}

int PrismImageCmd::generate_image()
{
   FILE *fd;
   char buf[129];
   char *s;
   bool ds9_exists = false; 
   int  zz; 
   string sts; 

   if (!_dset)  return 2;   // don't have a data file 

   string cmd = "xpaaccess " + _xpa_name; 
   if ((fd = popen(cmd.c_str(), "r")) != NULL)
   {
      *buf = '\0';
      if (s=fgets(buf, 128, fd))  ds9_exists =  (s[0] == 'y');
   }
   // refresh gui- since widgets in sheet seem to hang g_main_context_iter, add
   zz = 100; // a countdown to bail out so we can get back to the main process 
   while (g_main_context_iteration(NULL, FALSE) && zz--);   
   if (!ds9_exists)
   {
      if (_instantiate)
      {
         string lcmd = "ds9 -title " + _xpa_name + "&"; 
         // instantiate ds9 
         theApp->status_message("Starting a new ds9 window"); 
         system(lcmd.c_str()); 

         // loop to try to connect to ds9 
         int countdown = 30;
         while(countdown-- && !ds9_exists)
         {
            if ((fd = popen(cmd.c_str(), "r")) != NULL)
            {
               *buf = '\0';
               if (s=fgets(buf, 128, fd))  ds9_exists =  (s[0] == 'y');
            }
            zz=100;
            while (g_main_context_iteration(NULL, FALSE) &&zz--);   // refresh gui
            sleep(1); 
         }
         if (!ds9_exists)
            theApp->status_message("Failed to instantiate a copy of ds9"); 
         else if (_xpa_name == "ds9")
            _launched_ds9 = true; 
         else 
            _imagers.push_back(_xpa_name); 
      }
      else
      {
         sts = "Could not access ds9 via access point " + _xpa_name + " and launch option disabled"; 
         theApp->status_message(sts);
      }
   }

   if (ds9_exists)
   {
      int ret = 1; 
      zz=100;
      while (g_main_context_iteration(NULL, FALSE) &&zz--);   // refresh gui

      // send xpa message to load file 
      cmd = "xpaset -p " + _xpa_name + " file " + _dset->get_filename() + "> /dev/null; echo $?";
      if ((fd = popen(cmd.c_str(), "r")) != NULL)
      {
         *buf = '\0';
         if (s=fgets(buf, 128, fd))  ret = (s[0] != '0');
      } 
      if (ret == 0)
      {
	 if (!_img && _binx.length() && _biny.length())
	 {
            cmd = "xpaset -p " + _xpa_name + " bin cols " + _binx + " " + _biny + "> /dev/null; echo $?"; 

            if ((fd = popen(cmd.c_str(), "r")) != NULL)
	    {
	       *buf = '\0';
	       if (s=fgets(buf, 128, fd))  ret = (s[0] != '0');
	    }
	    if (ret)
	    {
	       sts = "Failed to bin image on requested columns: " + _binx + " and " + _biny;
//	       sts.append(_binx);
//	       sts.append(" and ");
//	       sts.append(_biny);
               theApp->status_message(sts);
	       return 2; 
            }
	    else
	    {
	       sts = "Binning data for image using columns: " + _binx +
	          " and " + _biny;
               theApp->status_message(sts);
	    }
         }
         if (!_img && (_blockx != 1 || _blocky != 1))
	 {
	    sprintf(buf, " bin factor %4.3f %4.3f", _blockx, _blocky); 
	    string bfactor = buf; 
            cmd = "xpaset -p " + _xpa_name + buf + "> /dev/null; echo $?";
            if ((fd = popen(cmd.c_str(), "r")) != NULL)
	    {
	       *buf = '\0';
	       if (s=fgets(buf, 128, fd))  ret = (s[0] != '0');
	    }
	    if (ret)
	    {
	       sts = "Failed to block image using" + bfactor;
               theApp->status_message(sts);
	       return 3; 
            }
	    else
	    {
	       sts = "Binning data for image using" + bfactor; 
               theApp->status_message(sts);
	    }
	 }
         theApp->status_message("Image should be displayed in ds9");
      }
      else 
         theApp->status_message("Unable to connect to ds9- image request failed");
      return ret; 
   }
   theApp->status_message("Unable to connect to ds9- image request failed");
   return 1; 
}

