// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismApp.hh"
#include "PrismMenu.hh"
#include "PrismFileMenu.hh"
#include "PrismEditMenu.hh"
#include "PrismViewMenu.hh"
#include "PrismHelpMenu.hh"
#include "CiaoAnalysisMenu.hh"


using namespace std;

PrismMenu::PrismMenu(string filename, GtkTooltips* tips):PrismBaseObj()
{
   base_w = gtk_menu_bar_new();

   menuFile = new PrismFileMenu();
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuFile->base_widget());

   menuEdit = new PrismEditMenu();
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuEdit->base_widget());

   menuView = new PrismViewMenu();
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuView->base_widget());

   menuAnalysis = new CiaoAnalysisMenu(filename, tips, 
      theApp->get_current_filename, theApp->set_status_message);
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuAnalysis->base_widget());

   menuHelp = new PrismHelpMenu();
   gtk_menu_item_set_right_justified(GTK_MENU_ITEM(menuHelp->base_widget()),
      TRUE);
   gtk_menu_bar_append(GTK_MENU_BAR(base_w), menuHelp->base_widget());
}

PrismMenu::~PrismMenu()
{
   if (menuFile)
   {
      delete menuFile;
      menuFile = NULL;
   }
   if (menuEdit)
   {
      delete menuEdit;
      menuEdit = NULL;
   }
   if (menuView)
   {
      delete menuView;
      menuView = NULL;
   }
   if (menuAnalysis)
   {
      delete menuAnalysis;
      menuAnalysis = NULL; 
   }
   if (menuHelp)
   {
      delete menuHelp;
      menuHelp = NULL;
   }
}

int PrismMenu::show()
{
   menuFile->show();
   menuEdit->show();
   menuView->show();
   menuHelp->show();
   menuAnalysis->show(); 
   gtk_widget_show(base_w);
   return 0; 
}

int PrismMenu::update_filelist(string fname)
{
   if (!menuFile)  return 1; 
   menuFile->update_filelist(fname); 
   return 0; 
}


