// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismMetaDialog.hh"
#include "PrismDataColumn.hh"
#include <gtkextra/gtksheet.h>
#include "PrismApp.hh"

using namespace std;

PrismMetaDialog::PrismMetaDialog(string title, PrismDataColumn* cols, 
   long colcnt):PrismBaseObj()
{
   _title = title; 
   _cols = cols;
   _colcnt = colcnt;
   for (int ii = 0; ii < META_DIALOG_MAXSZ; ii++) _vis[ii] = true;
}

PrismMetaDialog::~PrismMetaDialog()
{
   if (_cols)  delete[] _cols;
}

void PrismMetaDialog::generate_form()
{
   string ss; 
   long slen, mlen = 0, ndx = 0; 

   if (theApp)
   {
      theApp->get_preference("info.display.description", _vis[0]);
      theApp->get_preference("info.display.columntype", _vis[1]);
      theApp->get_preference("info.display.datatype", _vis[2]);
      theApp->get_preference("info.display.units", _vis[3]);
      theApp->get_preference("info.display.min", _vis[4]);
      theApp->get_preference("info.display.max", _vis[5]);
      theApp->get_preference("info.display.null", _vis[6]);
      theApp->get_preference("info.display.existence", _vis[7]);
      theApp->get_preference("info.display.coords", _vis[8]);
      theApp->get_preference("info.display.crpix", _vis[9]);
      theApp->get_preference("info.display.crval", _vis[10]);
      theApp->get_preference("info.display.cdelt", _vis[11]);
      _entrycnt = 0; 
      for (int ii = 0; ii < META_DIALOG_MAXSZ; ii++) 
         if (_vis[ii]) _entrycnt++;
   }

   base_w = gtk_dialog_new_with_buttons (_title.c_str(), NULL,
      GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_NONE, NULL);
   gtk_widget_set_size_request(base_w, 450, 200);
   // set up to delete dialog resources on response 
   g_signal_connect_swapped(base_w, "response", G_CALLBACK(gtk_widget_destroy),
      base_w);

   _scrolled = gtk_scrolled_window_new (NULL,NULL);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), _scrolled);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(_scrolled),
      GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   _sheet = gtk_sheet_new(_colcnt, _entrycnt, "");
   GtkSheetRange range = { 0,0,_colcnt,_entrycnt};
   gtk_sheet_range_set_editable(GTK_SHEET(_sheet), &range, 0);
   if (_vis[0])
      gtk_sheet_column_button_add_label(GTK_SHEET(_sheet),ndx++,"description"); 
   if (_vis[1])
      gtk_sheet_column_button_add_label(GTK_SHEET(_sheet),ndx++,"column type"); 
   if (_vis[2])
      gtk_sheet_column_button_add_label(GTK_SHEET(_sheet),ndx++,"data type"); 
   if (_vis[3])
      gtk_sheet_column_button_add_label(GTK_SHEET(_sheet),ndx++,"units"); 
   if (_vis[4])
      gtk_sheet_column_button_add_label(GTK_SHEET(_sheet),ndx++,"min"); 
   if (_vis[5])
      gtk_sheet_column_button_add_label(GTK_SHEET(_sheet),ndx++,"max"); 
   if (_vis[6])
      gtk_sheet_column_button_add_label(GTK_SHEET(_sheet),ndx++,"null"); 
   if (_vis[7])
      gtk_sheet_column_button_add_label(GTK_SHEET(_sheet),ndx++,"existence"); 
   if (_vis[8])
      gtk_sheet_column_button_add_label(GTK_SHEET(_sheet),ndx++,"associated coords");
   if (_vis[9])
      gtk_sheet_column_button_add_label(GTK_SHEET(_sheet),ndx++,"crpix");
   if (_vis[10])
      gtk_sheet_column_button_add_label(GTK_SHEET(_sheet),ndx++,"crval");
   if (_vis[11])
      gtk_sheet_column_button_add_label(GTK_SHEET(_sheet),ndx++,"cdelt");

   gtk_container_add(GTK_CONTAINER(_scrolled), _sheet);

   for (long cc = 0; cc < _colcnt; cc++)
   {
      ndx = 0; 
      ss = _cols[cc].get_name();
      gtk_sheet_row_button_add_label(GTK_SHEET(_sheet), cc, ss.c_str());
      ss = _cols[cc].get_comment();
      slen = ss.length();
      if (slen > mlen)  mlen = slen; 
      if (_vis[0])
         gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, ndx++, GTK_JUSTIFY_LEFT, ss.c_str());
      if (_vis[1])
         if (_cols[cc].is_vector_array())
            gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, ndx++, GTK_JUSTIFY_LEFT, "vector array");
         else if (_cols[cc].is_vector())
            gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, ndx++, GTK_JUSTIFY_LEFT, "vector");
         else if (_cols[cc].is_array())
            gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, ndx++, GTK_JUSTIFY_LEFT, "array");
         else if (_cols[cc].is_vector_array_comp())
            gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, ndx++, GTK_JUSTIFY_LEFT, "v.a. comp");
         else if (_cols[cc].is_vector_comp())
            gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, ndx++, GTK_JUSTIFY_LEFT, "v. comp");
         else if (_cols[cc].is_array_comp())
            gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, ndx++, GTK_JUSTIFY_LEFT, "a. comp");
         else 
            gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, ndx++, GTK_JUSTIFY_LEFT, "scalar");

      if (_vis[2])
      {
         ss = _cols[cc].get_type();
         gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, ndx++, GTK_JUSTIFY_LEFT, ss.c_str());
      }
      if (_vis[3])
      {
         ss = _cols[cc].get_units();
         gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, ndx++, GTK_JUSTIFY_LEFT, ss.c_str());
      }
      if (_vis[4])
      {
         ss = _cols[cc].get_tlmin();
         gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, ndx++, GTK_JUSTIFY_LEFT, ss.c_str());
      }
      if (_vis[5])
      {
         ss = _cols[cc].get_tlmax();
         gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, ndx++, GTK_JUSTIFY_LEFT, ss.c_str());
      }
      if (_vis[6])
      {
	 ss = _cols[cc].get_tnull();
         gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, ndx++, GTK_JUSTIFY_LEFT, ss.c_str());
      }
      if (_vis[7])
         if (_cols[cc].is_virtual()) 
            gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, ndx++, GTK_JUSTIFY_LEFT, "virtual");
         else 
            gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, ndx++, GTK_JUSTIFY_LEFT, "physical");
      if (_vis[8])
      {
         ss = _cols[cc].get_coordsys();
         if (ss.length()) 
            gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, ndx++, GTK_JUSTIFY_LEFT, ss.c_str());
      }
      if (_vis[9])
      {
         ss = _cols[cc].get_crpix();
         if (ss.length()) 
            gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, ndx++, GTK_JUSTIFY_LEFT, ss.c_str());
      }
      if (_vis[10])
      {
         ss = _cols[cc].get_crval();
         if (ss.length()) 
            gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, 10, GTK_JUSTIFY_LEFT, ss.c_str());
      }
      if (_vis[11])
      {
         ss = _cols[cc].get_cdelt();
         if (ss.length()) 
            gtk_sheet_set_cell(GTK_SHEET(_sheet), cc, 11, GTK_JUSTIFY_LEFT, ss.c_str());
      }
   }
   // resize description column width if an entry is wider than the col label
   if (_vis[0] && mlen > 12)  
      gtk_sheet_set_column_width(GTK_SHEET(_sheet), 0, 200); 

   gtk_widget_show_all(base_w);
}

int PrismMetaDialog::show()
{
   generate_form(); 
   gtk_widget_show_all(base_w);
   return 0;
}

int PrismMetaDialog::doit()
{
   generate_form();
   if (gtk_dialog_run(GTK_DIALOG(base_w)) == GTK_RESPONSE_OK)
   {
     // nothing to do 
   }
   return 0; 
}

