// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismPlotCmd.hh"
#include "PrismDataSet.hh"
#include "PrismDataBlock.hh"
#include "PrismDataColumn.hh"
#include "PrismApp.hh"

#include <gtkextra/gtksheet.h>
#include <sys/types.h>
#include <stdio.h>
#include <unistd.h>
#include <fstream>

using namespace std;

PrismPlotCmd::PrismPlotCmd():PrismBaseObj()
{
   _chipsId = ""; 
   _behavior = 2; 
   _lstyle = 1; 
   _sstyle = 1; 
   _estyle = 0; 
   _lthick = 1.0; 
   _sthick = 5; 
   _ethick = 1.0; 
   _lcolord = true; 
   _scolord = true; 
   _ecolord = true; 
   _lcolor = _scolor = "white";
   _ecolor = "red";
   _shell_py = false; 
}

PrismPlotCmd::~PrismPlotCmd()
{
}

void PrismPlotCmd::generate_form()
{
   base_w = gtk_dialog_new_with_buttons ("Plot Dialog", NULL,
      GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
      GTK_STOCK_APPLY, GTK_RESPONSE_APPLY, GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);

   _vbox_w = gtk_vbox_new (FALSE,5); 

   gtk_widget_set_size_request(base_w, 650, 440);
   _scrolled_w = gtk_scrolled_window_new (NULL,NULL);
   gtk_widget_set_size_request(_vbox_w, 650, -1);

   _frm_w = gtk_frame_new(" Plot Settings "); 
   gtk_box_pack_start(GTK_BOX(_vbox_w), _frm_w, FALSE, FALSE, 0);
   gtk_widget_show(_frm_w);
   gtk_widget_show(_vbox_w);

   GtkWidget* table = gtk_table_new(11,20, true); 
   gtk_container_add(GTK_CONTAINER(_frm_w), table);

   GtkWidget* label_w = gtk_label_new("Chips Server Id");
   GtkWidget* cb_w = gtk_combo_box_entry_new_text();
   _launch = gtk_check_button_new_with_label("Launch");
   GtkWidget* lab2_w = gtk_label_new("Interactive Shell");

   // status message area 
   _status_w = gtk_label_new("");
   gtk_box_pack_start(GTK_BOX(_vbox_w), _status_w, FALSE, FALSE, 0);

   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), _scrolled_w);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(_scrolled_w),
      GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(_scrolled_w), _vbox_w);
   gtk_widget_show(_status_w);

   // load combo box w/ possible access points
   list<string>::iterator ii = _plotters.begin();
   for(; ii !=_plotters.end(); ++ii)
      gtk_combo_box_append_text(GTK_COMBO_BOX(cb_w), (*ii).c_str());
   // set default xpa access name
   _entry = GTK_ENTRY(GTK_BIN(cb_w)->child);
   if (_entry)  gtk_entry_set_text(_entry, _chipsId.c_str()); 

   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_launch), _instantiate); 

   _shell_no_w = gtk_radio_button_new_with_label(NULL, "None");
   _shell_py_w = gtk_radio_button_new_with_label_from_widget(
      GTK_RADIO_BUTTON(_shell_no_w),"Python");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_shell_no_w), true);

   GtkWidget* cvxl_w = gtk_label_new("Curve  X");
   GtkWidget* cvyl_w = gtk_label_new("Y");
   GtkWidget* cvel_w = gtk_label_new("Y_Errs");

   // set up curve column entry widget
   _cvx_w = gtk_combo_box_entry_new_text();
   _cvy_w = gtk_combo_box_entry_new_text();
   _cve_w = gtk_combo_box_entry_new_text();

   set_default_column_names(true); 
 
   GtkWidget* ll_w = gtk_label_new("Line Style");
   _lstyle_w = gtk_combo_box_new_text();
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_lstyle_w), 0, "noline");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_lstyle_w), 1, "solid");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_lstyle_w), 2, "dot");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_lstyle_w), 3, "dotlongdash");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_lstyle_w), 4, "dotshortdash");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_lstyle_w), 5, "longdash");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_lstyle_w), 6, "shortdash");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_lstyle_w), 7, "shortdashlongdash");
   gtk_combo_box_set_active(GTK_COMBO_BOX(_lstyle_w), _lstyle);
   GtkWidget* lth_w = gtk_label_new("Thickness");
   GtkAdjustment *th_adj = (GtkAdjustment *)
   gtk_adjustment_new(_lthick, 0.5, 10.0, 1.0, 1.0, 0.0);
   _lthick_w   = gtk_spin_button_new(th_adj, 1.0, 1);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_lthick_w), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_lthick_w),
      GTK_UPDATE_IF_VALID);
   GdkColor clr; 
   GtkWidget* lc_w = gtk_label_new("Color");
   gdk_color_parse(_lcolor.c_str(), &clr);
   _lcolor_w = gtk_color_button_new_with_color(&clr);
   gtk_color_button_set_title(GTK_COLOR_BUTTON(_lcolor_w), 
      "Line Color Selection"); 
   _lcolord_w = gtk_check_button_new_with_label("Default");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_lcolord_w), _lcolord); 
   g_signal_connect(_lcolord_w,"toggled",G_CALLBACK(toggled_value_callback),
                    (void*)this);

   GtkWidget* sl_w = gtk_label_new("Symbol Style");
   _sstyle_w = gtk_combo_box_new_text();
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 0, "none");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 1, "cross");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 2, "diamond");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 3, "downtriangle");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 4, "circle");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 5, "plus");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 6, "square");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 7, "uptriangle");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 8, "point");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_sstyle_w), 9, "arrow");
   gtk_combo_box_set_active(GTK_COMBO_BOX(_sstyle_w), _sstyle);
   GtkWidget* sth_w = gtk_label_new("Size");
   th_adj = (GtkAdjustment *)gtk_adjustment_new(_sthick,1.0,100.0,1.0,5.0,0.0);
   _sthick_w   = gtk_spin_button_new(th_adj, 1.0, 0);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_sthick_w), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_sthick_w),
      GTK_UPDATE_IF_VALID);
   GtkWidget* sc_w = gtk_label_new("Color");
   gdk_color_parse(_scolor.c_str(), &clr);
   _scolor_w = gtk_color_button_new_with_color(&clr);
   gtk_color_button_set_title(GTK_COLOR_BUTTON(_scolor_w), 
      "Symbol Color Selection"); 
   _scolord_w = gtk_check_button_new_with_label("Default");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_scolord_w), _scolord); 
   g_signal_connect(_scolord_w,"toggled",G_CALLBACK(toggled_value_callback),
                    (void*)this);

   GtkWidget* el_w = gtk_label_new("Error Style");
   _estyle_w = gtk_combo_box_new_text();
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_estyle_w), 0, "line");
   gtk_combo_box_insert_text(GTK_COMBO_BOX(_estyle_w), 1, "capped");
   gtk_combo_box_set_active(GTK_COMBO_BOX(_estyle_w), _estyle);
   GtkWidget* eth_w = gtk_label_new("Thickness");
   th_adj = (GtkAdjustment *)gtk_adjustment_new(_ethick,0.5,10.0,1.0,1.0,0.0);
   _ethick_w   = gtk_spin_button_new(th_adj, 1.0, 1);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_ethick_w), 1);
   gtk_spin_button_set_update_policy(GTK_SPIN_BUTTON(_ethick_w),
      GTK_UPDATE_IF_VALID);
   GtkWidget* ec_w = gtk_label_new("Color");
   gdk_color_parse(_ecolor.c_str(), &clr);
   _ecolor_w = gtk_color_button_new_with_color(&clr);
   gtk_color_button_set_title(GTK_COLOR_BUTTON(_ecolor_w), 
      "Error Bar Color Selection"); 
   _ecolord_w = gtk_check_button_new_with_label("Default");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_ecolord_w), _ecolord); 
   g_signal_connect(_ecolord_w,"toggled",G_CALLBACK(toggled_value_callback),
                    (void*)this);

   // behavior toggles 
   GtkWidget* bh_w = gtk_label_new("Behavior");
   _oplot_w = gtk_radio_button_new_with_label(NULL, "Overplot");
   _nframe_w = gtk_radio_button_new_with_label_from_widget(
      GTK_RADIO_BUTTON(_oplot_w),"New Window");
   _erase_w = gtk_radio_button_new_with_label_from_widget(
      GTK_RADIO_BUTTON(_oplot_w),"Erase Existing"); 
   if (_behavior == 0) 
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_oplot_w), true);
   else if (_behavior == 1) 
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_nframe_w), true);
   else
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_erase_w), true);

   set_color_buttons_sensitive(); 

   gtk_table_attach_defaults(GTK_TABLE(table), cvxl_w, 1,4,1,2);
   gtk_table_attach_defaults(GTK_TABLE(table), _cvx_w, 4,8,1,2);
   gtk_table_attach_defaults(GTK_TABLE(table), cvyl_w, 8,9,1,2);
   gtk_table_attach_defaults(GTK_TABLE(table), _cvy_w, 9,13,1,2);
   gtk_table_attach_defaults(GTK_TABLE(table), cvel_w, 13,15,1,2);
   gtk_table_attach_defaults(GTK_TABLE(table), _cve_w, 15,19,1,2);
   gtk_table_attach_defaults(GTK_TABLE(table), ll_w, 1,4,3,4);
   gtk_table_attach_defaults(GTK_TABLE(table), _lstyle_w, 4,8,3,4);
   gtk_table_attach_defaults(GTK_TABLE(table), lth_w, 8,10,3,4);
   gtk_table_attach_defaults(GTK_TABLE(table), _lthick_w, 10,12,3,4);
   gtk_table_attach_defaults(GTK_TABLE(table), lc_w, 12,14,3,4);
   gtk_table_attach_defaults(GTK_TABLE(table), _lcolord_w, 14,17,3,4);
   gtk_table_attach_defaults(GTK_TABLE(table), _lcolor_w, 17,19,3,4);
   gtk_table_attach_defaults(GTK_TABLE(table), sl_w, 1,4,4,5);
   gtk_table_attach_defaults(GTK_TABLE(table), _sstyle_w, 4,8,4,5);
   gtk_table_attach_defaults(GTK_TABLE(table), sth_w, 8,10,4,5);
   gtk_table_attach_defaults(GTK_TABLE(table), _sthick_w, 10,12,4,5);
   gtk_table_attach_defaults(GTK_TABLE(table), sc_w, 12,14,4,5);
   gtk_table_attach_defaults(GTK_TABLE(table), _scolord_w, 14,17,4,5);
   gtk_table_attach_defaults(GTK_TABLE(table), _scolor_w, 17,19,4,5);
   gtk_table_attach_defaults(GTK_TABLE(table), el_w, 1,4,5,6);
   gtk_table_attach_defaults(GTK_TABLE(table), _estyle_w, 4,8,5,6);
   gtk_table_attach_defaults(GTK_TABLE(table), eth_w, 8,10,5,6);
   gtk_table_attach_defaults(GTK_TABLE(table), _ethick_w, 10,12,5,6);
   gtk_table_attach_defaults(GTK_TABLE(table), ec_w, 12,14,5,6);
   gtk_table_attach_defaults(GTK_TABLE(table), _ecolord_w, 14,17,5,6);
   gtk_table_attach_defaults(GTK_TABLE(table), _ecolor_w, 17,19,5,6);
   gtk_table_attach_defaults(GTK_TABLE(table), bh_w, 1,5,7,8);
   gtk_table_attach_defaults(GTK_TABLE(table), _oplot_w, 6,10,7,8);
   gtk_table_attach_defaults(GTK_TABLE(table), _nframe_w, 10,14,7,8);
   gtk_table_attach_defaults(GTK_TABLE(table), _erase_w, 14,19,7,8);

   gtk_table_attach_defaults(GTK_TABLE(table), label_w, 1,5,9,10);
   gtk_table_attach_defaults(GTK_TABLE(table), cb_w, 6,14,9,10);
   gtk_table_attach_defaults(GTK_TABLE(table), _launch, 15,19,9,10);
   gtk_table_attach_defaults(GTK_TABLE(table), lab2_w, 1,5,10,11);
   gtk_table_attach_defaults(GTK_TABLE(table), _shell_no_w, 6,9,10,11);
   gtk_table_attach_defaults(GTK_TABLE(table), _shell_py_w, 10,13,10,11);


   gtk_widget_show_all(_scrolled_w);

  g_signal_connect(_cvx_w,"changed",G_CALLBACK(changed_value_callback),
                      (void*)this);
  g_signal_connect(_cvy_w,"changed",G_CALLBACK(changed_value_callback),
                      (void*)this);
  g_signal_connect(_cve_w,"changed",G_CALLBACK(changed_value_callback),
                      (void*)this);
  validate(); 
}

int PrismPlotCmd::show()
{
   generate_form(); 
   gtk_widget_show_all(base_w);
   return 0;
}

int PrismPlotCmd::doit(PrismDataSet* dset, string xcol, string ycol,
   string ecol, int dialog, string rowspec)
{
   bool use_dialog = false;

   if (!theApp) return 1;       // app does not exist 
   
   if (dialog == -1)
   {
      if (theApp->get_preference("data.plot.dialog", use_dialog))  
      use_dialog = true;
   }
   else if (dialog == 1)
      use_dialog = true; 
   
   if ((_dset = dset) == NULL) 
   {
      theApp->status_message("No data to plot in current tab"); 
      return 1; 
   }

   _rowspec = rowspec; 

   // make sure enough columns are specified for quick plot
   if (!use_dialog)
   {
      if (xcol.length() == 0 && ycol.length() == 0)
      {
         theApp->status_message("Unable to generate plot. Please select at least 2 columns (0 selected).",1);
         return 1; 
      } 
      else if (ycol.length() == 0)
      {
         theApp->status_message("Unable to generate plot. Please select at least 2 columns (1 selected- " + xcol + ").",1);
         return 1; 
      }
   }

   // keep last value if a blank is passed in 
   _xcol = xcol;
   _ycol = ycol;
   _errcol = ecol;

   if (use_dialog)
   {
      // generate a dialog to allow user to specify defaults 
      generate_form(); 

      int resp = GTK_RESPONSE_APPLY; 
      while (resp == GTK_RESPONSE_APPLY)
      {
         resp = gtk_dialog_run(GTK_DIALOG(base_w));
         if (resp == GTK_RESPONSE_APPLY || resp == GTK_RESPONSE_OK)
         {
            GdkColor clr;
            char hexcolor[16]; 
	    if (_entry)  _reqId = gtk_entry_get_text(_entry);
            if (_launch) 
	       _instantiate = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_launch));
	    // set behavior mode 
	    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_oplot_w)))
               _behavior = 0;
            else if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_nframe_w))) 
               _behavior = 1;
	    else 
	       _behavior = 2; 
   
	    _lstyle = gtk_combo_box_get_active(GTK_COMBO_BOX(_lstyle_w));
	    _sstyle = gtk_combo_box_get_active(GTK_COMBO_BOX(_sstyle_w));
	    _estyle = gtk_combo_box_get_active(GTK_COMBO_BOX(_estyle_w));
   
            _lthick = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_lthick_w));
            _sthick = (int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(_sthick_w));
            _ethick = gtk_spin_button_get_value(GTK_SPIN_BUTTON(_ethick_w));
            _shell_py= gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_shell_py_w));
            // get cols to plot
	    GtkEntry* entx_w = GTK_ENTRY(GTK_BIN(_cvx_w)->child);
            _xcol = gtk_entry_get_text(GTK_ENTRY(entx_w));
	    GtkEntry* enty_w = GTK_ENTRY(GTK_BIN(_cvy_w)->child);
            _ycol = gtk_entry_get_text(GTK_ENTRY(enty_w));
            GtkEntry* ente_w = GTK_ENTRY(GTK_BIN(_cve_w)->child);
            _errcol = gtk_entry_get_text(GTK_ENTRY(ente_w));
   
            if (!(_lcolord=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_lcolord_w))))
	    {
	       gtk_color_button_get_color(GTK_COLOR_BUTTON(_lcolor_w), &clr);
               sprintf(hexcolor,"#%02x%02x%02x",(clr.red)/256,(clr.green)/256,(clr.blue)/256); 
	       _lcolor = hexcolor; 
	    }
            if (!(_scolord=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_scolord_w))))
	    {
	       gtk_color_button_get_color(GTK_COLOR_BUTTON(_scolor_w), &clr);
	       sprintf(hexcolor,"#%02x%02x%02x",(clr.red)/256,(clr.green)/256,(clr.blue)/256); 
	       _scolor = hexcolor;
            }
            if (!(_ecolord=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_ecolord_w))))
	    {
	       gtk_color_button_get_color(GTK_COLOR_BUTTON(_ecolor_w), &clr);
               sprintf(hexcolor,"#%02x%02x%02x",(clr.red)/256,(clr.green)/256,(clr.blue)/256); 
               _ecolor = hexcolor;
            }
            generate_plot(); 
         }
      }
      gtk_widget_destroy(base_w);
   }
   else
   {
      // skip dialog and just generate an image using defaults
      _reqId = _chipsId; 
      _shell_py = false; 

      set_default_column_names(); 

      if (_xcol.length() && _collist.find(";"+_xcol+";") == string::npos)
      {
         theApp->status_message("Unable to generate plot. Please verify that column " + _xcol + " is not a vector or array column.", 1);
         return 1; 
      }
      if (_ycol.length() && _collist.find(";"+_ycol+";") == string::npos)
      {
         theApp->status_message("Unable to generate plot. Please verify that column " + _ycol + " is not a vector or array column.", 1);
         return 1; 
      }
      generate_plot(); 
   }
   return 0;
}

int PrismPlotCmd::generate_plot()
{
   char props[2048]; 
   PyObject *py_retval;
   int  zz; 
   string filespec, extnam, axlab = "";

   if (!_dset)  return 2;   // don't have a data file 

   filespec = _dset->get_filename();
   extnam = _dset->get_current_extname();

   // add row spec
   if (_rowspec.length() && filespec.find("[#row") == string::npos) 
   {
      filespec.append(_rowspec);
   }

   if (extnam.length())  filespec.append("["+extnam+"]");


   zz = 100; // a countdown to bail out so we can get back to the main process 
   while (g_main_context_iteration(NULL, FALSE) && zz--);   

   theApp->connect_plotter(_reqId);

   zz = 100; // a countdown to bail out so we can get back to the main process 
   while (g_main_context_iteration(NULL, FALSE) && zz--);   

   _chipsId = theApp->get_chipsId();

   if (_shell_py )
   {
      char namebuf[256];
      int pd = (int)getpid();
      char* lgname = getlogin();
      if (lgname)  sprintf(namebuf,"/tmp/launch_chips.%s.%1d", lgname, pd);
         else      sprintf(namebuf,"/tmp/launch_chips.%1d", pd);

      ofstream out(namebuf); 
      out << "connect(\"" << _chipsId << "\");" << endl;
      out.close();
      string cmd = "ASCDS_RUN_CONFIG=: chips -x ";
      cmd.append(namebuf);
      cmd.append(" &");
      system(cmd.c_str());
   }

   if (_xcol.length() && _ycol.length())
   {
      unsigned int ll; 
      string::size_type p1, p2;

      // build filter string
      string tmpfilt = "[cols ";
      tmpfilt.append(_xcol);
      tmpfilt.append(",");
      tmpfilt.append(_ycol);
      if (_errcol.length())
      {
         tmpfilt.append(",");
         tmpfilt.append(_errcol);
      }
      tmpfilt.append("]");
      if ((p1 = filespec.find("[cols ")) != string::npos)
      {
         if ((p2 = filespec.find("]", p1)) != string::npos)
         {
            string tmp = filespec.substr(0, p1) + tmpfilt + filespec.substr(p2+1);
            filespec = tmp;
         }
      }
      else
         filespec.append(tmpfilt);
      // setup axis labels- replacing underscores to avoid latex subscripting
      axlab = "axis.x.label="; 
      for (ll = 0; ll < _xcol.length(); ll++)
         if (_xcol[ll] == '_') axlab.append("\\_");
	    else  axlab += _xcol[ll];
      axlab.append(" axis.y.label=");
      for (ll = 0; ll < _ycol.length(); ll++)
         if (_ycol[ll] == '_') axlab.append("\\_");
	    else  axlab += _ycol[ll];
   }
   else if (_xcol.length() || _ycol.length() || _errcol.length())
   {
      // don't have x and y - ignore col spcecifications for now
   }

   // refresh gui- since widgets in sheet seem to hang g_main_context_iter, add
   zz = 100; // a countdown to bail out so we can get back to the main process 
   while (g_main_context_iteration(NULL, FALSE) && zz--);   

   if ((py_retval = theApp->call_py_functions(NULL, "open_undo_block")) != NULL)
      Py_DECREF(py_retval);

   if (_behavior == 1)
   {
      if ((py_retval = theApp->call_py_functions(NULL, "add_window")) != NULL)
         Py_DECREF(py_retval);
   }
   else if (_behavior == 2) 
   {
      if ((py_retval = theApp->call_py_functions(NULL, "erase")) != NULL)
         Py_DECREF(py_retval);
   }

   zz = 100; // a countdown to bail out so we can get back to the main process 
   while (g_main_context_iteration(NULL, FALSE) && zz--);   

   // load attributes
   axlab.append(" line.color="); 
   if (_lcolord)   axlab.append("default");
   else        
   {
      if (_lcolor[0] == '#')
         axlab.append(_lcolor.substr(1, _lcolor.length()-1));
       else
         axlab.append(_lcolor);
   }
   axlab.append(" symbol.color="); 
   if (_scolord)   axlab.append("default");
   else
   {
      if (_scolor[0] == '#')
         axlab.append(_scolor.substr(1, _scolor.length()-1));
       else
         axlab.append(_scolor);
   }
   axlab.append(" err.color="); 
   if (_ecolord)   axlab.append("default");
   else
   {
      if (_ecolor[0] == '#')
         axlab.append(_ecolor.substr(1, _ecolor.length()-1));
       else
         axlab.append(_ecolor);
   }

   sprintf(props, "line.style=%d symbol.style=%d err.style=%d line.thickness=%1.1f symbol.size=%d err.thickness=%1.1f %s", 
      _lstyle, _sstyle, _estyle, _lthick, _sthick, _ethick, axlab.c_str()); 

   if (!(py_retval = theApp->call_py_functions(NULL, "add_curve", "%s %s", 
       filespec.c_str(), props)))
   {
      theApp->status_message("Failed to add curve of " + filespec + " to chips server '"+
         _chipsId + "'", 1);
      return 1; 
   }
   else
      theApp->status_message("Added curve of " + filespec + " to chips server '"+
         _chipsId + "'");
   Py_DECREF(py_retval);

   if ((py_retval = theApp->call_py_functions(NULL, "close_undo_block")) != NULL)
      Py_DECREF(py_retval);

   return 0;
}


int PrismPlotCmd::toggled_value_callback(GObject *object, gpointer user_data)
{
   PrismPlotCmd* cmd = (PrismPlotCmd*)user_data;

   if (!(cmd))  return 1;

   cmd->set_color_buttons_sensitive();
   return 0;
}


void PrismPlotCmd::set_color_buttons_sensitive()
{
   bool _toggle = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_lcolord_w));
   gtk_widget_set_sensitive(GTK_WIDGET(_lcolor_w), !_toggle);
   _toggle = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_scolord_w));
   gtk_widget_set_sensitive(GTK_WIDGET(_scolor_w), !_toggle);
   _toggle = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_ecolord_w));
   gtk_widget_set_sensitive(GTK_WIDGET(_ecolor_w), !_toggle);
}


void PrismPlotCmd::set_default_column_names(bool dialog)
{
   PrismDataBlock *bl = _dset->get_block();
   if (bl)
   {
      PrismDataColumn *dcols = bl->get_columns();
      long cc, rr, xfound = 0, yfound = 0, efound = 0, cnt = bl->get_totcols();
      string cname;

      _collist = ";"; 
      for (cc = rr = 0; rr < cnt; rr++)
         if (!(dcols[rr].is_array() || dcols[rr].is_array_comp() ||
	     dcols[rr].is_vector()))
         {
            cname = dcols[rr].get_name();
	    _collist.append(cname+";"); 
	    if (dialog)
	    {
               gtk_combo_box_append_text(GTK_COMBO_BOX(_cvx_w), cname.c_str());
               gtk_combo_box_append_text(GTK_COMBO_BOX(_cvy_w), cname.c_str());
               gtk_combo_box_append_text(GTK_COMBO_BOX(_cve_w), cname.c_str());
            }
            if (!xfound && cname == _xcol)
	    { 
               if (dialog)  gtk_combo_box_set_active(GTK_COMBO_BOX(_cvx_w), cc);
	       xfound = 1; 
	    }
            else if (!yfound && cname == _ycol)
	    {
               if (dialog)  gtk_combo_box_set_active(GTK_COMBO_BOX(_cvy_w), cc);
	       yfound = 1; 
	    }
            else if (!efound && cname == _errcol)
	    {
               if (dialog)  gtk_combo_box_set_active(GTK_COMBO_BOX(_cve_w), cc);
	       efound = 1; 
	    }
            cc++;
         }
      if (dialog)  gtk_combo_box_append_text(GTK_COMBO_BOX(_cve_w), "");
      if (!efound)  _errcol = ""; 
   }
}


void PrismPlotCmd::validate()
{
   bool valid =  false; 
   if (_cvx_w && _cvy_w)
   {
      GtkEntry* entx_w = GTK_ENTRY(GTK_BIN(_cvx_w)->child);
      GtkEntry* enty_w = GTK_ENTRY(GTK_BIN(_cvy_w)->child);
      GtkEntry* ente_w = GTK_ENTRY(GTK_BIN(_cve_w)->child);
      string xx = gtk_entry_get_text(GTK_ENTRY(entx_w)),
             yy = gtk_entry_get_text(GTK_ENTRY(enty_w)),
	     ee = gtk_entry_get_text(GTK_ENTRY(ente_w));
      if (xx.length() == 0 && yy.length() == 0)
         gtk_label_set_text(GTK_LABEL(_status_w), "Please specify values for Curve X and Y");
      else if (xx.length() == 0) 
         gtk_label_set_text(GTK_LABEL(_status_w), "Please specify a value for Curve X");
      else if (yy.length() == 0) 
         gtk_label_set_text(GTK_LABEL(_status_w), "Please specify a value for Curve Y");
      else if (xx == yy)
         gtk_label_set_text(GTK_LABEL(_status_w), "Please specify different columns for Curve X and Y");
      else if (_collist.find(";"+xx+";") == string::npos) 
         gtk_label_set_text(GTK_LABEL(_status_w), "Please specify a X column name from the pulldown options");  
      else if (_collist.find(";"+yy+";") == string::npos) 
         gtk_label_set_text(GTK_LABEL(_status_w), "Please specify a Y column name from the pulldown options");  
      else if (ee.length() && _collist.find(";"+ee+";") == string::npos)
         gtk_label_set_text(GTK_LABEL(_status_w), "Please specify an Errs column name from the pulldown options");  
      else if (ee.length() && ee == xx)
         gtk_label_set_text(GTK_LABEL(_status_w), "Please specify different columns for Curve X and Errs");  
      else if (ee.length() && ee == yy)
         gtk_label_set_text(GTK_LABEL(_status_w), "Please specify different columns for Curve Y and Errs");  
      else 
      {
         gtk_label_set_text(GTK_LABEL(_status_w), "");
         valid = true;
      }
   }
   gtk_dialog_set_response_sensitive(GTK_DIALOG(base_w), GTK_RESPONSE_OK, valid);
   gtk_dialog_set_response_sensitive(GTK_DIALOG(base_w), GTK_RESPONSE_APPLY, valid);
}

int PrismPlotCmd::changed_value_callback(GObject *object, gpointer user_data)
{
   PrismPlotCmd* cmd = (PrismPlotCmd*)user_data;
   if (!(cmd))  return 1;
   cmd->validate();
   return 0;
}

