// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef PRISMPREFS_HH
#define PRISMPREFS_HH

#include <stdio.h>
#include <string>
#include <list>

#define PPREF_MIN_ROWS_PP       1
#define PPREF_MAX_ROWS_PP   10000

using namespace std;

typedef struct
{
   string colname;
   string extname;
   string format;
} FORMAT_ENTRY, *FORMAT_ENTRY_P;

enum
{
   FORMAT_DOUBLE,
   FORMAT_FLOAT,
   FORMAT_LONG,
   FORMAT_SHORT,
   FORMAT_STRING
}; 



class PrismPrefs
{
  public:
    PrismPrefs();
    ~PrismPrefs();

    int load_preferences(string fname=""); 
    int save_preferences(string fname=""); 

    int get_preference(string prop, bool &val);
    int get_preference(string prop, long &val);
    int get_preference(string prop, string &val);

    bool display_tooltips()              { return _tips; }
    bool display_header_keyname()        { return _hdr_name_vis; }
    bool display_header_value()          { return _hdr_value_vis; }
    bool display_header_type()           { return _hdr_type_vis; }
    bool display_header_comment()        { return _hdr_comment_vis; }
    bool display_header_units()          { return _hdr_units_vis; }
    bool display_header_keyno()          { return _hdr_keyno_vis; }
    bool use_header_add_dialog()         { return _hdr_add_dialog; } 
    bool use_header_delete_dialog()      { return _hdr_delete_dialog; } 
    bool use_img_dialog()                { return _img_dialog; } 
    bool use_plot_dialog()               { return _plot_dialog; } 
    bool use_histo_dialog()              { return _histo_dialog; } 
    bool use_edit_confirm_dialog()       { return _confirm_dialog; }
    bool expand_vectors()                { return _v_expand; }
    bool expand_arrays()                 { return _a_expand; }
    bool display_info_description()      { return _info_desc_vis; }
    bool display_info_coltype()          { return _info_ctype_vis; }
    bool display_info_datatype()         { return _info_dtype_vis; }
    bool display_info_units()            { return _info_units_vis; }
    bool display_info_min()              { return _info_min_vis; }
    bool display_info_max()              { return _info_max_vis; }
    bool display_info_null()             { return _info_null_vis; }
    bool display_info_exist()            { return _info_exist_vis; }
    bool display_info_coord()            { return _info_coord_vis; }
    bool display_info_crpix()            { return _info_crpix_vis; }
    bool display_info_crval()            { return _info_crval_vis; }
    bool display_info_cdelt()            { return _info_cdelt_vis; }
    bool filter_raw_keywords()           { return _hdr_filter_raw; }
    bool error_text_bold()               { return _err_txt_bold; }
    bool error_text_italic()             { return _err_txt_italic; }
    bool alternate_rows()                { return _altrow; }

    string expand_vectors_filter()       { return _vexp_str; }
    string expand_arrays_filter()        { return _aexp_str; }
    string collapse_vectors_filter()     { return _vclp_str; }
    string collapse_arrays_filter()      { return _aclp_str; }
    string select_fgcolor()              { return _select_fg; }
    string select_bgcolor()              { return _select_bg; }
    string error_text_color()            { return _err_txt_color; }
    string oddrow_color()                { return _oddrow_color; }
    string evenrow_color()               { return _evenrow_color; }

    long get_rows_per_page()             { return _rowspp; }
    int  set_rows_per_page(long val);

    void set_tooltips(bool val)          { _tips = val; }
    void set_header_keyname(bool val)    { _hdr_name_vis = val; } 
    void set_header_value(bool val)      { _hdr_value_vis = val; } 
    void set_header_type(bool val)       { _hdr_type_vis = val; } 
    void set_header_comment(bool val)    { _hdr_comment_vis = val; } 
    void set_header_units(bool val)      { _hdr_units_vis = val; } 
    void set_header_keyno(bool val)      { _hdr_keyno_vis = val; } 
    void set_header_add_dialog(bool val) { _hdr_add_dialog = val; }
    void set_header_delete_dialog(bool val) { _hdr_delete_dialog = val; }
    void set_image_dialog(bool val)      { _img_dialog = val; } 
    void set_plot_dialog(bool val)       { _plot_dialog = val; } 
    void set_histogram_dialog(bool val)  { _histo_dialog = val; } 
    void set_edit_confirm_dialog(bool val) { _confirm_dialog = val; }
    void set_expand_vectors(bool val)    { _v_expand = val; }
    void set_expand_arrays(bool val)     { _a_expand = val; }
    void set_select_fgcolor(string val)  { _select_fg = val; }
    void set_select_bgcolor(string val)  { _select_bg = val; }

    void set_info_description(bool val)  { _info_desc_vis = val; }
    void set_info_coltype(bool val)      { _info_ctype_vis = val; }
    void set_info_datatype(bool val)     { _info_dtype_vis = val; }
    void set_info_units(bool val)        { _info_units_vis = val; }
    void set_info_min(bool val)          { _info_min_vis = val; }
    void set_info_max(bool val)          { _info_max_vis = val; }
    void set_info_null(bool val)         { _info_null_vis = val; }
    void set_info_exist(bool val)        { _info_exist_vis = val; }
    void set_info_coord(bool val)        { _info_coord_vis = val; }
    void set_info_crpix(bool val)        { _info_crpix_vis = val; }
    void set_info_crval(bool val)        { _info_crval_vis = val; }
    void set_info_cdelt(bool val)        { _info_cdelt_vis = val; }
    void set_filter_raw_keywords(bool val) { _hdr_filter_raw = val; }

    void set_expand_vectors_filter(string val); 
    void set_expand_arrays_filter(string val); 
    void set_collapse_vectors_filter(string val); 
    void set_collapse_arrays_filter(string val); 
  
    bool check_column_expansion(string colname, int vdim, int adim); 

    // format support
    int handle_data_format(string val); 
    string get_format(string cname, string ename, int type);

  protected: 
    int  parse_line(const char *in_txt, char del);  
    void initialize();             // set initial values 
    void extract_exception_range(string val, list<string> &lst, long &emin, long &emax); 

    string filename;               // name of loaded preferences file

    // general 
    bool _tips;                    // show tool tips 
    bool _info_dialogs;            // use info dialogs to display messages
    bool _confirm_dialog;          // dialog on first edit lock change

    // data table
    bool _img_dialog;              // data toolbar img button invokes dialog
    bool _plot_dialog;             // data toolbar plot button invokes dialog
    bool _histo_dialog;            // data toolbar histo button invokes dialog

    // expand/collapse preferences
    bool _v_expand;                // vector expansion 
    bool _a_expand;                // array expansion  
    long _vexp_min;                // min expansion range for vectors
    long _vexp_max;                // max expansion range for vectors
    long _vclp_min;                // min collapse range for vectors
    long _vclp_max;                // max collapse range for vectors
    long _aexp_min;                // min expansion range for arrays
    long _aexp_max;                // max expansion range for arrays
    long _aclp_min;                // min collapse range for arrays
    long _aclp_max;                // max collapse range for arrays
    string _vclp_str;              // vector collapse exception list 
    string _aclp_str;              // array collapse exception list 
    string _vexp_str;              // vector expand exception list 
    string _aexp_str;              // array expand exception list 
    list<string> _vexp_items;      // list of vector expansion filter entries
    list<string> _aexp_items;      // list of array expansion filter entries
    list<string> _vclp_items;      // list of vector collapse filter entries
    list<string> _aclp_items;      // list of array collapse filter entries

    string _select_fg;             // selected foreground color 
    string _select_bg;             // selected background color 
    string _err_txt_color;         // color for error text 
    string _oddrow_color;          // optional odd row color 
    string _evenrow_color;         // optional even row color 

    list <FORMAT_ENTRY_P> fmt_double;
    list <FORMAT_ENTRY_P> fmt_float;
    list <FORMAT_ENTRY_P> fmt_long;
    list <FORMAT_ENTRY_P> fmt_short;
    list <FORMAT_ENTRY_P> fmt_string;

    // header table 
    bool _hdr_name_vis;            // display keyword name
    bool _hdr_value_vis;           // display keyword value
    bool _hdr_type_vis;            // display keyword type 
    bool _hdr_comment_vis;         // display keyword comment 
    bool _hdr_units_vis;           // display keyword units 
    bool _hdr_keyno_vis;           // display keyword keyno  

    bool _hdr_add_dialog;          // header toolbar add button invokes dialog
    bool _hdr_delete_dialog;       // header toolbar delete button invokes dialog
    bool _hdr_filter_raw;          // filter out raw header keywords 

    bool _info_desc_vis;           // display description in info dialog  
    bool _info_ctype_vis;          // display column type in info dialog  
    bool _info_dtype_vis;          // display data type in info dialog  
    bool _info_units_vis;          // display units in info dialog  
    bool _info_min_vis;            // display range min in info dialog  
    bool _info_max_vis;            // display range max in info dialog  
    bool _info_null_vis;           // display tnull in info dialog  
    bool _info_exist_vis;          // display existence  in info dialog  
    bool _info_coord_vis;          // display coords in info dialog  
    bool _info_crpix_vis;          // display crpix in info dialog  
    bool _info_crval_vis;          // display crval in info dialog  
    bool _info_cdelt_vis;          // display cdelt in info dialog  

    bool _err_txt_bold;            // display status errs in bold
    bool _err_txt_italic;          // display status errs in italic
    bool _altrow;                  // alternate row colors 
 
    long _rowspp;                  // rows per page

};


#endif 
