// 
//  Copyright (C) 2008,2013  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismPrefsDialog.hh"
#include "PrismPrefs.hh"

#include <iostream>

using namespace std;

// static members
bool PrismPrefsDialog::_hdrexp = false; 
bool PrismPrefsDialog::_datexp = false; 
bool PrismPrefsDialog::_infoexp = false; 

PrismPrefsDialog::PrismPrefsDialog(PrismPrefs *prefs):PrismBaseObj()
{
   _prefs = prefs; 
   load_prefs();
}

PrismPrefsDialog::~PrismPrefsDialog()
{
}

void PrismPrefsDialog::generate_form()
{
   base_w = gtk_dialog_new_with_buttons ("Prism Prefereneces", NULL,
      GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_SAVE, GTK_RESPONSE_APPLY,
      GTK_STOCK_OK, GTK_RESPONSE_OK,GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, NULL);
   gtk_widget_set_size_request(base_w, 300, 300);
   _scrolled = gtk_scrolled_window_new (NULL,NULL);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), _scrolled);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(_scrolled),
      GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

   _vbox = gtk_vbox_new (FALSE, 5);
   gtk_container_set_border_width(GTK_CONTAINER(_vbox), 5);
   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(_scrolled), _vbox);

   _tips_w = gtk_check_button_new_with_label("Display Tool Tips");
   _conf_dlg_w = gtk_check_button_new_with_label("Edit Confirmation Dialog");

   GtkAdjustment *row_adj = (GtkAdjustment *) gtk_adjustment_new (_rowspp,
      PPREF_MIN_ROWS_PP, PPREF_MAX_ROWS_PP, 10.0, 10.0, 0.0);
   _data_rows_w   = gtk_spin_button_new(row_adj, 1.0, 0);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(_data_rows_w), 1);
   GtkWidget* hbox = gtk_hbox_new (FALSE, 5);
   GtkWidget* lab0 = gtk_label_new("Rows per Page"); 
   gtk_box_pack_start(GTK_BOX(hbox), lab0, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), _data_rows_w, FALSE, FALSE, 0);

   _hdr_exp = gtk_expander_new("Header Display Options");
   _hdr_table = gtk_table_new(3,11, false);
   gtk_container_add (GTK_CONTAINER (_hdr_exp), _hdr_table); 
   GtkWidget *lab = gtk_label_new("Display Columns:");
   _hdr_raw_w = gtk_check_button_new_with_label("Filter Raw Header Keywords");
   _disp_name_w = gtk_check_button_new_with_label("Key Name");
   _disp_value_w = gtk_check_button_new_with_label("Value");
   _disp_type_w = gtk_check_button_new_with_label("Type");
   _disp_cmt_w = gtk_check_button_new_with_label("Comment");
   _disp_units_w = gtk_check_button_new_with_label("Units");
   _disp_keyno_w = gtk_check_button_new_with_label("Key Number");
   GtkWidget *lab2 = gtk_label_new("Toolbar Preferences:");
   _hdr_add_dlg_w = gtk_check_button_new_with_label("Dialog for Add Button");
   _hdr_del_dlg_w = gtk_check_button_new_with_label("Dialog for Delete Button");

   _exp_vector_w = gtk_radio_button_new_with_label(NULL, "Expand Vectors");
   _clp_vector_w = gtk_radio_button_new_with_label_from_widget(
      GTK_RADIO_BUTTON(_exp_vector_w), "Collapse Vectors");
   _exp_array_w = gtk_radio_button_new_with_label(NULL, "Expand Arrays");
   _clp_array_w = gtk_radio_button_new_with_label_from_widget(
      GTK_RADIO_BUTTON(_exp_array_w), "Collapse Arrays");

   gtk_table_attach_defaults(GTK_TABLE(_hdr_table), _hdr_raw_w, 0,2,1,2);
   gtk_table_attach_defaults(GTK_TABLE(_hdr_table), lab, 0,2,2,3);
   gtk_table_attach_defaults(GTK_TABLE(_hdr_table), _disp_name_w, 0,1,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_hdr_table), _disp_value_w, 1,2,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_hdr_table), _disp_type_w, 2,3,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_hdr_table), _disp_cmt_w, 0,1,5,6);
   gtk_table_attach_defaults(GTK_TABLE(_hdr_table), _disp_units_w, 1,2,5,6);
   gtk_table_attach_defaults(GTK_TABLE(_hdr_table), _disp_keyno_w, 2,3,5,6);
   gtk_table_attach_defaults(GTK_TABLE(_hdr_table), lab2, 0,2,6,7);
   gtk_table_attach_defaults(GTK_TABLE(_hdr_table), _hdr_add_dlg_w, 0,2,8,9);
   gtk_table_attach_defaults(GTK_TABLE(_hdr_table), _hdr_del_dlg_w, 0,2,9,10);
   
   _data_exp = gtk_expander_new("Data Display Options");
   _data_table = gtk_table_new(3,17, false);
   gtk_container_add(GTK_CONTAINER(_data_exp), _data_table); 
   _data_plot_dlg_w = gtk_check_button_new_with_label("Dialog for Plot Button");
   _data_hist_dlg_w=gtk_check_button_new_with_label("Dialog for Histogram Button");
   _data_img_dlg_w = gtk_check_button_new_with_label("Dialog for Image Button");
   GtkWidget *lab3 = gtk_label_new("Toolbar Preferences:");
   gtk_table_attach_defaults(GTK_TABLE(_data_table), lab3, 0,2,2,3);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), _data_plot_dlg_w, 0,2,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), _data_hist_dlg_w, 0,2,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), _data_img_dlg_w, 0,2,5,6);

   GtkWidget *lab4 = gtk_label_new("Column Expansion:");
   GtkWidget *lab5 = gtk_label_new("except"); 
   _exp_ve_w= gtk_entry_new();
   GtkWidget *lab6 = gtk_label_new("except"); 
   _clp_ve_w= gtk_entry_new();
   GtkWidget *lab7 = gtk_label_new("except"); 
   _exp_ae_w= gtk_entry_new();
   GtkWidget *lab8 = gtk_label_new("except"); 
   _clp_ae_w= gtk_entry_new();

   GtkWidget* lab9 = gtk_label_new("Select Foreground Color");
   GdkColor clr;
   gdk_color_parse(_selectfg.c_str(), &clr);
   _selectfg_w = gtk_color_button_new_with_color(&clr);
   gtk_color_button_set_title(GTK_COLOR_BUTTON(_selectfg_w),
                             "Selected Foreground Color"); 
   GtkWidget* lab10 = gtk_label_new("Select Background Color");
   gdk_color_parse(_selectbg.c_str(), &clr);
   _selectbg_w = gtk_color_button_new_with_color(&clr);
   gtk_color_button_set_title(GTK_COLOR_BUTTON(_selectbg_w),
                             "Selected Background Color"); 

   gtk_table_attach_defaults(GTK_TABLE(_data_table), lab4, 0,2,6,7);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), _exp_vector_w, 0,2,7,8);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), lab5, 0,1,8,9);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), _exp_ve_w, 1,2,8,9);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), _clp_vector_w, 0,2,9,10);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), lab6, 0,1,10,11);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), _clp_ve_w, 1,2,10,11);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), _exp_array_w, 0,2,11,12);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), lab7, 0,1,12,13);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), _exp_ae_w, 1,2,12,13);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), _clp_array_w, 0,2,13,14);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), lab8, 0,1,14,15);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), _clp_ae_w, 1,2,14,15);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), lab9, 0,2,15,16);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), _selectfg_w, 2,3,15,16);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), lab10, 0,2,16,17);
   gtk_table_attach_defaults(GTK_TABLE(_data_table), _selectbg_w, 2,3,16,17);

   _info_exp = gtk_expander_new("Info Dialog Options");
   _info_table = gtk_table_new(3,10, false);
   gtk_container_add (GTK_CONTAINER (_info_exp), _info_table); 
   lab = gtk_label_new("Display Columns:");
   _info_desc_w = gtk_check_button_new_with_label("Description");
   _info_ctype_w = gtk_check_button_new_with_label("Column Type");
   _info_dtype_w = gtk_check_button_new_with_label("Data Type");
   _info_units_w = gtk_check_button_new_with_label("Units");
   _info_min_w = gtk_check_button_new_with_label("Minimum");
   _info_max_w = gtk_check_button_new_with_label("Maximum");
   _info_null_w = gtk_check_button_new_with_label("Null");
   _info_exist_w = gtk_check_button_new_with_label("Existence");
   _info_coord_w = gtk_check_button_new_with_label("Coord Sys");
   _info_crpix_w = gtk_check_button_new_with_label("Crpix");
   _info_crval_w = gtk_check_button_new_with_label("Crval");
   _info_cdelt_w = gtk_check_button_new_with_label("Cdelt");

   gtk_table_attach_defaults(GTK_TABLE(_info_table), lab, 0,2,1,2);
   gtk_table_attach_defaults(GTK_TABLE(_info_table), _info_desc_w, 0,1,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_info_table), _info_ctype_w, 1,2,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_info_table), _info_dtype_w, 2,3,3,4);
   gtk_table_attach_defaults(GTK_TABLE(_info_table), _info_units_w, 0,1,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_info_table), _info_min_w, 1,2,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_info_table), _info_max_w, 2,3,4,5);
   gtk_table_attach_defaults(GTK_TABLE(_info_table), _info_null_w, 0,1,5,6);
   gtk_table_attach_defaults(GTK_TABLE(_info_table), _info_exist_w, 1,2,5,6);
   gtk_table_attach_defaults(GTK_TABLE(_info_table), _info_coord_w, 2,3,5,6);
   gtk_table_attach_defaults(GTK_TABLE(_info_table), _info_crpix_w, 0,1,6,7);
   gtk_table_attach_defaults(GTK_TABLE(_info_table), _info_crval_w, 1,2,6,7);
   gtk_table_attach_defaults(GTK_TABLE(_info_table), _info_cdelt_w, 2,3,6,7);

   gtk_box_pack_start(GTK_BOX(_vbox), _tips_w, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(_vbox), _conf_dlg_w, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(_vbox), hbox, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(_vbox), _hdr_exp, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(_vbox), _data_exp, FALSE, FALSE, 0);
   gtk_box_pack_start(GTK_BOX(_vbox), _info_exp, FALSE, FALSE, 0);
   
   load_dialog_prefs(); 

   show();
}

int PrismPrefsDialog::show()
{
   gtk_widget_show_all(base_w);
   return 0;
}


void PrismPrefsDialog::doit()
{
   generate_form(); 

   int resp = gtk_dialog_run(GTK_DIALOG(base_w)); 

   if (resp == GTK_RESPONSE_APPLY)
   {
      // save to file 
      save_prefs();
   }
   else if (resp == GTK_RESPONSE_OK)
   {
      // update local preferences 
      commit_prefs();
   }
   gtk_widget_destroy(base_w);
}

void PrismPrefsDialog::load_prefs()
{
   if (!_prefs) return; 
   _rowspp = _prefs->get_rows_per_page(); 
   _ttips = _prefs->display_tooltips(); 
   _conf = _prefs->use_edit_confirm_dialog(); 
   _hdrraw = _prefs->filter_raw_keywords();
   _hdrname = _prefs->display_header_keyname();
   _hdrvalue = _prefs->display_header_value();
   _hdrtype = _prefs->display_header_type();
   _hdrcomment = _prefs->display_header_comment();
   _hdrunit = _prefs->display_header_units();
   _hdrkeyno = _prefs->display_header_keyno();
   _hdradd = _prefs->use_header_add_dialog();
   _hdrdel = _prefs->use_header_delete_dialog();

   _datplot = _prefs->use_plot_dialog();
   _datimg = _prefs->use_img_dialog();
   _dathist = _prefs->use_histo_dialog();

   _expv = _prefs->expand_vectors();
   _expa = _prefs->expand_arrays();
   _expv_filt = _prefs->expand_vectors_filter(); 
   _expa_filt = _prefs->expand_arrays_filter(); 
   _clpv_filt = _prefs->collapse_vectors_filter(); 
   _clpa_filt = _prefs->collapse_arrays_filter(); 
   _selectfg = _prefs->select_fgcolor(); 
   _selectbg = _prefs->select_bgcolor(); 
   gdk_color_parse(_selectfg.c_str(), &_prevfg);
   gdk_color_parse(_selectbg.c_str(), &_prevbg);

   _info_desc = _prefs->display_info_description();
   _info_ctype = _prefs->display_info_coltype();
   _info_dtype = _prefs->display_info_datatype();
   _info_units = _prefs->display_info_units();
   _info_min  = _prefs->display_info_min();
   _info_max  = _prefs->display_info_max();
   _info_null = _prefs->display_info_null();
   _info_exists = _prefs->display_info_exist();
   _info_coord = _prefs->display_info_coord();
   _info_crpix = _prefs->display_info_crpix();
   _info_crval = _prefs->display_info_crval();
   _info_cdelt = _prefs->display_info_cdelt();
}

void PrismPrefsDialog::commit_prefs()
{
   retrieve_dialog_prefs(); 

   if (!_prefs) return; 

   _prefs->set_tooltips(_ttips);
   _prefs->set_edit_confirm_dialog(_conf);

   _prefs->set_filter_raw_keywords(_hdrraw);
   _prefs->set_header_keyname(_hdrname); 
   _prefs->set_header_value(_hdrvalue); 
   _prefs->set_header_type(_hdrtype); 
   _prefs->set_header_comment(_hdrcomment); 
   _prefs->set_header_units(_hdrunit); 
   _prefs->set_header_keyno(_hdrkeyno); 
   _prefs->set_header_add_dialog(_hdradd);
   _prefs->set_header_delete_dialog(_hdrdel);

   _prefs->set_image_dialog(_datimg);
   _prefs->set_plot_dialog(_datplot);
   _prefs->set_histogram_dialog(_dathist);
   _prefs->set_rows_per_page(_rowspp);

   _prefs->set_expand_arrays(_expa);
   _prefs->set_expand_vectors(_expv);
   _prefs->set_expand_arrays_filter(_expa_filt); 
   _prefs->set_expand_vectors_filter(_expv_filt); 
   _prefs->set_collapse_arrays_filter(_clpa_filt); 
   _prefs->set_collapse_vectors_filter(_clpv_filt); 
   _prefs->set_select_fgcolor(_selectfg);
   _prefs->set_select_bgcolor(_selectbg);

   _prefs->set_info_description(_info_desc);
   _prefs->set_info_coltype(_info_ctype);
   _prefs->set_info_datatype(_info_dtype);
   _prefs->set_info_units(_info_units);
   _prefs->set_info_min(_info_min );
   _prefs->set_info_max(_info_max );
   _prefs->set_info_null(_info_null);
   _prefs->set_info_exist(_info_exists);
   _prefs->set_info_coord(_info_coord);
   _prefs->set_info_crval(_info_crval);
   _prefs->set_info_crpix(_info_crpix);
   _prefs->set_info_cdelt(_info_cdelt);
}

void PrismPrefsDialog::save_prefs()
{
   if (!_prefs) return; 
   commit_prefs();
   _prefs->save_preferences(); 
}

void PrismPrefsDialog::load_dialog_prefs()
{
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_tips_w), _ttips);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_conf_dlg_w), _conf);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_hdr_raw_w), _hdrraw);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_disp_name_w), _hdrname);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_disp_value_w), _hdrvalue);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_disp_type_w), _hdrtype);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_disp_cmt_w), _hdrcomment);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_disp_units_w), _hdrunit);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_disp_keyno_w), _hdrkeyno);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_hdr_add_dlg_w), _hdradd);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_hdr_del_dlg_w), _hdrdel);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_data_plot_dlg_w), _datplot);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_data_hist_dlg_w), _dathist);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_data_img_dlg_w), _datimg);

   if (_expv)
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_exp_vector_w), _expv);
   else 
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_clp_vector_w), true);
   if (_expa)
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_exp_array_w), _expa);
   else
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_clp_array_w), true);
   gtk_entry_set_text(GTK_ENTRY(_exp_ve_w), _expv_filt.c_str()); 
   gtk_entry_set_text(GTK_ENTRY(_exp_ae_w), _expa_filt.c_str()); 
   gtk_entry_set_text(GTK_ENTRY(_clp_ve_w), _clpv_filt.c_str()); 
   gtk_entry_set_text(GTK_ENTRY(_clp_ae_w), _clpa_filt.c_str()); 
   gtk_expander_set_expanded(GTK_EXPANDER(_hdr_exp), _hdrexp);
   gtk_expander_set_expanded(GTK_EXPANDER(_data_exp), _datexp);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(_data_rows_w), _rowspp); 
   GdkColor clr;
   gdk_color_parse(_selectfg.c_str(), &clr);
   gtk_color_button_set_color(GTK_COLOR_BUTTON(_selectfg_w), &clr);
   gdk_color_parse(_selectbg.c_str(), &clr);
   gtk_color_button_set_color(GTK_COLOR_BUTTON(_selectbg_w), &clr);

   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_info_desc_w), _info_desc);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_info_ctype_w), _info_ctype);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_info_dtype_w), _info_dtype);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_info_units_w), _info_units);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_info_min_w), _info_min);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_info_max_w), _info_max);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_info_null_w), _info_null);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_info_exist_w), _info_exists);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_info_coord_w), _info_coord);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_info_crpix_w), _info_crpix);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_info_crval_w), _info_crval);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_info_cdelt_w), _info_cdelt);
}

void PrismPrefsDialog::retrieve_dialog_prefs()
{
   _ttips = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_tips_w));
   _conf = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_conf_dlg_w));
   _hdrraw = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_hdr_raw_w));
   _hdrname = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_disp_name_w));
   _hdrvalue = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_disp_value_w));
   _hdrtype = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_disp_type_w));
   _hdrcomment = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_disp_cmt_w));
   _hdrunit = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_disp_units_w));
   _hdrkeyno = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_disp_keyno_w));
   _hdradd = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_hdr_add_dlg_w));
   _hdrdel = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_hdr_del_dlg_w));
   _datplot = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_data_plot_dlg_w));
   _dathist = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_data_hist_dlg_w));
   _datimg = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_data_img_dlg_w));
   _expv = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_exp_vector_w));
   _expa = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_exp_array_w));
   _expv_filt = gtk_entry_get_text(GTK_ENTRY(_exp_ve_w));
   _expa_filt = gtk_entry_get_text(GTK_ENTRY(_exp_ae_w));
   _clpv_filt = gtk_entry_get_text(GTK_ENTRY(_clp_ve_w));
   _clpa_filt = gtk_entry_get_text(GTK_ENTRY(_clp_ae_w));
   _hdrexp = gtk_expander_get_expanded(GTK_EXPANDER(_hdr_exp));
   _datexp = gtk_expander_get_expanded(GTK_EXPANDER(_data_exp)); 
   _rowspp = (long)gtk_spin_button_get_value(GTK_SPIN_BUTTON(_data_rows_w)); 
   // when we migrate to gtk 2.12, replace w/ gdk_color_to_string
   GdkColor clr;
   gtk_color_button_get_color(GTK_COLOR_BUTTON(_selectfg_w), &clr);
   char hexconv[32];
   int rr, gg, bb;
   if (!gdk_color_equal(&clr, &_prevfg))
   {
      // since we're converting to rrggbb- only update if the original changed
      if ((rr = clr.red) > 0) rr--;
      if ((gg = clr.green) > 0) gg--;
      if ((bb = clr.blue) > 0) bb--;
      sprintf(hexconv, "#%02x%02x%02x", rr/256, gg/256, bb/256); 
      _selectfg = hexconv; 
   }
   gtk_color_button_get_color(GTK_COLOR_BUTTON(_selectbg_w), &clr);
   if (!gdk_color_equal(&clr, &_prevbg))
   {
      // since we're converting to rrggbb- only update if the original changed
      if ((rr = clr.red) > 0) rr--;
      if ((gg = clr.green) > 0) gg--;
      if ((bb = clr.blue) > 0) bb--;
      sprintf(hexconv, "#%02x%02x%02x", rr/256, gg/256, bb/256); 
      _selectbg = hexconv; 
   }
   // info dialog preferences 
   _info_desc  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_info_desc_w));
   _info_ctype = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_info_ctype_w));
   _info_dtype = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_info_dtype_w));
   _info_units = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_info_units_w));
   _info_min   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_info_min_w));
   _info_max   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_info_max_w));
   _info_null  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_info_null_w));
   _info_exists = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_info_exist_w));
   _info_coord = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_info_coord_w));
   _info_crpix = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_info_crpix_w));
   _info_crval = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_info_crval_w));
   _info_cdelt = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_info_cdelt_w));
}



