// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismRefDialog.hh"
#include "PrismApp.hh"

#include <fstream>
#include <iostream>
#include <map>

using namespace std;

// track the ref files by name sos that we only allow a single dialog
// per file to be displayed at any time 
map<string, bool, less<string> > PrismRefDialog::_active;

PrismRefDialog::PrismRefDialog(string tname, string fname):PrismBaseObj()
{
   _title = tname; 
   _filename = fname; 
}

PrismRefDialog::~PrismRefDialog()
{
}

void PrismRefDialog::generate_form()
{
   string ss; 

   if (_filename.length() == 0)   return;
   // if we are currently displaying the ref file - exit
   if (_active.count(_filename) && _active[_filename]) return;
   _active[_filename] = true;    

   base_w = gtk_dialog_new_with_buttons (_title.c_str(), NULL,
      GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_OK, GTK_RESPONSE_NONE, NULL);
   gtk_widget_set_size_request(base_w, 450, 200);
   // set up to delete dialog resources on response 
   g_signal_connect_swapped(base_w, "response", G_CALLBACK(PrismRefDialog::cleanup),
      this);

   _scrolled = gtk_scrolled_window_new (NULL,NULL);
   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(base_w)->vbox), _scrolled);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(_scrolled),
      GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

   _view = gtk_text_view_new();
   _buffer =  gtk_text_view_get_buffer(GTK_TEXT_VIEW(_view));

   if (_filename.length())
   {
      ifstream *infile =  new ifstream(_filename.c_str(), ios::in);
      char   linebuffer[256];

      if (infile->good())  
      {
         while (infile->getline(linebuffer, 256))
	 {
            gtk_text_buffer_insert_at_cursor(_buffer,linebuffer, strlen(linebuffer));
            gtk_text_buffer_insert_at_cursor(_buffer,"\n",1);
         }
      }
      else 
         gtk_text_buffer_set_text(_buffer, "Unable to open reference documentation!!!",-1);
   }
   else 
   {
      gtk_text_buffer_set_text(_buffer, "Unable to locate reference documentation!!!",-1);
   }
   gtk_container_add(GTK_CONTAINER(_scrolled), _view);
   gtk_widget_show_all(base_w);
}

int PrismRefDialog::show()
{
   generate_form(); 
   return 0;
}

int PrismRefDialog::doit()
{
   generate_form();
   if (gtk_dialog_run(GTK_DIALOG(base_w)) == GTK_RESPONSE_OK)
   {
     // nothing to do 
   }
   return 0; 
}

void PrismRefDialog::clear_entry()
{
   if (_filename.length())
   {
      if (_active.count(_filename))
         _active[_filename] = false; 
   }
   if (base_w)  gtk_widget_destroy(base_w); 
}

void PrismRefDialog::cleanup(PrismRefDialog *dialog, gint arg1, gpointer user_data)
{
   PrismRefDialog* prd = (PrismRefDialog*)dialog;
   // remove the reference count and destroy the dialog
   prd->clear_entry(); 
}


