// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismUtils.hh"
#include "ascdm.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <iostream>
#include <sstream>
#include <float.h>
 #include <string.h>
 #include <stdlib.h>
 
using namespace std; 


int _string_to_double(string val, double &outv)
{
   string tmp = val;
   tmp.erase(tmp.find_last_not_of(" \t")+1);
   istringstream ss(tmp);
   ss.precision(DBL_DIG);
   if ((ss >> outv) && ss.eof()) return 0;  
   return 1; 
}

int _string_to_float(string val, float &outv)
{
   string tmp = val;
   tmp.erase(tmp.find_last_not_of(" \t")+1);
   istringstream ss(tmp);
   ss.precision(FLT_DIG);
   if ((ss >> outv) && ss.eof()) return 0;  
   return 1; 
}

int _string_to_long(string val, long &outv)
{
   string tmp = val;
   tmp.erase(tmp.find_last_not_of(" \t")+1);
   istringstream ss(tmp);
   return (!((ss >> outv) && ss.eof()));
}

int _string_to_ulong(string val, unsigned long &outv)
{
   string tmp = val;
   tmp.erase(tmp.find_last_not_of(" \t")+1);
   istringstream ss(tmp);
   return (!((ss >> outv) && ss.eof()));
}

int _string_to_short(string val, short &outv)
{
   string tmp = val;
   tmp.erase(tmp.find_last_not_of(" \t")+1);
   istringstream ss(tmp);
   if ((ss >> outv) && ss.eof()) return 0;  
   return 1; 
}

int _string_to_ushort(string val, unsigned short &outv)
{
   string tmp = val;
   tmp.erase(tmp.find_last_not_of(" \t")+1);
   istringstream ss(tmp);
   if ((ss >> outv) && ss.eof()) return 0;  
   return 1; 
}

int _string_to_bool(string val, bool &outv)
{
   char cbuf[129];
   char *cp = cbuf; 

   string tmp2, tmp = val;
   tmp.erase(tmp.find_last_not_of(" \t")+1);
   istringstream ss(tmp);
   if (!((ss >> tmp2) && ss.eof())) return 1;

   strncpy(cbuf, tmp2.c_str(), 128); 
   while (*cp)
   {
      if (isupper(*cp)) *cp = tolower(*cp);
      cp++;
   }
   if  ((strcmp(cbuf, "yes") == 0) || (strcmp(cbuf, "y") == 0) || 
        (strcmp(cbuf, "true") == 0) || (strcmp(cbuf, "t") == 0) || 
        (strcmp(cbuf, "on") == 0) || (strcmp(cbuf, "1") == 0))
      outv = 1;
   else if  ((strcmp(cbuf, "no") == 0) || (strcmp(cbuf, "n") == 0) || 
        (strcmp(cbuf, "false") == 0) || (strcmp(cbuf, "f") == 0) || 
        (strcmp(cbuf, "off") == 0) || (strcmp(cbuf, "0") == 0))
      outv = 0;  
  else 
      return 1;   // unrecognized text as value 
   return 0; 
}

int _string_to_dmdatatype(string val, dmDataType &outv)
{
   char cbuf[129];
   char *cp = cbuf; 

   string tmp2, tmp = val;
   tmp.erase(tmp.find_last_not_of(" \t")+1);
   istringstream ss(tmp);
   if (!((ss >> tmp2) && ss.eof())) return 1;

   strncpy(cbuf, tmp2.c_str(), 128); 
   while (*cp)
   {
      if (isupper(*cp)) *cp = tolower(*cp);
      cp++;
   }
   if ((strcmp(cbuf, "string") == 0) || (strcmp(cbuf, "comment") == 0) ||
       (strcmp(cbuf, "history") == 0) || (strcmp(cbuf, "text") == 0) )
      outv = dmTEXT;
   else if ((strcmp(cbuf, "long") == 0) || (strcmp(cbuf, "int") == 0) ||
            (strcmp(cbuf, "integer") == 0))
      outv = dmLONG;
   else if (strcmp(cbuf, "short") == 0)
      outv = dmSHORT;
   else if (strcmp(cbuf, "float") == 0)
      outv = dmFLOAT;
   else if (strcmp(cbuf, "double") == 0)
      outv = dmDOUBLE;
   else if ((strcmp(cbuf, "bool") == 0) || (strcmp(cbuf, "boolean") == 0) ||
            (strcmp(cbuf, "logical") == 0))
      outv = dmBOOL;
   else if (strcmp(cbuf, "u.short") == 0)
      outv = dmUSHORT;
   else if (strcmp(cbuf, "u.long") == 0)
      outv = dmULONG;
   else 
      return 2;    // unsupported type 
   return 0; 
}

int _dmdatatype_to_string(dmDataType val, string &outv)
{
   switch(val)
   {
      case dmSHORT:  outv = "short"; break; 
      case dmLONG:  outv = "long"; break; 
      case dmFLOAT:  outv = "float"; break; 
      case dmDOUBLE:  outv = "double"; break; 
      case dmTEXT:  outv = "string"; break; 
      case dmBYTE:  outv = "u. char"; break; 
      case dmUSHORT:  outv = "u. short"; break; 
      case dmULONG:  outv = "u. long"; break; 
      case dmBIT:    outv= "bit";   break; 
      case dmBOOL:    outv= "boolean";   break; 
      default:
        return 2; 
      break; 
   }
   return 0; 
}


// returns 0 if values are equal (ignoring case) 
int _strcmp_cis(const char *str1, const char *str2)
{
   unsigned int ii;
   int status = 1; 
   if (str1 && str2)
   {
      if ((ii = strlen(str1)) == strlen(str2))
      {
         status = 0;
         while (ii-- && !status)
            if (tolower(str1[ii]) != tolower(str2[ii]))   status = 1;
      }
   }
   return status;
}

bool _file_writable(string fname)
{
   struct stat   sbuf;
   mode_t        mode;

   if (!stat(fname.c_str(), &sbuf))
      mode = sbuf.st_mode;
   else
      return 0;

   //Get the REAL user information to determine r/w privileges
   uid_t uid = getuid();
   gid_t gid = getgid();

   // Get the EFFECTIVE user information to determine r/w privileges
   uid_t euid = geteuid();
   gid_t egid = getegid();

   return ((mode & S_IWOTH) ||
       ((mode & S_IWUSR) && (uid == sbuf.st_uid || euid == sbuf.st_uid)) ||
       ((mode & S_IWGRP) && (gid == sbuf.st_gid || egid == sbuf.st_gid)));
}


// returns 0 if successful 
int _file_copy(string infile, string outfile, mode_t mode)
{
   int     rval = 0;
   struct stat buf, buf2;
 
   // blank file specified 
   if (!(infile.length() && outfile.length()))
      return -1;    // blank file name specified 

   if ((stat(infile.c_str(),&buf) == 0) && (stat(outfile.c_str(),&buf2) ==0))
      if (buf.st_ino == buf2.st_ino)   return -2;   // same file 

   if (!access(outfile.c_str(),F_OK) && !access(outfile.c_str(),W_OK))
      return -3;    // file alread exists but is read only 
   
    string cmd = "cp ";
    cmd.append(infile);
    cmd.append(" ");
    cmd.append(outfile);
    rval = system(cmd.c_str());
   if (rval == 0)   chmod(outfile.c_str(), mode);  // set file permissions
   return rval;
}

bool _file_executable(string fname)
{
   struct stat   sbuf;
   mode_t        mode;

   if (!stat(fname.c_str(), &sbuf))
      mode = sbuf.st_mode;
   else
      return 0;

   //Get the REAL user information to determine r/w privileges
   uid_t uid = getuid();
   gid_t gid = getgid();

   // Get the EFFECTIVE user information to determine r/w privileges
   uid_t euid = geteuid();
   gid_t egid = getegid();

   return ((mode & S_IXOTH) ||
       ((mode & S_IXUSR) && (uid == sbuf.st_uid || euid == sbuf.st_uid)) ||
       ((mode & S_IXGRP) && (gid == sbuf.st_gid || egid == sbuf.st_gid)));
}


// expand out env vars w/in supplied path. Blindy substitute so don't worry about
// unknown env vars or multiple entries with env var (ie PATH may have ".:/bin")  
string _path_expand(string str)
{
   string env, sub, path;
   char* val; 
   string::size_type start = 0, stop;

   path = str; 
   while ((start = path.find('$', start)) != string::npos)
   {
      if ((path[start+1]=='{') && ((stop=path.find('}',start)) != string::npos))
      {
         sub = path.substr(start+2, stop-2-start);
         if ((val = getenv(sub.c_str())) != NULL)  env = val; 
            else env = "";
         path.replace(start, stop-start+1, env);
      }
      else
      {
         if ((stop = path.find('/', start+1)) != string::npos)
	    sub = path.substr(start+1, stop-1-start);
         else
	 {
	    stop = path.length();
	    sub = path.substr(start+1, stop-start);
	 }
         if ((val = getenv(sub.c_str())) != NULL) env = val;  
            else env = "";
         path.replace(start, stop-start, env); 
      }
      start++;
   }
   return path;
}

// return true if path is a directory, false otherwise 
bool _is_directory(string path)
{
   struct stat   sbuf;
   mode_t        mode;

   if (!stat(path.c_str(), &sbuf))
      mode = sbuf.st_mode;
   else
      return 0;
   return (S_ISDIR(mode) && !(S_ISREG(mode) || S_ISCHR(mode) || 
           S_ISBLK(mode) || S_ISFIFO(mode))); 
}



// return true if valid format of    %{flag}{width}{.precision}{size}conv
// where optional flag    = [-|+|0| |#]*  
//       optional width   =  integer  
//       optional precision = .  integer 
//       optional size      = [h|l|L]
//       required conv      = [d|u|o|x|X|f|e|E|g|G|c|s]
// dtype- 0 double, 1 float, 2 long, 3 short, 4 string
bool _valid_format(string val, int dtype)
{
   char cv, buffer[256];
   int state = 0, len = val.length(); 
   char* start = &buffer[1]; 
   char *end, *period;

   if (len == 0)  return true;   // for our use- empty string is fine
   if (len == 1 || len > 255)  return false;  // too short/long 
   strcpy(buffer, val.c_str());
   if (buffer[0] != '%')  return false;  // must start w/ % 
   cv = buffer[len-1];
   buffer[len-1] = '\0'; 
   end = &buffer[len - 2]; 
   period = strchr(start, '.');
   if ((dtype == 2 || dtype == 3) && strchr("duoxXc", cv) == NULL)
      return false; 
   else if ((dtype == 0 || dtype == 1) && strchr("feEgG", cv) == NULL)
      return false; 
   else if (dtype == 4  && strchr("s", cv) == NULL) return false;

   if (len == 2) return true; // simple format- ie. %d, %g 
   if (*end == 'h' || *end == 'l' || *end == 'L') *end = '\0';  
   if (period) 
   {
      *period = '\0';
      period++; 
      while(period != end && isdigit(*period))  { *period = '\0'; period++;} 
   }
   while (*start != '\0')
   {
      if (strchr("-+#0 ", *start))
      {
         if ((!(state == 0 || state == 1)) && (*start != '0'))  return false; 
	 *start = '\0'; 
	 state = 1; 
	 start++;
      } 
      else if (isdigit(*start))
      {
	 state = 2;  
	 *start = '\0'; 
	 start++; 
      }
      else return false;  // invalid flag or field width 
   }
   return (strlen(&buffer[1]) == 0);
}




