// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PrismApp.hh"
#include "PrismVizMenu.hh"

using namespace std;

PrismVizMenu::PrismVizMenu(): PrismBaseObj()
{
   GtkWidget *sep = NULL; 

   base_w = gtk_menu_item_new_with_mnemonic("_Visualization");

   menu = gtk_menu_new();

   menuInterPlot = gtk_menu_item_new_with_label("Interactive Plot");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuInterPlot, 0);
   menuQuickPlot = gtk_menu_item_new_with_label("Quick Plot");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuQuickPlot, 1);
   menuPrintPlot = gtk_menu_item_new_with_label("Print Plot");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuPrintPlot, 2);
   sep = gtk_separator_menu_item_new(); 
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), sep, 3);

   menuInterHist = gtk_menu_item_new_with_label("Interactive Histogram");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuInterHist, 4);
   menuQuickHist = gtk_menu_item_new_with_label("Quick Histogram");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuQuickHist, 5);
   menuPrintHist = gtk_menu_item_new_with_label("Print Histogram");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuPrintHist, 6);
   sep = gtk_separator_menu_item_new(); 
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), sep, 7);

   menuImage = gtk_menu_item_new_with_label("Image");
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuImage, 8);

   g_signal_connect_swapped (G_OBJECT (menuInterPlot), "activate",
                             G_CALLBACK (inter_plot_cb),
                             (gpointer) NULL);
   g_signal_connect_swapped (G_OBJECT (menuQuickPlot), "activate",
                             G_CALLBACK (quick_plot_cb),
                             (gpointer) NULL);
   g_signal_connect_swapped (G_OBJECT (menuPrintPlot), "activate",
                             G_CALLBACK (print_plot_cb),
                             (gpointer) NULL);
   g_signal_connect_swapped (G_OBJECT (menuInterHist), "activate",
                             G_CALLBACK (inter_hist_cb),
                             (gpointer) NULL);
   g_signal_connect_swapped (G_OBJECT (menuQuickHist), "activate",
                             G_CALLBACK (quick_hist_cb),
                             (gpointer) NULL);
   g_signal_connect_swapped (G_OBJECT (menuPrintHist), "activate",
                             G_CALLBACK (print_hist_cb),
                             (gpointer) NULL);
   g_signal_connect_swapped (G_OBJECT (menuImage), "activate",
                             G_CALLBACK (image_cb),
                             (gpointer) NULL);
   gtk_menu_item_set_submenu (GTK_MENU_ITEM(base_w), menu);
}

PrismVizMenu::~PrismVizMenu()
{
   
}

int PrismVizMenu::show()
{
   gtk_widget_show(menuInterPlot);
   gtk_widget_show(menuQuickPlot);
   gtk_widget_show(menuPrintPlot);
   gtk_widget_show(menuInterHist);
   gtk_widget_show(menuQuickHist);
   gtk_widget_show(menuPrintHist);
   gtk_widget_show(menuImage);
   gtk_widget_show(base_w); 
   return 0; 
}

int PrismVizMenu::inter_plot_cb()
{
   return theApp->visualize_current_tab(0, 1);
}

int PrismVizMenu::quick_plot_cb()
{
   return theApp->visualize_current_tab(0, 0);
}

int PrismVizMenu::print_plot_cb()
{
   theApp->print_current_plot(); 
   return 0; 
}

int PrismVizMenu::inter_hist_cb()
{
   return theApp->visualize_current_tab(1, 1);
}

int PrismVizMenu::quick_hist_cb()
{
   return theApp->visualize_current_tab(1, 0);
}

int PrismVizMenu::print_hist_cb()
{
   theApp->print_current_plot(); 
   return 0; 
}

int PrismVizMenu::image_cb()
{
   return theApp->visualize_current_tab(2, 1);
}


