// 
//  Copyright (C) 2007-2008,2012  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <stdio.h>
#include <gtk/gtk.h>
#include <string>

#include "PrismApp.hh"
#include "cxcds_version.h"

using namespace std;

PrismApp *theApp = NULL;

int main(int argc, char *argv[])
{
   gchar *rcf = NULL, *tmf = NULL, *xname = NULL; 
   gboolean version = FALSE;
   GOptionEntry options[] = {
      { "rcfile",'r',0,G_OPTION_ARG_STRING,&rcf, "Alternate configuration (rc) file", NULL}, 
      { "toolmenu",'t',0,G_OPTION_ARG_STRING,&tmf, "Alternate Analysis Menu configuration file", NULL}, 
      { "version",'v',0,G_OPTION_ARG_NONE,&version, "Display program version", NULL},
      { "xpaname",'x',0,G_OPTION_ARG_STRING,&xname, "Specifies XPA access name", NULL},
      { NULL }
   };
   GError *errs = NULL; 
   string fname = "", tmfile = "", xpaname = "";
   string rclocal, rcname = getenv("HOME"); 
   char **fnames = NULL; 
   if (rcname.length()) rcname.append("/");
   rcname.append(".gtkrc"); 
   if (!gtk_init_with_args(&argc, &argv, "[FILE1 .. FILE-N] - launches prism, a graphical file viewer", options, NULL, &errs))
   {
      // see why the init failed  
      if (errs && errs->message) printf("ERROR: %s\n", errs->message); 
      return -1; 
   }

   if (version)
   {
      printf("Prism  - %s\n", CXCDS_VERSION_STRING); 
      printf("   gtk %1d.%1d.%1d   (compiled against %1d.%1d.%1d)\n",
         gtk_major_version, gtk_minor_version, gtk_micro_version,
	 (int)GTK_MAJOR_VERSION, (int)GTK_MINOR_VERSION, (int)GTK_MICRO_VERSION);  
      return 0;
   }

   if (tmf)
   {
      if (access(tmf, R_OK) == 0) tmfile = tmf;
   }
   if (rcf)
   {
      // rc file override provided on command line
      rclocal = rcf; 
      if (access(rclocal.c_str(), R_OK) == 0)   gtk_rc_parse(rclocal.c_str());
   }
   else
   {
      // rc file precedence: application specific, ciao specific, default
      rclocal = rcname + ".prism";
      if (access(rclocal.c_str(), R_OK) == 0)
         gtk_rc_parse(rclocal.c_str());
      else
      {
         rclocal = rcname + ".ciao";
         if (access(rclocal.c_str(), R_OK) == 0)
            gtk_rc_parse(rclocal.c_str());
         else
            gtk_rc_parse(rcname.c_str());
      }
   }
   if (xname)
      xpaname = xname;
   else
      xpaname = "prism";

   if (argc > 1)
   {
      fnames = new char*[argc];
      for (int ii = 1; ii < argc; ii++)  fnames[ii-1] = argv[ii];
      fnames[argc-1] = NULL; 
   }
   
   theApp = new PrismApp();
   theApp->initialize(tmfile, xpaname, fnames); 
   if (fnames)  delete []fnames; 
   gtk_main(); 

   return 0; 
}

