// 
//  Copyright (C) 2008-2009  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "TaskmonApp.hh"
#include "TaskmonCmdButtons.hh"

#include "CiaoGuiIPCClient.hh"
#include "CiaoGuiIPCComm.hh"
#include "CiaoGuiIPCMessage.hh"
#include "CiaoGuiIPCServer.hh"


#include "TaskmonMenu.hh"
#include "TaskmonTextArea.hh"
#include "TaskmonUtils.hh"


#include "TaskmonStatusArea.hh"

#include "cxcds_version.h"
#include <unistd.h>

#include <gtk/gtk.h>
#include <gtk/gtkmain.h>

using namespace std;

string TaskmonApp::statusId = "0"; 

TaskmonApp::TaskmonApp()
{
   _init = 0; 
   _tips = gtk_tooltips_new();
   _showtips = TRUE;
   reset_tips();
}

void TaskmonApp::initialize(string toolfile, string xpaname)
{
   if (!_init)
   {
    _height = 600;
    _width  = 500; 
    _window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

   _status = TASKM_UNKNOWN_STS; 

    // set some attributes of the GUI
      gtk_window_set_default_size(GTK_WINDOW(_window), _width, _height);
      gtk_window_set_title(GTK_WINDOW(_window), "Taskmonitor");
      gtk_container_set_border_width (GTK_CONTAINER(_window), 10);


    // Create the main vertical box as a holder of all the children
      _mainvbox = gtk_vbox_new(FALSE, 20);
      gtk_container_add (GTK_CONTAINER(_window), _mainvbox);
      gtk_container_set_border_width (GTK_CONTAINER (_window), 0);


    // Create the menu bar across the top of the vbox
      _topmenu  = new TaskmonMenu(toolfile, _tips); 
      gtk_box_pack_start(GTK_BOX(_mainvbox), _topmenu->base_widget(), 0, 0, 0);
      _topmenu->show(); 
      

    // now create the two frames and add them to the upper and lover pane as children
    // Create the Top Frame (COMMAND)
      _topframe = gtk_frame_new(NULL);
      gtk_frame_set_label(GTK_FRAME(_topframe), "Command");
      gtk_frame_set_label_align(GTK_FRAME(_topframe), 0.5, 0.0);
      gtk_frame_set_shadow_type(GTK_FRAME(_topframe), GTK_SHADOW_ETCHED_OUT);
      gtk_box_pack_start(GTK_BOX(_mainvbox), _topframe, 0, 0, 0);

      _commandArea = new TaskmonTextArea();
      gtk_container_add(GTK_CONTAINER(_topframe), _commandArea->base_widget());


    // create the bottom frame (OUTPUT)
      _bottomframe = gtk_frame_new(NULL);
      gtk_widget_set_size_request( _bottomframe, -1, 300);
      gtk_frame_set_label(GTK_FRAME(_bottomframe), "Output");
      gtk_frame_set_label_align(GTK_FRAME(_bottomframe), 0.5, 0.0);
      gtk_box_pack_start(GTK_BOX(_mainvbox), _bottomframe, FALSE, 0, 0);

      _outputArea = new TaskmonTextArea();
      gtk_container_add(GTK_CONTAINER(_bottomframe), _outputArea->base_widget());


      // Add the row of Command Buttons 
      	 _buttons = new TaskmonCmdButtons();
         gtk_box_pack_start(GTK_BOX(_mainvbox), _buttons->base_widget(), FALSE, 0, 0);

     // Add the application Status Report area

	GtkWidget *lab = gtk_label_new("Status:  ");
	GtkWidget* _hbox = gtk_hbox_new(false, 1); 

	_status_area = new TaskmonStatusArea();
        gtk_tooltips_set_tip(_tips, _status_area->base_widget(),
                  "Indicates the status of the application", NULL);

	gtk_box_pack_start(GTK_BOX(_hbox), lab, 0, 0, 0);
	gtk_box_pack_start(GTK_BOX(_hbox),_status_area->base_widget() , 1, 1, 10);
	gtk_box_pack_start(GTK_BOX(_mainvbox), _hbox, TRUE, 0, 10);


      // handle exit via <alt><f4> or close button in titlebar 
      g_signal_connect(_window, "destroy", G_CALLBACK(exit_cb), &_window);


      //gtk_widget_show_all(_scrolled);
      //gtk_widget_show_all(_vpane);
      gtk_widget_show_all(_window);

      statusId = _int_to_string(getpid()); 

      if (xpaname.length())  _xpaname = xpaname;
         else                _xpaname = "taskmonitor"; 

      // Create a new  CiaoGuiIPCServer instance
      add_ipc();

      _init = 1; 
  }
}

TaskmonApp::~TaskmonApp()
{
   cleanup();
   exit(0);
}

void TaskmonApp::exit_cb()
{
   theApp->cleanup();
   exit(0);
}

void TaskmonApp::exit_application()
{
   theApp->cleanup();
   exit(0);
}

string TaskmonApp::get_file_path()
{
       string fpath = "";
  //     if (thePData)  fpath = thePData->GetFilePath();
       return fpath; 


}

void TaskmonApp::cleanup()
{
  //   if (_window)
  //   {
  //      gtk_widget_destroy (_window);
  //      _window = NULL;
  //   }
   //   if (thePData)
   //   {
   //      delete thePData;
   //      thePData = NULL; 
   //   }

}

int TaskmonApp::command_message(string val)
{
     if (!_commandArea)  return -1;
     return _commandArea->add_message(val);
}

int TaskmonApp::output_message(string val)
{
    if (!_outputArea)  return -1;

      _outputArea->add_message(val);
      _outputArea->MoveToEnd();

      return(0);

}


int TaskmonApp::status_message(string val)
{
      if (!_status_area)  return -1;
      return _status_area->add_message(val);
  return(0);

}


void TaskmonApp::set_window_title(string val)
{
   if (_window)   gtk_window_set_title(GTK_WINDOW(_window), val.c_str());   
}


void TaskmonApp::reset_tips()
{
   if (_tips)
   {
      if (_showtips)
         gtk_tooltips_enable(GTK_TOOLTIPS(_tips));
      else
         gtk_tooltips_disable(GTK_TOOLTIPS(_tips));
   }
}




void TaskmonApp::add_ipc()
{
   ipc = new CiaoGuiIPCServer((char*)_xpaname.c_str(), 1);
   if (ipc)
   {
      ipc->AddCallback("status", &TaskmonApp::StatusMessageCB,
         "status         : retrieve application execution status");
      ipc->AddCallback("command", &TaskmonApp::CommandMsgCB,
         "command        : retrieve issued command");
      ipc->AddCallback("toolstatus", &TaskmonApp::ToolStatusMsgCB);

      // since find is currently not implemented- omit help so it isn't listed
      ipc->AddCallback("find", &TaskmonApp::FindMsgCB); 
//       "find <pattern> : returns whether the requested pattern was found");

      ipc->GtkLoopAdd();
   }
}


// -------------------------------------------------------------
// Find the string in the command output/stderr window
// -------------------------------------------------------------
bool TaskmonApp::Find(string str, bool caseflag, bool startflag)
{
  bool retval=0;

  //  retval = _out->Find(str, caseflag, startflag);

  return retval;
}


CiaoGuiIPCMessage* TaskmonApp::StatusMessageCB(CiaoGuiIPCMessage &in_msg)
{
   string sts;

   if (theApp->ProcessFinished())
   {
      sts = _int_to_string(theApp->GetStatus());
      if (sts == TASKM_UNKNOWN_STS_STR)   sts = "unknown"; 
   }
   else   sts = " ";

   CiaoGuiIPCMessage* returnMsg = new CiaoGuiIPCMessage((char*)sts.c_str());

   return (returnMsg);
}

/////////////////////////////////////////////////////////////////////////////
//
//     Routine: TaskApp::CommandMsgCB
//
// Description: handles IPC request for the command being monitored
//
//

CiaoGuiIPCMessage* TaskmonApp::CommandMsgCB(CiaoGuiIPCMessage&)
{
   string cmdname = theApp->GetCmdline();
   CiaoGuiIPCMessage* returnMsg = NULL;

   if (cmdname.length())
   {
      returnMsg = new CiaoGuiIPCMessage((char*)cmdname.c_str()  );
   }
 
   return (returnMsg);
}


/////////////////////////////////////////////////////////////////////////////
//
//     Routine: TaskApp::FindMsgCB
//
// Description: handles IPC request to find a string in the output window
//
//

CiaoGuiIPCMessage* TaskmonApp::FindMsgCB(CiaoGuiIPCMessage& in_msg)
{
   CiaoGuiIPCMessage* returnMsg = NULL;
   
   if (in_msg.GetTotalParams() == 2)
   {
      string pattern; 
      string res;
 
      in_msg.GetParam(1, pattern);

      if (pattern.length())
      {
         res = pattern;

         if (theApp->Find(pattern, 0, 0))
            res.append(" was found."); 
         else 
            res.append(" was not found."); 
         returnMsg = new CiaoGuiIPCMessage((char*)res.c_str());
      } 
   }
   return (returnMsg);
}


/////////////////////////////////////////////////////////////////////////////
//
//     Routine: TaskApp::ToolStatusMsgCB
//
// Description: set's tool completion status based on toolstatus message
//  WE NEVER GET HERE!!!!!
//

CiaoGuiIPCMessage* TaskmonApp::ToolStatusMsgCB(CiaoGuiIPCMessage& in_msg)
{
   if (in_msg.GetTotalParams() == 4)
   {
      string status, msgpid;
      in_msg.GetParam( 1, msgpid);
      if (statusId == msgpid) 
      {
         int val; 
         in_msg.GetParam( 3, status);
         if (!_string_to_int(status, val))
         {
            theApp->SetStatus(val); 
         }
      }
   }
   return NULL;
}




/////////////////////////////////////////////////////////////////////////////
//
//     Routine: TaskApp::PipeSetup
//
// Description: fork to run command and set up pipes
//
///////////////////////////////////////////////////////////////////////////

void TaskmonApp::PipeSetup()
{

   pipe(pipefd1);

   if (!(_pid = fork()))
   {
      string _build_cmd;

      close(pipefd1[0]);
      close(1);
      dup(pipefd1[1]);

      if((_cmdline.find('"') != string::npos) && 
         (_cmdline.find('\'') == string::npos))
      {
         // write 'commandline' to propagate " through cxcwrapper 
         _build_cmd = "cxcwrapper -m " + theApp->GetXpaName() + " toolstatus " + statusId + " '" + _cmdline + "'";
      }
      else if ((_cmdline.find('"') == string::npos) && 
               (_cmdline.find('\'') != string::npos))
      {
         // write "commandline" to propogate ' through cxcwrapper 
         _build_cmd = "cxcwrapper -m " + theApp->GetXpaName() + " toolstatus " + statusId + " \"" + _cmdline + "\"";
      }
      else 
      {
         // no quotes to propogate- write commandline 
         _build_cmd = "cxcwrapper -m " + theApp->GetXpaName() + " toolstatus " + statusId + " " + _cmdline;
      }
      _build_cmd = _build_cmd + " 2>&1  &";
      system(_build_cmd.c_str());
      _exit(0); 
   }
      else
        {
          close(pipefd1[1]);
          theApp->status_message("The application is currently running...");
          GIOChannel* ioc = g_io_channel_unix_new(pipefd1[0]); 
	  g_io_add_watch(ioc, 
			 (GIOCondition)(G_IO_IN | G_IO_HUP | G_IO_NVAL),
	                 handle_command_output,
			 (void*)pipefd1[0]);    
 
	} 
} // end function PipeSetup()



/////////////////////////////////////////////////////////////////////////////
//
//     Routine: TaskmonApp::handle_command_output
//
// Description: handle piped output data and set tool status on completion
//
/////////////////////////////////////////////////////////////////////////////

gboolean TaskmonApp::handle_command_output(GIOChannel *gio, 
				      GIOCondition condition, 
				      gpointer data)
{
  char pipe_buffer[MAX_PIPE_SIZE+1];
  string  sts;
  int     val;
  gint myfd = GPOINTER_TO_INT(data);  
  int msgstatus;


  // Read the pipe buffer
  int len = read((int)myfd, pipe_buffer, MAX_PIPE_SIZE);


   if (len)
   {
     if (len < MAX_PIPE_SIZE) 
       pipe_buffer[len] = '\0';
     else
        pipe_buffer[MAX_PIPE_SIZE] = '\0';
     msgstatus = theApp->output_message(pipe_buffer);

     return (true); 
   } 
   else //(!len)
   {
     if ((val = theApp->GetStatus()) != TASKM_UNKNOWN_STS)
      {
        if (val) 
          {
           sts = "Failure - exit status=";
           sts.append( _int_to_string(val) );  
          }
        else 
            sts = "Success"; 
      }
     else
        sts = "The application has exited.";

     theApp->SetFinished(1);

     msgstatus = theApp->status_message(sts);

     return (false);
   }


} // end function handle_command_output

