// 
//  Copyright (C) 2002-2003,2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "TaskmonFileDialog.hh"
#include "TaskmonFindDialog.hh"
#include "TaskmonStatusArea.hh"
#include "TaskmonPrintFileDialog.hh"
#include "TaskmonApp.hh"
#include "TaskmonCmdButtons.hh"
#include "TaskmonUtils.hh"


// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
TaskmonCmdButtons::TaskmonCmdButtons()
{
   int pos = 0;

   // create the buttons in toolbar
   _toolbar = gtk_toolbar_new ();

   GtkTooltips* tips = NULL;
   if (theApp)  tips = theApp->get_tips_group();
      else      tips = gtk_tooltips_new();

   gtk_toolbar_set_style(GTK_TOOLBAR(_toolbar), GTK_TOOLBAR_TEXT);

   if (_FindBtn = gtk_tool_button_new(NULL, "Find"))
   {
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _FindBtn, pos++);
      g_signal_connect(_FindBtn, "clicked", G_CALLBACK(FindCallback),
         (void*)this);
      gtk_tool_item_set_expand(GTK_TOOL_ITEM(_FindBtn), true);
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(_FindBtn), GTK_TOOLTIPS(tips),
         "Locate a text string in the Command and/or Output buffer", "Run & Exit Button");
   }
   if (_printBtn = gtk_tool_button_new(NULL, "Print"))
   {
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _printBtn, pos++);
      g_signal_connect(_printBtn, "clicked", G_CALLBACK(PrintCallback),
         (void*)this);
      gtk_tool_item_set_expand(GTK_TOOL_ITEM(_printBtn), true);
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(_printBtn), GTK_TOOLTIPS(tips),
         "Print the contents of the Command, Output, and Status buffer", "Run Button");
   }

   if (_saveBtn = gtk_tool_button_new(NULL, "Save As"))
   {
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _saveBtn, pos++);
      g_signal_connect(_saveBtn, "clicked", G_CALLBACK(SaveAsCallback),
         (void*)this);
      gtk_tool_item_set_expand(GTK_TOOL_ITEM(_saveBtn), true);
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(_saveBtn), GTK_TOOLTIPS(tips),
         "Save the contents of the Command, Output, and Status buffer in a file", "Save Button");
   }

   if (_quitBtn = gtk_tool_button_new(NULL, "Exit"))
   {
      gtk_toolbar_insert(GTK_TOOLBAR(_toolbar), _quitBtn, pos++);
      g_signal_connect(_quitBtn, "clicked", G_CALLBACK(QuitCallback),
         (void*)this);
      gtk_tool_item_set_expand(GTK_TOOL_ITEM(_quitBtn), true);
      gtk_tool_item_set_tooltip(GTK_TOOL_ITEM(_quitBtn), GTK_TOOLTIPS(tips),
	 "Exit Taskmonitor", "Quit Button");
   }

   // place the toolbar widgets
   _layout = gtk_table_new(3, 10, true); 
   gtk_table_attach_defaults(GTK_TABLE(_layout), _toolbar, 1,9,1,2);
}


// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
TaskmonCmdButtons::~TaskmonCmdButtons()
{
  gtk_widget_destroy(_toolbar);
  gtk_widget_destroy(_layout);
}


// --------------------------------------------------------
// Callback for the FIND button push event
// --------------------------------------------------------
int TaskmonCmdButtons::FindCallback(GObject *object, gpointer user_data)
{

   TaskmonFindDialog* _find_dialog = new TaskmonFindDialog();
   return 0; 
}


// --------------------------------------------------------
// Callback for Print button push event
// --------------------------------------------------------
int TaskmonCmdButtons::PrintCallback(GObject *object, gpointer user_data)
{
  TaskmonPrintFileDialog* _find_dialog = new TaskmonPrintFileDialog();
   return 0; 
}


// --------------------------------------------------------
// Callback for SaveAs button push event
// --------------------------------------------------------
int TaskmonCmdButtons::SaveAsCallback(GObject *object, gpointer user_data)
{
  int status=0;

  TaskmonFileDialog* fd = new TaskmonFileDialog();
  status = fd->_pop_it_up();

   return status; 
}



// --------------------------------------------------------
// Callback for Quit button push event
// --------------------------------------------------------
int TaskmonCmdButtons::QuitCallback(GObject *object, gpointer user_data)
{
   theApp->exit_application(); 
   return 0; 
}


