// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <fstream>
#include <unistd.h>

#include "TaskmonFileMenu.hh"

#define TASKMON_FLIST_BUF_SZ   256

using namespace std;


TaskmonFileMenu::TaskmonFileMenu()
{
   //   menuFile = gtk_menu_item_new_with_mnemonic(("_File"));
   //   gtk_menu_shell_insert(GTK_MENU_SHELL(menubar), menuFile, 0);
   int pos = 0; 
   base_w = gtk_menu_item_new_with_mnemonic("_File");

   menu = gtk_menu_new();

   menuItemPrint = gtk_menu_item_new_with_mnemonic(("_Print"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemPrint, pos++);

   menuItemSaveAs = gtk_menu_item_new_with_mnemonic(("Save _As"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemSaveAs, pos++);

   sep = gtk_separator_menu_item_new();
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), sep, pos++);

   menuItemQuit = gtk_menu_item_new_with_mnemonic(("_Quit"));
   gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItemQuit, pos++);

   /* We can attach the Quit menu item to our exit function */
   g_signal_connect_swapped(G_OBJECT(menuItemPrint), "activate",
                    G_CALLBACK (print_cb), (gpointer) NULL);
   g_signal_connect(G_OBJECT(menuItemSaveAs), "activate",
                    G_CALLBACK(saveas_cb), (gpointer) this);
   g_signal_connect_swapped(G_OBJECT(menuItemQuit), "activate",
                    G_CALLBACK (quit_cb), (gpointer) NULL);

   GtkTooltips *tips = theApp->get_tips_group();
   if (tips)
   {
      gtk_tooltips_set_tip(tips, menuItemPrint,
         "Print the current parameter file",NULL);
      gtk_tooltips_set_tip(tips, menuItemSaveAs,
         "Save changes to a different file",NULL);
      gtk_tooltips_set_tip(tips, menuItemQuit,
         "Exit Taskmonitor",NULL);
   }

   gtk_menu_item_set_submenu (GTK_MENU_ITEM(base_w), menu);
   _prev_dir = "";
}

TaskmonFileMenu::~TaskmonFileMenu()
{
   
}

int TaskmonFileMenu::show()
{
  gtk_widget_show(menuItemQuit);
   gtk_widget_show(base_w); 
   return 0; 
}

int TaskmonFileMenu::print_cb()
{
   TaskmonPrintFileDialog* dlg = new TaskmonPrintFileDialog();
   //if (dlg)  dlg->doit(); 
   return 0;
}


int TaskmonFileMenu::saveas_cb(GObject *object, gpointer user_data)
{
  int status=0;

  TaskmonFileDialog* fd = new TaskmonFileDialog();
  status = fd->_pop_it_up();

   return status; 
}


int TaskmonFileMenu::quit_cb()
{
   if (theApp)   delete theApp;
   return 0;
}




