// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "TaskmonFindDialog.hh"
#include <iostream>
#include <unistd.h>
#include <gtk/gtk.h>

using namespace std;

TaskmonFindDialog::TaskmonFindDialog()
{
  /* Create the widgets */
  dialog = gtk_dialog_new();


  gtk_widget_set_size_request( dialog, 300, 150);


  // create the entry section
  entry_label = gtk_label_new ("Search for:");
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), entry_label);

  string_text_box = gtk_entry_new();
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox), string_text_box);


 // create the buttons in the Action Area

   if (_PopupFindBtn = gtk_button_new_with_label("Find"))
   {
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		     _PopupFindBtn );

    g_signal_connect(_PopupFindBtn, "clicked", 
		     G_CALLBACK(PopupFindCB),
		     (void*)string_text_box);
   }

   if (_PopupClearBtn = gtk_button_new_with_label("Clear"))
   {
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		       _PopupClearBtn );

    g_signal_connect(_PopupClearBtn, "clicked", 
		     G_CALLBACK(PopupClearCB),
		     (void*)string_text_box);

   }

   if (_PopupCancelBtn = gtk_button_new_with_label("Cancel"))
   {

    gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
		       _PopupCancelBtn );

     g_signal_connect(_PopupCancelBtn, "clicked", 
		      G_CALLBACK(PopupCancelCB),
		      (void*)dialog);
   }



   // Set sensitivity of buttons 
      gtk_widget_set_sensitive(GTK_WIDGET(_PopupFindBtn), true);
      gtk_widget_set_sensitive(GTK_WIDGET(_PopupClearBtn), true);
      gtk_widget_set_sensitive(GTK_WIDGET(_PopupCancelBtn), true);

  /* Show everything we've added to the dialog. */

  gtk_widget_show_all (dialog);

} //End Constructor

TaskmonFindDialog::~TaskmonFindDialog()
{
   if (dialog)
    {
      gtk_widget_destroy (dialog);
      dialog = NULL;
    }
} // end Destructor


int TaskmonFindDialog::PopupFindCB(GObject *object, gpointer user_data)
{
  const gchar* text_buffer_contents;

  string search_string;

  text_buffer_contents = gtk_entry_get_text((GtkEntry*)user_data);
  search_string = text_buffer_contents;
  if (search_string.size() == 0)
    {
     TaskmonMessageDialog* _error_dialog = new TaskmonMessageDialog();
       _error_dialog->_pop_error_up("No search criteria entered. Please try again.");

       delete _error_dialog;

    }
  else
    theApp->_outputArea->MoveToString(text_buffer_contents);

  return(0);

} // end callback PopupFindCB

int TaskmonFindDialog::PopupClearCB(GObject *object, gpointer user_data)
{
  string nuthin="";
  gtk_entry_set_text((GtkEntry*)user_data, nuthin.c_str() );
  return(0);
} // end callback PopupClearCB


int TaskmonFindDialog::PopupCancelCB(GObject *object, gpointer user_data)
{
  gtk_widget_destroy ((GtkWidget*)user_data);

  return(0);

} // end callback PopupClearCB
