/////////////////////////////////////////////////////////////////////////////
//
//  FILE NAME:		TaskmonIPCComm.hh
//
//  DEVELOPMENT: 	AXAF Science Center, User Infrastructure
//
//  DESCRIPTION:	Base Class definition for the Interprocess
//			Communication classes
//			
//---------------------------------------------------------------------------
// Copyright (c) 1998, Smithsonian Astrophysical Observatory
//---------------------------------------------------------------------------

#if !defined(_TASKMONIPC_COMM)
#define	_TASKMONIPC_COMM

#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <string>


extern "C"
{
#if defined(__sun) || defined(__alpha)
extern          int usleep(unsigned int useconds);
#endif
}

#if defined(_TASKMONIPC_LIBRARY_)
#include	"xpa.h"
#endif


#include "TaskmonIPCMessage.hh"



// Wait some number of seconds before attempting to re-launch the application
#define	COMM_WAIT_FOR_LAUNCH_SECONDS	30
#define TaskmonIPC_MAX_MESSAGE_LENGTH          150

// Structure used to keep track of callbacks, internally.

typedef struct _callbacks
{
	char			key[TaskmonIPC_KEY_LENGTH], name[128];
  	TaskmonIPCMessage*		(*cb)(TaskmonIPCMessage&);
  	struct	_callbacks	*next;
} TaskmonIPCCallbackTable, *TaskmonIPCCallbackTablePtr;

#if !defined(_TASKMONIPC_LIBRARY_)
struct XPA;
#endif

// The Interprocess Communication Base Class. Contains all routines
// and data common to both Clients and Servers

class TaskmonIPCComm
{
	public:

       		TaskmonIPCComm(char* pname, char **args=NULL);
      		virtual ~TaskmonIPCComm();

		int     		SendWithReply(char *dest,
						       TaskmonIPCMessage&,
						       TaskmonIPCMessage&,
						       int launch=1,
						       char *alias=NULL,
                                                       char** argv=NULL);
		int			Send(char *dest, TaskmonIPCMessage&,
							int launch=1,
							char *alias=NULL,
                                                       char** argv=NULL);
  		void			EventLoop();
  		void			Poll(int msec);
  		int			Launch(char*, char** argv = NULL,
							char *alias=NULL);
		int			AlreadyRunning(char*);

  		int			WaitForLaunch(char *);
                int                     LaunchedApp() { return _launched_app;}
                int                     XpaIsSetup() { return (pxpa != NULL);}

	protected:

  		string			name;
  		char 			**envp, **argv;

#if defined(_TASKMONIPC_LIBRARY_)
  		XPA			pxpa;
#else
  		XPA			*pxpa;
#endif

	private:

  		void		Initialize(char*, char**);
  		int		CheckParameters(char *, TaskmonIPCMessage&);
  		int		waittime;
                int             _launched_app; //non-zero= launched application
};

// Messages for all applications

#define EXIT                            "exit"
#define QUIT                            "quit"

#ifdef WLM_DELETED 
// ASC File Browser
#define BROWSER_LAUNCH                  "BROWSER_LAUNCH"

// Filter Window
#define FILTWIN_LAUNCH                  "FILTWIN_LAUNCH"

// First Look
#define FL_LAUNCH                       "FL_LAUNCH"

// Pipeline Editor
#define PED_LAUNCH                      "PED_LAUNCH"
#define PED_SHOW                        "PED_SHOW"

// Notebook
#define NBK_LAUNCH                      "NBK_LAUNCH"
#define NBK_CMD                         "NBK_CMD"
#define NBK_DATA                        "NBK_DATA"

// Imager defines
#define IMAGER_VIEWFILE                 "IMAGER_VIEWFILE"
#define IMAGER_SETIMAGEAREA             "IMAGER_SETIMAGEAREA"
#define IMAGER_NEWIMAGEAREA             "IMAGER_NEWIMAGEAREA"
#define IMAGER_GETIMAGEAREA             "IMAGER_GETIMAGEAREA"
#define IMAGER_GETREGIONS               "IMAGER_GETREGIONS"
#define IMAGER_SETREGION_CIRCLE         "IMAGER_SETREGION_CIRCLE"
#define IMAGER_SETREGION_RECTANGLE      "IMAGER_SETREGION_RECTANGLE"
#define IMAGER_SETREGION_LINE           "IMAGER_SETREGION_LINE"
#define IMAGER_SETREGION_ELLIPSE        "IMAGER_SETREGION_ELLIPSE"
#define IMAGER_SETREGION_POINT          "IMAGER_SETREGION_POINT"
#define IMAGER_SETREGION_POLYGON        "IMAGER_SETREGION_POLYGON"
#define IMAGER_SETREGION_MULTIPOINTS    "IMAGER_SETREGION_MULTIPOINTS"
#define IMAGER_SETREGION_CIRCLES        "IMAGER_SETREGION_CIRCLES"
#define IMAGER_SETREGION_RECTANGLES     "IMAGER_SETREGION_RECTANGLES"
#define IMAGER_SET_SCALING_ALGORITHM    "IMAGER_SET_SCALING_ALGORITHM"

// PLOT application (SM)
#define PLOT_LAUNCH                     "PLOT_LAUNCH"
#define PLOT_EXIT                       "PLOT_EXIT"
#define PLOT_GENERIC                    "PLOT_GENERIC"
#define PLOT_PICKPOINTS                 "PLOT_PICKPOINTS"
#define PLOT_GETVARIABLEVALS            "PLOT_GETVARIABLEVALS"
#define PLOT_CHECKVECS                  "PLOT_CHECKVECS"
#define PLOT_FREE_X11_DEVICE            "PLOT_FREE_X11_DEVICE"

// ToolAgent
#define TLA_LAUNCH                      "TLA_LAUNCH"
#define TLA_INFO                        "TLA_INFO"
#define TLA_EXEC                        "TLA_EXEC"
#define TLA_WHAT                        "TLA_WHAT"
#define TLA_WHATALL                     "TLA_WHATALL"
#define TLA_REFRESH                     "TLA_REFRESH"

// ASCFit
#define SAOFIT_LAUNCH                   "SAOFIT_LAUNCH"
#define SAOFIT_EXIT                     "SAOFIT_EXIT"

// Constants associated with messages defined above
#define	IMAGER_SCALE_LINEAR		0
#define	IMAGER_SCALE_HISTOGRAM		1
#define	IMAGER_SCALE_LOG		2
#endif

#endif
