// 
//  Copyright (C) 2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "TaskmonUtils.hh"
#include "float.h"

#include <sstream>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

using namespace std; 

string _int_to_string(int val)
{
   static string outval; 
   stringstream ss;
   ss << val;
   ss >> outval;
   return outval;
}

int _string_to_int(string val, int &outv)
{
   string tmp = val;
   tmp.erase(tmp.find_last_not_of(" \t")+1);
   istringstream ss(tmp);
   if ((ss >> outv) && ss.eof()) return 0;  
   return 1; 
}

//
// _tempfile_envar_path - Check the values of the environment variables
//                        ASCDS_WORK_PATH and ASCDS_TMP. If ASCDS_WORK_PATH
//                        is defined use that path. If not, and if ASCDS_TMP
//                        is defined, use that path. If neither are defined,
//                        use /tmp as the path.  Select the first path in the
//                        value of either environment variable if there 
//                        are multiples, as delimited by a colon, semi-colon 
//                        or a space.

string _tempfile_envar_path()
{
      char*  wdir = getenv("ASCDS_WORK_PATH");
      size_t nonspace=0;
      size_t found;
      string filepath;

      if (wdir == NULL)
      {
        wdir = getenv("ASCDS_TMP");

         if (wdir == NULL)
            wdir = "/tmp";
      }

      filepath = wdir;

      // strip off any leading spaces in the path

      nonspace = filepath.find_first_not_of(" '\t'\n");
      filepath.erase (0,nonspace);

      // chop off anything that might exist after a ;, :, or space

      found=filepath.find_first_of(";: ");
       if(found != filepath.npos)
	 filepath.erase(found);

      return(filepath);

} // end _tempfile_envar_path()

