// 
//  Copyright (C) 2007-2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include <unistd.h>
#include <stdlib.h>

#include "CiaoAnalysisMenu.hh"

using namespace std;

// takes in filename, tooltip group, and function pointer. The function
// should return a filename to seed $filename macros- ie. string (*fp)(void) 
CiaoAnalysisMenu::CiaoAnalysisMenu(string menufile, GtkTooltips* tips, 
   funcPtr fp, statusFuncPtr sfp)
{
   char* pth = NULL; 
   string def_file;
   base_w = gtk_menu_item_new_with_mnemonic("_Analysis");

   menu_loaded = false; 
   menu = gtk_menu_new();
   _menufile = menufile; 
   _tips = tips;
   replace_filename = fp; // sets replace_filename method to function pointer
   status_message = sfp;  // sets status_message method to function pointer

   if (_menufile.length()) 
   {
      if (!access(_menufile.c_str(),F_OK))  configure_menu(_menufile);
   }
   if (!menu_loaded)
   {
      if ((pth = getenv("HOME")) != NULL)
      {
         def_file = pth;
         def_file.append("/ciao.ans");
         if (!access(def_file.c_str(),F_OK)) configure_menu(def_file); 
      }
   }
   if (!menu_loaded)
   {
      if ((pth = getenv("ASCDS_INSTALL")) != NULL)
      {
         def_file = pth;
         def_file.append("/bin/ciao.ans");
      }
      if (!access(def_file.c_str(),F_OK)) configure_menu(def_file); 
   }
   if (!menu_loaded && (access("ciao.ans", F_OK) == 0)) 
      configure_menu("ciao.ans"); 

   if (status_message && menu_loaded)
   {
      string msg = "Configuring Analysis Menu from file: " + _loadedfile; 
      status_message(msg); 
   }
}

CiaoAnalysisMenu::~CiaoAnalysisMenu()
{
   
}

int CiaoAnalysisMenu::show()
{
   gtk_widget_show(base_w); 
   return 0; 
}

int CiaoAnalysisMenu::execute_cmd_cb(gpointer user_data)
{
   CiaoToolEntry* cmd = (CiaoToolEntry*)user_data;  

   if (!cmd) return 0;

   if ((cmd->getType() == CiaoToolEntry::COMMAND_TYPE) ||
       (cmd->getType() == CiaoToolEntry::HELP_TYPE))
   {
      cmd->doit(); 
   }
   return 0; 
}


void CiaoAnalysisMenu::configure_menu(string menu_file)
{
   string mfile;
   CiaoToolList* _ctlist = new CiaoToolList(menu_file);
   if (_ctlist->isValidList())
   {
      list<CiaoToolEntry*> mylist = _ctlist->getList();
      list<CiaoToolEntry*>::iterator it;
      CiaoToolEntry *te_p = NULL;
      int prev; 
      GtkWidget *sep = NULL;
      int loc[CTL_MAX_DEPTH];

      for (int rr = CTL_MAX_DEPTH; rr--; ) loc[rr] = 0; 

      for(it = mylist.begin(); it != mylist.end(); it++)
      {
         GtkWidget* pd[CTL_MAX_DEPTH];
         pd[0] = menu; 
         te_p = (CiaoToolEntry*)(*it);
         string val = te_p->getName();
         string tip = te_p->getTip();
         int dep = te_p->getDepth();
         int typ = te_p->getType();
   
         prev = dep;
	 if ((dep > -1) && (dep < CTL_MAX_DEPTH))
         {
	    if (typ == CiaoToolEntry::CATEGORY_TYPE)
	    {
	       GtkWidget *mi = gtk_menu_item_new_with_label(val.c_str());
	       gtk_menu_shell_insert(GTK_MENU_SHELL(pd[dep]), mi, loc[dep]++);
               pd[dep+1] = gtk_menu_new();
	       loc[dep+1] = 0; 
	       gtk_menu_item_set_submenu(GTK_MENU_ITEM(mi), pd[dep+1]);
	       if (_tips)  gtk_tooltips_set_tip(_tips, mi, tip.c_str(), NULL);
	    }
            else if (typ == CiaoToolEntry::COMMAND_TYPE)
            {
               if (te_p->getSupported()!= CiaoToolEntry::NEVER_SUPPORTED)
               {
                  // don't display bind entries
	          GtkWidget *mi = gtk_menu_item_new_with_label(val.c_str());
                  gtk_menu_shell_insert(GTK_MENU_SHELL(pd[dep]), mi, loc[dep]++);
		  g_signal_connect_swapped(G_OBJECT(mi), "activate",
		      G_CALLBACK(execute_cmd_cb), (gpointer) te_p);
	          if (_tips)
	             gtk_tooltips_set_tip(_tips, mi, tip.c_str(), NULL);
	          if (te_p->DisableUnsupported())
		     gtk_widget_set_sensitive(GTK_WIDGET(mi), FALSE);
	          if (replace_filename)  
		     te_p->set_replace_filename_method(replace_filename); 
	          if (status_message)  
		     te_p->set_status_message_method(status_message); 
               }
            }
            else if (typ == CiaoToolEntry::SEPARATOR_TYPE)
            {
	       sep = gtk_separator_menu_item_new();
	       gtk_menu_shell_insert(GTK_MENU_SHELL(pd[dep]), sep, loc[dep]++);
            }
	    else if (typ == CiaoToolEntry::HELP_TYPE)
            {
	       GtkWidget *mi = gtk_menu_item_new_with_label(val.c_str());
               gtk_menu_shell_insert(GTK_MENU_SHELL(pd[dep]), mi, loc[dep]++);
               g_signal_connect_swapped(G_OBJECT(mi), "activate",
		      G_CALLBACK(execute_cmd_cb), (gpointer) te_p);
            }
	 }
      }
      gtk_menu_item_set_submenu (GTK_MENU_ITEM(base_w), menu);
   }
   _loadedfile = menu_file; 
   menu_loaded = true; 
}

