// 
//  Copyright (C) 2002,2003,2007,2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//



#include "PegApp.hh"
#include "PegParData.hh"
#include "PegParEntry.hh"
#include "PegParEntryBoolean.hh"
#include "PegParEntryInteger.hh"
#include "PegParEntryReal.hh"
#include "PegParEntryString.hh"
#include "Parameter.hh"
#include "ParamItem.hh"
#include "PegFloatingMenu.hh"
#include "PegUtils.hh"

#include "AhelpInterface.hh"

// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
PegFloatingMenu::PegFloatingMenu(PegParEntry* entry)
{
   _menu = gtk_menu_new(); 
   
   int min_active = 0, 
       max_active = 0,
       browse_active = 0; 
       
   _entry = entry; 

   ParamItem *item = _entry->GetItem();
   if (item)
   {
      switch(item->GetType())
      {
         case ParamItem::STRING_TYPE:
         {
            ParamString *ps = ((PegParEntryString*)_entry)->GetItem();
            switch (ps->GetRangeCheck())
            {
               case MIN_MAX_CHECK:
                  min_active = max_active = 1;
               break; 
               case MIN_ONLY_CHECK:
                  min_active = 1; 
               break; 
               case MAX_ONLY_CHECK:
                  max_active = 1; 
               break; 
               case DEFAULT_CHECK:
               default:
                  // do nothing 
               break; 
            }
         }
         break;
         case ParamItem::INTEGER_TYPE:
         {
            ParamInteger *pi = ((PegParEntryInteger*)_entry)->GetItem();
            switch (pi->GetRangeCheck())
            {
               case MIN_MAX_CHECK:
                  min_active = max_active = 1;
               break;
               case MIN_ONLY_CHECK:
                  min_active = 1;
               break;
               case MAX_ONLY_CHECK:
                  max_active = 1;
               break;
               case DEFAULT_CHECK:
               default:
                  // do nothing 
               break;
            }
         }
         break;
         case ParamItem::REAL_TYPE:
         {
            ParamReal *pr = ((PegParEntryReal*)_entry)->GetItem();
            switch (pr->GetRangeCheck())
            {
               case MIN_MAX_CHECK:
                  min_active = max_active = 1;
               break;
               case MIN_ONLY_CHECK:
                  min_active = 1;
               break;
               case MAX_ONLY_CHECK:
                  max_active = 1;
               break;
               case DEFAULT_CHECK:
               default:
                  // do nothing 
               break;
            }
         }
         break;
         case ParamItem::FILENAME_TYPE:
         {
            browse_active = 1; 
         }
         break;
         default:
            // for now, don't worry about PSET and BOOLEAN types
         break; 
      }
   }

   GtkTooltips* tips = NULL;
   if (theApp)  tips = theApp->get_tips_group();
      else      tips = gtk_tooltips_new();

   int pos = 0, cnt = 0; 

   _menuIndir = gtk_menu_item_new_with_label("Indirection/Default");
   gtk_menu_shell_insert(GTK_MENU_SHELL(_menu), _menuIndir, pos++);
   g_signal_connect(G_OBJECT(_menuIndir), "activate",
                    G_CALLBACK(indirection_cb), (gpointer)entry);
   gtk_tooltips_set_tip(tips, GTK_WIDGET(_menuIndir),
      "Enter data as default type or indirection", NULL);

   _menuRestore = gtk_menu_item_new_with_label("Restore Initial Value");
   gtk_menu_shell_insert(GTK_MENU_SHELL(_menu), _menuRestore, pos++);
   g_signal_connect(G_OBJECT(_menuRestore), "activate",
                    G_CALLBACK(restore_init_value_cb), (gpointer)entry);
   gtk_tooltips_set_tip(tips, GTK_WIDGET(_menuRestore),
      "Restore the parameter to its initial value", NULL);
   
   _menuMin = gtk_menu_item_new_with_label("Set Value to Min");
   gtk_menu_shell_insert(GTK_MENU_SHELL(_menu), _menuMin, pos++);
   g_signal_connect(G_OBJECT(_menuMin), "activate",
                    G_CALLBACK(set_to_min_cb), (gpointer)entry);
   gtk_tooltips_set_tip(tips, GTK_WIDGET(_menuMin),
      "Use lower bound (minimum) for value", NULL);
   gtk_widget_set_sensitive(GTK_WIDGET(_menuMin), min_active); 
   
   _menuMax = gtk_menu_item_new_with_label("Set Value to Max");
   gtk_menu_shell_insert(GTK_MENU_SHELL(_menu), _menuMax, pos++);
   g_signal_connect(G_OBJECT(_menuMax), "activate",
                    G_CALLBACK(set_to_max_cb), (gpointer)entry);
   gtk_tooltips_set_tip(tips, GTK_WIDGET(_menuMax),
      "Use upper bound (maximum) for value", NULL);
   gtk_widget_set_sensitive(GTK_WIDGET(_menuMax), max_active); 
   
   _menuDisplay = gtk_menu_item_new_with_label("Display");
   gtk_menu_shell_insert(GTK_MENU_SHELL(_menu), _menuDisplay, pos++);
   _submenuDisplay = gtk_menu_new();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(_menuDisplay), _submenuDisplay);
   _menuSubDisp[cnt] = gtk_menu_item_new_with_label("Initial Value");
   gtk_tooltips_set_tip(tips, GTK_WIDGET(_menuSubDisp[cnt]),
      "Display parameter's initial value in Status Area", NULL);
   g_signal_connect(G_OBJECT(_menuSubDisp[cnt]), "activate",
                    G_CALLBACK(display_init_value_cb), (gpointer)entry);
   gtk_menu_shell_insert(GTK_MENU_SHELL(_submenuDisplay),_menuSubDisp[cnt],cnt);
   _menuSubDisp[++cnt] = gtk_menu_item_new_with_label("Minimum");
   gtk_tooltips_set_tip(tips, GTK_WIDGET(_menuSubDisp[cnt]),
      "Display parameter's minimum value in Status Area", NULL);
   g_signal_connect(G_OBJECT(_menuSubDisp[cnt]), "activate",
                    G_CALLBACK(display_min_cb), (gpointer)entry);
   gtk_menu_shell_insert(GTK_MENU_SHELL(_submenuDisplay),_menuSubDisp[cnt],cnt);
   _menuSubDisp[++cnt] = gtk_menu_item_new_with_label("Maximum");
   gtk_tooltips_set_tip(tips, GTK_WIDGET(_menuSubDisp[cnt]),
      "Display parameter's maximum value in Status Area", NULL);
   g_signal_connect(G_OBJECT(_menuSubDisp[cnt]), "activate",
                    G_CALLBACK(display_max_cb), (gpointer)entry);
   gtk_menu_shell_insert(GTK_MENU_SHELL(_submenuDisplay),_menuSubDisp[cnt],cnt);
   _menuSubDisp[++cnt] = gtk_menu_item_new_with_label("Type");
   gtk_tooltips_set_tip(tips, GTK_WIDGET(_menuSubDisp[cnt]),
      "Display parameter's type value in Status Area", NULL);
   g_signal_connect(G_OBJECT(_menuSubDisp[cnt]), "activate",
                    G_CALLBACK(display_type_cb), (gpointer)entry);
   gtk_menu_shell_insert(GTK_MENU_SHELL(_submenuDisplay),_menuSubDisp[cnt],cnt);
   _menuSubDisp[++cnt] = gtk_menu_item_new_with_label("Mode");
   gtk_tooltips_set_tip(tips, GTK_WIDGET(_menuSubDisp[cnt]),
      "Display parameter's mode value in Status Area", NULL);
   g_signal_connect(G_OBJECT(_menuSubDisp[cnt]), "activate",
                    G_CALLBACK(display_mode_cb), (gpointer)entry);
   gtk_menu_shell_insert(GTK_MENU_SHELL(_submenuDisplay),_menuSubDisp[cnt],cnt);
   _menuSubDisp[++cnt] = gtk_menu_item_new_with_label("Prompt");
   gtk_tooltips_set_tip(tips, GTK_WIDGET(_menuSubDisp[cnt]),
      "Display parameter's prompt value in Status Area", NULL);
   gtk_menu_shell_insert(GTK_MENU_SHELL(_submenuDisplay),_menuSubDisp[cnt],cnt);
   g_signal_connect(G_OBJECT(_menuSubDisp[cnt]), "activate",
                    G_CALLBACK(display_prompt_cb), (gpointer)entry);
   
   _menuHelp = gtk_menu_item_new_with_label("Parameter Help");
   gtk_tooltips_set_tip(tips, GTK_WIDGET(_menuHelp),
      "Display help for the selected parameter", NULL);
   g_signal_connect(G_OBJECT(_menuHelp), "activate",
                    G_CALLBACK(param_help_cb), (gpointer)entry);

   // disable parameter specific help if it does not exist for entry
   // need to unset default context so that we can search any context for the
   // current parameter file and reset default to peg when finished
   if (item)
   {
      AhelpStrings key, cont = item->GetName().c_str(), 
         tmp = theApp->GetFilename().c_str();
      if (tmp.length())   key = _file_get_root_name(tmp.chars()).c_str();
      GlobalAhelpInterface::SetDefaultContext("");
      if (GlobalAhelpInterface::GetHelp(key, "", "PARAMLIST", cont, 0, tmp)!= 1)
         gtk_widget_set_sensitive(GTK_WIDGET(_menuHelp), 0);
      GlobalAhelpInterface::SetDefaultContext("peg");
   }
   gtk_menu_shell_insert(GTK_MENU_SHELL(_menu), _menuHelp, pos++);
   
   _menuBrowse = gtk_menu_item_new_with_label("Browse Files");
   g_signal_connect(G_OBJECT(_menuBrowse), "activate",
                    G_CALLBACK(browse_file_cb), (gpointer)entry);
   gtk_menu_shell_insert(GTK_MENU_SHELL(_menu), _menuBrowse, pos++);
   gtk_tooltips_set_tip(tips, GTK_WIDGET(_menuBrowse),
      "Use file selection dialog to specify a file", NULL);
   gtk_widget_set_sensitive(GTK_WIDGET(_menuBrowse), browse_active); 
   gtk_widget_show_all(_menu); 

}

/////////////////////////////////////////////////////////////////////////////
//
//     Routine: PegFloatingMenu::~PegFloatingMenu
//
// Description: Destructor.
//

PegFloatingMenu::~PegFloatingMenu()
{
}


int PegFloatingMenu::indirection_cb(GObject *object, gpointer user_data)
{
   PegParEntry* ent = (PegParEntry*)user_data;
   if (ent) ent->toggleDispType();
   return 0; 
}

int PegFloatingMenu::display_init_value_cb(GObject *object, gpointer user_data)
{
   PegParEntry* ent = (PegParEntry*)user_data;
   Parameter *par = thePData->GetParameter();
   ParamItem *item;

   if (!ent || !par)  return 1;
   if (!(item = ent->GetItem())) return 2;

   string nam = item->GetName(); 
   string msg = nam + "'s initial value: ";
   switch(item->GetType())
   {
      case ParamItem::PSET_TYPE:       // fallthrough intended:
      case ParamItem::FILENAME_TYPE:   // fallthrough intended
      case ParamItem::STRING_TYPE:
      {
         ParamString *ps = ((PegParEntryString*)ent)->GetItem();
         string redir = ps->GetRedirectValue();
         if (redir.length())
         {
            msg.append(redir);
            string eval = par->EvalIndir((char*)nam.c_str(),(char*)redir.c_str());
            if (eval.length())
            {
               msg.append("  ->  ");
               msg.append(eval);
            }
         }
         else
	    msg.append(ps->GetValue());
      }
      break;
      case ParamItem::BOOLEAN_TYPE:
      {
         ParamBoolean *pb = ((PegParEntryBoolean*)ent)->GetItem();
         string redir = pb->GetRedirectValue();
         if (redir.length())
         {
            msg.append(redir);
            string eval = par->EvalIndir((char*)nam.c_str(),(char*)redir.c_str());
            if (eval.length())
            {
               msg.append("  ->  ");
               msg.append(eval);
            }
         }
         else if (pb->GetValue())  msg.append("yes");
         else                      msg.append("no");
      }
      break;
      case ParamItem::INTEGER_TYPE:
      {
         ParamInteger *pi = ((PegParEntryInteger*)ent)->GetItem();
         string redir = pi->GetRedirectValue();
         if (redir.length())
         {
            msg.append(redir);
            string eval = par->EvalIndir((char*)nam.c_str(),(char*)redir.c_str());
            if (eval.length())
            {
               msg.append("  ->  ");
               msg.append(eval); 
            }
         }
         else
         {
            int tmpval = pi->GetValue();
            if (tmpval == INDEF_INT)
               msg.append("INDEF");
            else
               msg.append(_int_to_string(tmpval));
         }
      }
      break;
      case ParamItem::REAL_TYPE:
      {
         ParamReal *pr = ((PegParEntryReal*)ent)->GetItem();
         string redir = pr->GetRedirectValue();
         if (redir.length())
         {
            msg.append(redir);
            string eval = par->EvalIndir((char*)nam.c_str(),(char*)redir.c_str());
            if (eval.length())
            {
               msg.append("  ->  ");
               msg.append(eval);
            }
         }
         else
         {
            double tmpval = pr->GetValue();

            if (tmpval == INDEF_REAL)
               msg.append("INDEF");
            else
            {
               char temp[100];
               ostrstream oss(temp, 100);
               oss << setprecision(pr->GetPrecision());
               oss << tmpval << ends;
               msg.append(temp);
            }
         }
      }
      break;
      default:
      break;
   }
   theApp->status_message(msg); 
   return 0; 
}

int PegFloatingMenu::display_prompt_cb(GObject *object, gpointer user_data)
{
   PegParEntry* ent = (PegParEntry*)user_data;
   ParamItem *item;

   if (!ent) return 1;
   if (!(item = ent->GetItem())) return 2;
   string msg = item->GetName() + "'s prompt: ";
   msg.append(item->GetPrompt());
   theApp->status_message(msg);
   return 0; 
}

int PegFloatingMenu::display_type_cb(GObject *object, gpointer user_data)
{
   PegParEntry* ent = (PegParEntry*)user_data;
   ParamItem *item;

   if (!ent) return 1;
   if (!(item = ent->GetItem())) return 2;
   string msg = item->GetName() + "'s type: ";
   switch(item->GetType())
   {
      case ParamItem::STRING_TYPE:
         msg.append("string");
      break;
      case ParamItem::FILENAME_TYPE:
         msg.append("filename");
      break;
      case ParamItem::PSET_TYPE:
         msg.append("pset");
      break;
      case ParamItem::BOOLEAN_TYPE:
         msg.append("boolean");
      break;
      case ParamItem::INTEGER_TYPE:
         msg.append("integer");
      break;
      case ParamItem::REAL_TYPE:
         msg.append("real");
      break;
      default:
         msg.append("unspecified/unknown");
      break;
   }
   theApp->status_message(msg);
   return 0; 
}

int PegFloatingMenu::display_mode_cb(GObject *object, gpointer user_data)
{
   PegParEntry* ent = (PegParEntry*)user_data;
   ParamItem *item;
   if (!ent) return 1;
   if (!(item = ent->GetItem())) return 2;
   string msg = item->GetName() + "'s mode: ";
   if (item->IsAutoMode()) msg.append("auto ");
   if (item->IsHiddenMode()) msg.append("hidden ");
   if (item->IsQueryMode()) msg.append("query ");
   if (item->IsLearnMode()) msg.append("learn ");
   theApp->status_message(msg);
   return 0; 
}

int PegFloatingMenu::display_min_cb(GObject *object, gpointer user_data)
{
   PegParEntry* ent = (PegParEntry*)user_data;
   ParamItem *item;
   if (!ent) return 1;
   if (!(item = ent->GetItem())) return 2;
   string msg = item->GetName() + "'s minimum: ";
   switch(item->GetType())
   {
      case ParamItem::PSET_TYPE:       // fallthrough intended:
      case ParamItem::FILENAME_TYPE:   // fallthrough intended
      case ParamItem::STRING_TYPE:
      {
         ParamString *ps = ((PegParEntryString*)ent)->GetItem();
         switch(ps->GetRangeCheck())
         {
            case SUBSET_CHECK:
            {
               int scnt = ps->GetSubsetCount() -1;
               char** tmps = ps->GetSubset();
               for (int rr = 0; rr < scnt; rr++)
                  if (tmps[rr]) 
		  {
		     msg.append(tmps[rr]);
                     msg.append("|");
                  }
               if (tmps[scnt]) msg.append(tmps[scnt]);
            }
            break;
            case NO_RANGE_CHECK:
               msg.append("[none specified]");
            break;
            default:
            {
               string redir = ps->GetRedirectMin();
               if (redir.length())
               {
                  msg.append(redir);
                  msg.append("  ->  ");
               }
               msg.append(ps->GetMinValue());
            }
            break;
         }
      }
      break; 
      case ParamItem::BOOLEAN_TYPE:
      {
         ParamBoolean *pb = ((PegParEntryBoolean*)ent)->GetItem();
         if (pb->GetMinValue())  msg.append("yes");
         else                    msg.append("no");
      }
      break;
      case ParamItem::INTEGER_TYPE:
      {
         ParamInteger *pi = ((PegParEntryInteger*)ent)->GetItem();
         switch(pi->GetRangeCheck())
         {
            case SUBSET_CHECK:
            {
               int scnt = pi->GetSubsetCount() -1;
               int* tmps = pi->GetSubset();
               for (int rr = 0; rr < scnt; rr++)
                  if (tmps[rr])
		  {
		     msg.append(_int_to_string(tmps[rr]));
                     msg.append("|");
                  }
               if (tmps[scnt]) msg.append(_int_to_string(tmps[scnt]));
            }
            break;
            case MAX_ONLY_CHECK:  // fallthru intended
            case NO_RANGE_CHECK:
               msg.append("[none specified]");
            break;
            default:
               string redir = pi->GetRedirectMin();
               int tmpval = pi->GetMinValue();
               if (redir.length())
               {
                  msg.append(redir);
                  msg.append("  ->  ");
               }
               if (tmpval == INDEF_INT)
                  msg.append("INDEF");
               else
                  msg.append(_int_to_string(tmpval));
            break;
         }
      }
      break;
      case ParamItem::REAL_TYPE:
      {
         ParamReal *pr = ((PegParEntryReal*)ent)->GetItem();
         switch(pr->GetRangeCheck())
         {
            case SUBSET_CHECK:
            {
               int scnt = pr->GetSubsetCount() -1;
               double* tmps = pr->GetSubset();
               for (int rr = 0; rr < scnt; rr++)
                  if (tmps[rr])
		  {
		     msg.append(_double_to_string(tmps[rr]));
                     msg.append("|");
                  }
               if (tmps[scnt]) msg.append(_double_to_string(tmps[scnt]));
            }
            break;
            case NO_RANGE_CHECK:
               msg.append("[none specified]");
            break;
            default:
            {
               string redir = pr->GetRedirectMin();
               double tmpval = pr->GetMinValue();
               if (redir.length())
               {
                  msg.append(redir);
                  msg.append("  ->  ");
               }
               if (tmpval == INDEF_REAL)
                  msg.append("INDEF");
               else
                  msg.append(_double_to_string(tmpval));
            }
            break;
         }
      }
      break;
      default:
         msg.append("unspecified/unknown");
      break;
   }
   theApp->status_message(msg);
   return 0;
}

int PegFloatingMenu::display_max_cb(GObject *object, gpointer user_data)
{
   PegParEntry* ent = (PegParEntry*)user_data;
   ParamItem *item;
   if (!ent) return 1;
   if (!(item = ent->GetItem())) return 2;
   string msg = item->GetName() + "'s maximum: ";
   switch(item->GetType())
   {
      case ParamItem::PSET_TYPE:       // fallthrough intended:
      case ParamItem::FILENAME_TYPE:   // fallthrough intended
      case ParamItem::STRING_TYPE:
      {
         ParamString *ps = ((PegParEntryString*)ent)->GetItem();
         switch(ps->GetRangeCheck())
         {
            case SUBSET_CHECK:
	       msg.append("[subset- see minimum]");
            break;
            case MIN_ONLY_CHECK:  // fallthru intended
            case NO_RANGE_CHECK:
               msg.append("[none specified]");
            break;
            default:
            {
               string redir = ps->GetRedirectMax();
               if (redir.length())
               {
                  msg.append(redir);
                  msg.append("  ->  ");
               }
               msg.append(ps->GetMaxValue());
            }
            break;
         }
      }
      break;
      case ParamItem::BOOLEAN_TYPE:
      {
         ParamBoolean *pb = ((PegParEntryBoolean*)ent)->GetItem();
	 if (pb->GetMaxValue())  msg.append("yes");
         else                    msg.append("no");
      }
      break;
      case ParamItem::INTEGER_TYPE:
      {
         ParamInteger *pi = ((PegParEntryInteger*)ent)->GetItem();
	 switch(pi->GetRangeCheck())
         {
            case SUBSET_CHECK:
               msg.append("[subset- see minimum]");
            break;
            case MIN_ONLY_CHECK:  // fallthru intended
            case NO_RANGE_CHECK:
               msg.append("[none specified]");
            break;
            default:
            {
               string redir = pi->GetRedirectMax();
               int tmpval = pi->GetMaxValue();
               if (redir.length())
               {
                  msg.append(redir);
                  msg.append("  ->  ");
               }
               if (tmpval == INDEF_INT)  msg.append("INDEF");
               else                      msg.append(_int_to_string(tmpval));
            }
            break;
         }
      }
      break;
      case ParamItem::REAL_TYPE:
      {
         ParamReal *pr = ((PegParEntryReal*)ent)->GetItem();
         switch(pr->GetRangeCheck())
         {
            case SUBSET_CHECK:
               msg.append("[subset- see minimum]");
            break;
            case MIN_ONLY_CHECK:  // fallthru intended
            case NO_RANGE_CHECK:
               msg.append("[none specified]");
            break;
            default:
            {
               string redir = pr->GetRedirectMax();
               double tmpval = pr->GetMaxValue();
               if (redir.length())
               {
                  msg.append(redir);
                  msg.append("  ->  ");
               }
               if (tmpval == INDEF_REAL)  msg.append("INDEF");
               else
                  msg.append(_double_to_string(tmpval));
            }
            break;
         }
      }
      break;
      default:
         msg.append("unspecified/unknown");
      break;
   }
   theApp->status_message(msg);
   return 0;
}

int PegFloatingMenu::restore_init_value_cb(GObject *object, gpointer user_data)
{
   PegParEntry* ent = (PegParEntry*)user_data;
   Parameter *par = thePData->GetParameter();
   ParamItem *item;
   string sval;

   if (!ent || !par)  return 1;
   if (!(item = ent->GetItem())) return 2;


   if (ent->getDispType() != ent->getOrigType())   ent->toggleDispType();

   switch(item->GetType())
   {
      case ParamItem::STRING_TYPE:      // fallthrough intended
      case ParamItem::FILENAME_TYPE:    // fallthrough intended
      case ParamItem::PSET_TYPE:
      {
         ParamString *ps = ((PegParEntryString*)ent)->GetItem();
         string redir = ps->GetRedirectValue();

         if (redir.length())   sval = redir;
         else                  sval = ps->GetValue();

         ((PegParEntryString*)ent)->set(sval.c_str());
      }
      break;
      case ParamItem::INTEGER_TYPE:
      {
         ParamInteger *pi = ((PegParEntryInteger*)ent)->GetItem();
	 string redir = pi->GetRedirectValue();

         if (redir.length())   sval = redir;
	 else
	 {
            int val = pi->GetValue();
            if (val == INDEF_INT)
               sval = "INDEF";
            else
               sval = _int_to_string(val);
         }
         ((PegParEntryInteger*)ent)->set(sval.c_str());
      }
      break;
      case ParamItem::REAL_TYPE:
      {
         ParamReal *pr = ((PegParEntryReal*)ent)->GetItem();
	 string redir = pr->GetRedirectValue();

         if (redir.length())  sval = redir;
         else
         {
            double val = pr->GetValue();
	    if (val == INDEF_REAL)
	       sval = "INDEF";
            else
            {
               // restore value using specified precision
	       char temp[100];
	       ostrstream oss(temp, 100);
	       oss << setprecision(pr->GetPrecision());
	       oss << val << ends;
	       sval = temp;
            }
         }
         ((PegParEntryReal*)ent)->set(sval.c_str());
      }
      break;
      case ParamItem::BOOLEAN_TYPE:
      {
         ParamBoolean *pb = ((PegParEntryBoolean*)ent)->GetItem();
         string redir = pb->GetRedirectValue();

         if (redir.length())       sval = redir;
         else if (pb->GetValue())  sval = "yes";
	 else                      sval = "no";

         ((PegParEntryBoolean*)ent)->set(sval.c_str());
      }
      break;
      default:  // do nothing
      break; 
   }
   return 0; 
}


int PegFloatingMenu::browse_file_cb(GObject *object, gpointer user_data)
{
   PegParEntryString* ent = (PegParEntryString*)user_data;
   ParamItem *item;
   if (!ent) return 1;
   if (!(item = ent->GetItem())) return 2;

   GtkWidget *dialog = gtk_file_chooser_dialog_new("Browse Files",
                                         GTK_WINDOW(NULL),
                                         GTK_FILE_CHOOSER_ACTION_OPEN,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                         GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                         NULL);
   if (ent)
   {
      const char* predir = ent->get_prev_dir();
      if (predir && strlen(predir))
         gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), predir);
   }
   // add basic filter support
   GtkFileFilter *filt = gtk_file_filter_new ();
   gtk_file_filter_add_pattern (filt, "*");
   gtk_file_filter_set_name(filt,"All Files");
   gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);
   filt = gtk_file_filter_new ();
   gtk_file_filter_add_pattern (filt, "*.par");
   gtk_file_filter_set_name(filt,"Parameter Files (*.par)");
   gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);

   if (gtk_dialog_run(GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
   {
      // statically allocated memory- don't free
      gchar * filename = (char*)
         gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
      if (filename)
      {
         ent->set_text_field(filename);
	 if (ent)
         {
            string newpath = filename;
            unsigned int delim = newpath.find_last_of('/');

            if (_check_is_directory(newpath))
            {
               if (delim + 1 != newpath.length()) newpath.append("/");
	    }
	    else if (delim != string::npos)
	       newpath = newpath.substr(0, delim+1);
	       ent->set_prev_dir(newpath);
         }
         g_free (filename);
      }
   }
   gtk_widget_destroy (dialog);
   return 0;
}

int PegFloatingMenu::set_to_min_cb(GObject *object, gpointer user_data)
{
   PegParEntryString* ent = (PegParEntryString*)user_data;
   ParamItem *item;
   if (!ent) return 1;
   if (!(item = ent->GetItem())) return 2;

   string sval; 

   if (ent->getDispType() != PegParEntry::DEFAULT)  ent->toggleDispType();

   switch(item->GetType())
   {
      case ParamItem::STRING_TYPE:
      {
         ParamString *ps = ((PegParEntryString*)ent)->GetItem();
         sval.append(ps->GetMinValue());
         ((PegParEntryString*)ent)->set(sval.c_str());
      }
      break;
      case ParamItem::INTEGER_TYPE:
      {
         ParamInteger *pi = ((PegParEntryInteger*)ent)->GetItem();
	 char *tmpred = pi->GetRedirectMin();
         if (tmpred && tmpred[0] != '\0')
         {
	    sval = tmpred;
	 }
	 else
	 {
	    int tmpval = pi->GetMinValue();
	    if (tmpval == INDEF_INT)
               sval = "INDEF";
            else
               sval = _int_to_string(tmpval);
         }
	 ((PegParEntryInteger*)ent)->set(sval.c_str());
      }
      break;
      case ParamItem::REAL_TYPE:
      {
         ParamReal *pr = ((PegParEntryReal*)ent)->GetItem();
         char *tmpred = pr->GetRedirectMin();
         if (tmpred && tmpred[0] != '\0')
         {
            sval = tmpred;
	 }
	 else
	 {
	    double tmpval = pr->GetMinValue();
	    if (tmpval == INDEF_REAL)
	       sval = "INDEF";
	    else
	       sval.append(_double_to_string(tmpval));
	}
	((PegParEntryReal*)ent)->set(sval.c_str());
      }
      break;
      default:  // do nothing
      break;
   }
   return 0; 
}

int PegFloatingMenu::set_to_max_cb(GObject *object, gpointer user_data)
{
   PegParEntryString* ent = (PegParEntryString*)user_data;
   ParamItem *item;
   if (!ent) return 1;
   if (!(item = ent->GetItem())) return 2;

   string sval; 

   if (ent->getDispType() != PegParEntry::DEFAULT)  ent->toggleDispType();

   switch(item->GetType())
   {
      case ParamItem::STRING_TYPE:
      {
         ParamString *ps = ((PegParEntryString*)ent)->GetItem();
         sval.append(ps->GetMaxValue());
         ((PegParEntryString*)ent)->set(sval.c_str());
      }
      break;
      case ParamItem::INTEGER_TYPE:
      {
         ParamInteger *pi = ((PegParEntryInteger*)ent)->GetItem();
	 char *tmpred = pi->GetRedirectMax();
         if (tmpred && tmpred[0] != '\0')   sval = tmpred;
         else
         {
            int tmpval = pi->GetMaxValue();
            if (tmpval == INDEF_INT)  sval = "INDEF";
            else                      sval = _int_to_string(tmpval);
         }
         ((PegParEntryInteger*)ent)->set(sval.c_str());
      }
      break;
      case ParamItem::REAL_TYPE:
      {
         ParamReal *pr = ((PegParEntryReal*)ent)->GetItem();
         char *tmpred = pr->GetRedirectMax();
         if (tmpred && tmpred[0] != '\0')   sval = tmpred;
         else
         {
            double tmpval = pr->GetMaxValue();
            if (tmpval == INDEF_REAL)  sval = "INDEF";
            else                       sval = _double_to_string(tmpval);
         }
         ((PegParEntryReal*)ent)->set(sval.c_str());
      }
      break;
      default:  // do nothing 
      break;
   }
   return 0; 
}

int PegFloatingMenu::param_help_cb(GObject *object, gpointer user_data)
{
   PegParEntryString* ent = (PegParEntryString*)user_data;
   ParamItem *item;
   if (!ent) return 1;
   if (!(item = ent->GetItem())) return 2;

   string cmd = "cxcwrapper -x -- ahelp -b PARAMLIST -t ";
   int process = 0;
   string msg = item->GetName();
   if (msg.length())
   {
      string fname = theApp->GetFilename();
      if (fname.length())
      {
         cmd.append(msg); 
         cmd.append(" ");
	 cmd.append(_file_get_root_name(fname)); 
         cmd.append(" &");
         process = 1;
      }
   }
   if (process)
      system(cmd.c_str());
   else
      theApp->status_message(
         "Unable to obtain context specific help on parameter " + msg);
   return 0; 
}



