// 
//  Copyright (C) 2002,2003,2007,2008  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#include "PegApp.hh"
#include "PegParEntryString.hh"
#include "PegParData.hh"
#include "Parameter.hh"


string PegParEntryString::_prev_dir = "";

// --------------------------------------------------------
// Constructor
// --------------------------------------------------------
PegParEntryString::PegParEntryString(ParamString* item):
   PegParEntry((ParamItem*)item)
{
   _item = new ParamString(*item);
   string curval = _item->GetCurrent(); 
   _browse = NULL; 
   switch(item->GetType())
   {
      case ParamItem::FILENAME_TYPE:
      {
         _entry_type = PegParEntry::FILENAME; 
	 _browse = gtk_button_new_with_label("Browse");
	  g_signal_connect(_browse, "clicked", G_CALLBACK(BrowseButtonCB),
	     (void*)this);
      }
      break; 
      case ParamItem::PSET_TYPE:
         _entry_type = PegParEntry::PSET; 
      break; 
      default:
         _entry_type = PegParEntry::STRING; 
      break; 
   }

   _txt_w = gtk_entry_new();
   
   gtk_widget_add_events(_txt_w, GDK_FOCUS_CHANGE_MASK);

   // The 'changed' signal operate on a per key change- bad for _ind_txt since
   // it evaluates the indirection value as it goes, so use 'activate' and
   // 'focus-out' signals instead.  
   g_signal_connect(_txt_w, "activate", G_CALLBACK(TextEditCB), (void*)this); 
   g_signal_connect(_ind_txt,"activate", G_CALLBACK(IndirEditCB), (void*)this); 
   g_signal_connect(_txt_w, "focus-out-event", G_CALLBACK(FocusOutCB),
                   (void*)this);
   g_signal_connect(_ind_txt, "focus-out-event", G_CALLBACK(FocusOutCB),
                   (void*)this);

   if (_tip_grp)
   {
      string prompt = item->GetPrompt(); 
      gtk_tooltips_set_tip(_tip_grp, _txt_w, prompt.c_str(), NULL);
      gtk_tooltips_set_tip(_tip_grp, _ind_txt, prompt.c_str(), NULL);
   }

   char *rval = _item->GetRedirectCurrent(); 
   if (rval[0] != '\0') 
   {
      _disp_type = PegParEntry::INDIR;
   }
   else 
      _disp_type = PegParEntry::DEFAULT;

   int subcnt = _item->GetSubsetCount();
   if (subcnt)
   {
      int pos = -1; 
      char** tmplbls = _item->GetSubset();
      _enumer = gtk_combo_box_new_text();
      for (int rr = 0; rr < subcnt; rr++) 
      {
	 gtk_combo_box_append_text(GTK_COMBO_BOX(_enumer),tmplbls[rr]); 
         if (curval == tmplbls[rr])  pos = rr;  
      }

      _disp_type = PegParEntry::ENUM; 
      if (pos == -1)
      {
         string msg = "The value \"" + curval + "\" for enumerated string " +
            _name + " is invalid- defaulting to \"";
         msg.append(tmplbls[0]);
         msg.append("\"");
         theApp->status_message(msg);
         pos = 0; 
         curval = tmplbls[0]; 
      }
      gtk_combo_box_set_active(GTK_COMBO_BOX(_enumer), pos);
   }
   else 
   {
      string tmp = "\0";
      if (_disp_type == PegParEntry::INDIR)
      {
         tmp.append(_item->GetRedirectCurrent());
      }
      else
      {
         tmp.append(curval);
      }
      set(tmp.c_str());
   }

   initEntry(); 

   if (_disp_type == PegParEntry::ENUM && _enumer) 
      gtk_box_pack_start(GTK_BOX(_entry), _enumer, false, false, 0);
   else if (_txt_w) 
      gtk_box_pack_start(GTK_BOX(_entry), _txt_w, true, true, 0);

   if (_browse)
      gtk_box_pack_start(GTK_BOX(_entry), _browse, true, true, 0);
}


// --------------------------------------------------------
// Destructor
// --------------------------------------------------------
PegParEntryString::~PegParEntryString()
{
   if (_item)
   {
      delete _item;
      _item = NULL; 
   }
   if (_enumer)
   {
      gtk_widget_destroy(_enumer);
      _enumer = NULL;
   }
   if (_browse)
   {
      gtk_widget_destroy(_browse);
      _browse = NULL; 
   }
}


// --------------------------------------------------------
// Set value of entry
// --------------------------------------------------------
void PegParEntryString::set(const char* val)
{
   if (val)
   {
      // make sure that indirections are displayed as indirections
      // and entries as entries
      if (((val[0] == ')') && (_disp_type != PegParEntry::INDIR)) ||
          ((val[0] != ')') && (_disp_type == PegParEntry::INDIR)))
         toggleDispType();

      if (_disp_type == PegParEntry::DEFAULT)
      {
         string cur = gtk_entry_get_text(GTK_ENTRY(_txt_w));
	 if (cur != val)   gtk_entry_set_text(GTK_ENTRY(_txt_w), (char*)val);
      }
      else if (_disp_type == PegParEntry::INDIR)
      {
         string cur = gtk_entry_get_text(GTK_ENTRY(_ind_txt));
         if (cur != val)  gtk_entry_set_text(GTK_ENTRY(_ind_txt), (char*)val);
         if (thePData && _item)
         {
            Parameter *par = NULL;

            if ((par = thePData->GetParameter()) != NULL)
            {
               string eval = par->EvalIndir((char*)_name.c_str(), (char*)val);
	       gtk_entry_set_text(GTK_ENTRY(_ind_val), eval.c_str());
            }
         }
      }
      else
      {
         string sval = val;

/**WLM
         char* sel = gtk_combo_box_get_active_text(GTK_COMBO_BOX(_enumer));

         int pos = -1;
         int subcnt = _item->GetSubsetCount();
         char** tmplbls = _item->GetSubset();
         string getv = gtk_entry_get_text(GTK_ENTRY(_enumer));

         for (int rr = subcnt; rr--; )
            if (strcmp(tmplbls[rr], val) == 0) 
            { 
               pos = rr; 
               break; 
            }
         if ((pos != -1) && (getv.StrcmpLoose(val) != 0))
            _enumer->Set(sval, pos);
      **/
      }

      if (thePData)
      {
         Parameter *par = thePData->GetParameter();
         int error = par->SetValueAtPosition(_item->GetPosition(), val);

	 if (error != 1)
	    set_error_color();
	 else 
	    clear_error_color(); 
      }
   }
}

// --------------------------------------------------------
// Get the value of the entry
// --------------------------------------------------------
char* PegParEntryString::get()
{
   static string val;

   switch(_disp_type)
   {
      case PegParEntry::DEFAULT:
         val = gtk_entry_get_text(GTK_ENTRY(_txt_w));
      break; 
      case PegParEntry::INDIR:
         val = gtk_entry_get_text(GTK_ENTRY(_ind_txt));
      break; 
      case PegParEntry::ENUM:
      {
         char* sel = gtk_combo_box_get_active_text(GTK_COMBO_BOX(_enumer));
         if (sel)   val = sel;
	    else    val = ""; 
      }
      break; 
      default:
         val = _item->GetValue();
      break; 
   }
   return (char*)val.c_str();
}

// callback to handle leaving focus event from text|indirection field
int PegParEntryString::FocusOutCB(GObject *object, GdkEvent *event, 
   gpointer user_data)
{
   PegParEntryString *obj = (PegParEntryString*)user_data;

   GtkWidget* ww = GTK_WIDGET(object);
   if (obj)
      if (ww == obj->GetIndirField())         return obj->update_indir();
         else if (ww == obj->GetTextField())  return obj->update_text(); 
   return 0;
}


// callback to handle activate signal ('return') from text field 
int PegParEntryString::TextEditCB(GObject *object, gpointer user_data)
{
   PegParEntryString *obj = (PegParEntryString*)user_data;
   if (obj) return obj->update_text();
      else return 0; 
}


// callback to handle activate signal ('return') from indirection field 
int PegParEntryString::IndirEditCB(GObject *object, gpointer user_data)
{
   PegParEntryString *obj = (PegParEntryString*)user_data;
   if (obj) return obj->update_indir();
      else return 0; 
}


// --------------------------------------------------------
// Callback to handle browse button clicks
// --------------------------------------------------------
int PegParEntryString::BrowseButtonCB(GObject *object, gpointer user_data)
{
   PegParEntryString *obj = (PegParEntryString*)user_data;
   GtkWidget *dialog = gtk_file_chooser_dialog_new("Browse Files",
                                         GTK_WINDOW(NULL),
                                         GTK_FILE_CHOOSER_ACTION_OPEN,
                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                         GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
                                         NULL);
   if (obj)
   {
      const char* predir = obj->get_prev_dir();
      if (predir && strlen(predir))
         gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), predir);
   }
   // add basic filter support
   GtkFileFilter *filt = gtk_file_filter_new ();
   gtk_file_filter_add_pattern (filt, "*");
   gtk_file_filter_set_name(filt,"All Files");
   gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);
   filt = gtk_file_filter_new ();
   gtk_file_filter_add_pattern (filt, "*.par");
   gtk_file_filter_set_name(filt,"Parameter Files (*.par)");
   gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);
   filt = gtk_file_filter_new ();
   gtk_file_filter_add_pattern (filt, "*.fits");
   gtk_file_filter_set_name(filt,"Fits Files (*.fits)");
   gtk_file_chooser_add_filter(GTK_FILE_CHOOSER (dialog), filt);

   if (gtk_dialog_run(GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
   {
      gchar * filename = 
         (char*)gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
      if (filename)   
      {
         obj->set_text_field(filename);
         if (obj)
	 {
	    string newpath = filename;
            unsigned int delim = newpath.find_last_of('/');

            if (_check_is_directory(newpath))
            {
               if (delim + 1 != newpath.length()) newpath.append("/");
            }
            else if (delim != string::npos)
               newpath = newpath.substr(0, delim+1);
            obj->set_prev_dir(newpath);
	 }
         g_free (filename);
      }
   }
   gtk_widget_destroy (dialog);
   return 0; 
}


// --------------------------------------------------------
// display the appropriate widgets for indirection/default
// --------------------------------------------------------
void PegParEntryString::manage()
{
   string tmp = get();

   gtk_widget_show(_lbl_w); 
   switch(_disp_type)
   {
      case PegParEntry::INDIR:
	 gtk_widget_show_all(_ind_box);
	 gtk_widget_hide_all(_entry_box); 

         // if the current data is not an indirection, clear buffer
         if (tmp.length() && tmp.find(')') == 0)
         {
            set(tmp.c_str());
         }
         else
         {
            gtk_entry_set_text(GTK_ENTRY(_ind_txt), "\0");
            gtk_entry_set_text(GTK_ENTRY(_ind_val), "\0");
         }
      break;
      case PegParEntry::ENUM:
	 gtk_widget_show_all(_entry_box);
	 gtk_widget_hide_all(_ind_box); 
         if (tmp.length())  set(tmp.c_str());

      break;
      case PegParEntry::DEFAULT:  // fallthrough intended 
      default:
	 gtk_widget_show_all(_entry_box);
	 gtk_widget_hide_all(_ind_box); 

         // if current text is an indirection, clear buffer
         if (tmp.length() && tmp.find(')') != 0)
         {
            set(tmp.c_str());
         }
         else
         {
            gtk_entry_set_text(GTK_ENTRY(_txt_w), "\0");
         }
      break; 
   }
}

int PegParEntryString::update_indir()
{
   Parameter *par = NULL;
   string val = gtk_entry_get_text(GTK_ENTRY(_ind_txt));
 
   if (val == "\0")
   {
      clear_error_color(); 
   }
   else if (val.find(')') != 0)
   {
      if (val.find(')') != string::npos)
      {
         string msg = "Indirection for parameter " + _name +
	    " must begin with )  - try removing any white space characters";
	 set_error_color(); 
         theApp->status_message(msg);
      }
   }
   else if ((par = thePData->GetParameter()) != NULL)
   {
      if (par->SetValueAtPosition(_item->GetPosition(), val) != 1) 
      {
	 set_error_color(); 
      }
      else
      {
	 clear_error_color(); 
         string evl = par->EvalIndir((char*)_name.c_str(), (char*)val.c_str());
         if (evl.length())
            gtk_entry_set_text(GTK_ENTRY(_ind_val), evl.c_str());
         else
            gtk_entry_set_text(GTK_ENTRY(_ind_val), "\0");
      }
   }
   set(val.c_str());
   return 0; 
}


int PegParEntryString::update_text()
{
   Parameter *par = NULL; 

   if ((par = thePData->GetParameter()) != NULL)
   {
      string val = gtk_entry_get_text(GTK_ENTRY(_txt_w));

      if (par->SetValueAtPosition(_item->GetPosition(), val.c_str()) != 1)
      {
         string msg = "Invalid value entered: " + val;
	 set_error_color(); 
         switch(_item->GetRangeCheck())
         {
            case SUBSET_CHECK:
               msg.append(" - value must be: ");
               msg.append(_item->GetMinValue());
            break;  
            case MIN_MAX_CHECK:
               msg.append(" - value must be between : ");
               msg.append(_item->GetMinValue());
               msg.append(" and "); 
               msg.append(_item->GetMaxValue());
            break;  
            case MIN_ONLY_CHECK:
               msg.append(" - value must be greater than : ");
               msg.append(_item->GetMinValue());
            break;  
            case MAX_ONLY_CHECK:
               msg.append(" - value must be less than : ");
               msg.append(_item->GetMaxValue());
            break;  
            default:
               // no range to output
            break;  
         }
         theApp->status_message(msg);
      }
      else 
	 clear_error_color(); 
       
      set(val.c_str());
   }
   return 0; 
}


void PegParEntryString::set_text_field(char* val)
{
   if (val)  gtk_entry_set_text(GTK_ENTRY(_txt_w), (char*)val);
}


void PegParEntryString::set_error_color()
{
   if (_disp_type == PegParEntry::DEFAULT)
      gtk_widget_modify_bg(_txt_w, GTK_STATE_NORMAL, &_errcolor);
   else
      gtk_widget_modify_bg(_ind_txt, GTK_STATE_NORMAL, &_errcolor);
}


void PegParEntryString::clear_error_color()
{
   if (_disp_type == PegParEntry::DEFAULT)
      gtk_widget_modify_bg(_txt_w, GTK_STATE_NORMAL, NULL);
   else
      gtk_widget_modify_bg(_ind_txt, GTK_STATE_NORMAL, NULL);
}



